@@ -67,17 +67,17 @@ import org.apache.commons.compress.utils.IOUtils;
 public class CpioArchiveInputStream extends ArchiveInputStream implements
         CpioConstants {
 
-    private boolean closed = false;
+    private boolean closed;
 
     private CpioArchiveEntry entry;
 
-    private long entryBytesRead = 0;
+    private long entryBytesRead;
 
-    private boolean entryEOF = false;
+    private boolean entryEOF;
 
-    private final byte tmpbuf[] = new byte[4096];
+    private final byte[] tmpbuf = new byte[4096];
 
-    private long crc = 0;
+    private long crc;
 
     private final InputStream in;
 
@@ -89,7 +89,7 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
     private final int blockSize;
 
     /**
-     * The encoding to use for filenames and labels.
+     * The encoding to use for file names and labels.
      */
     private final ZipEncoding zipEncoding;
 
@@ -148,10 +148,14 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
      * @param encoding
      *            The encoding of file names to expect - use null for
      *            the platform's default.
+     * @throws IllegalArgumentException if <code>blockSize</code> is not bigger than 0
      * @since 1.6
      */
     public CpioArchiveInputStream(final InputStream in, final int blockSize, final String encoding) {
         this.in = in;
+        if (blockSize <= 0) {
+            throw new IllegalArgumentException("blockSize must be bigger than 0");
+        }
         this.blockSize = blockSize;
         this.encoding = encoding;
         this.zipEncoding = ZipEncodingHelper.getZipEncoding(encoding);
@@ -203,7 +207,7 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
     private void closeEntry() throws IOException {
         // the skip implementation of this class will not skip more
         // than Integer.MAX_VALUE bytes
-        while (skip((long) Integer.MAX_VALUE) == Integer.MAX_VALUE) { // NOPMD
+        while (skip((long) Integer.MAX_VALUE) == Integer.MAX_VALUE) { // NOPMD NOSONAR
             // do nothing
         }
     }
@@ -257,7 +261,7 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
                     this.entry = readOldAsciiEntry();
                     break;
                 default:
-                    throw new IOException("Unknown magic [" + magicString + "]. Occured at byte: " + getBytesRead());
+                    throw new IOException("Unknown magic [" + magicString + "]. Occurred at byte: " + getBytesRead());
             }
         }
 
@@ -302,7 +306,8 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
         ensureOpen();
         if (off < 0 || len < 0 || off > b.length - len) {
             throw new IndexOutOfBoundsException();
-        } else if (len == 0) {
+        }
+        if (len == 0) {
             return 0;
         }
 
@@ -314,7 +319,7 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
             this.entryEOF = true;
             if (this.entry.getFormat() == FORMAT_NEW_CRC
                 && this.crc != this.entry.getChksum()) {
-                throw new IOException("CRC Error. Occured at byte: "
+                throw new IOException("CRC Error. Occurred at byte: "
                                       + getBytesRead());
             }
             return -1; // EOF for this entry
@@ -349,23 +354,31 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
         return count;
     }
 
+    private final byte[] readRange(final int len)
+            throws IOException {
+        final byte[] b = IOUtils.readRange(in, len);
+        count(b.length);
+        if (b.length < len) {
+            throw new EOFException();
+        }
+        return b;
+    }
+
     private long readBinaryLong(final int length, final boolean swapHalfWord)
             throws IOException {
-        final byte tmp[] = new byte[length];
-        readFully(tmp, 0, tmp.length);
+        final byte[] tmp = readRange(length);
         return CpioUtil.byteArray2long(tmp, swapHalfWord);
     }
 
     private long readAsciiLong(final int length, final int radix)
             throws IOException {
-        final byte tmpBuffer[] = new byte[length];
-        readFully(tmpBuffer, 0, tmpBuffer.length);
+        final byte[] tmpBuffer = readRange(length);
         return Long.parseLong(ArchiveUtils.toAsciiString(tmpBuffer), radix);
     }
 
     private CpioArchiveEntry readNewEntry(final boolean hasCrc)
             throws IOException {
-        CpioArchiveEntry ret;
+        final CpioArchiveEntry ret;
         if (hasCrc) {
             ret = new CpioArchiveEntry(FORMAT_NEW_CRC);
         } else {
@@ -374,7 +387,7 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
 
         ret.setInode(readAsciiLong(8, 16));
         final long mode = readAsciiLong(8, 16);
-        if (CpioUtil.fileType(mode) != 0){ // mode is initialised to 0
+        if (CpioUtil.fileType(mode) != 0){ // mode is initialized to 0
             ret.setMode(mode);
         }
         ret.setUID(readAsciiLong(8, 16));
@@ -382,18 +395,24 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
         ret.setNumberOfLinks(readAsciiLong(8, 16));
         ret.setTime(readAsciiLong(8, 16));
         ret.setSize(readAsciiLong(8, 16));
+        if (ret.getSize() < 0) {
+            throw new IOException("Found illegal entry with negative length");
+        }
         ret.setDeviceMaj(readAsciiLong(8, 16));
         ret.setDeviceMin(readAsciiLong(8, 16));
         ret.setRemoteDeviceMaj(readAsciiLong(8, 16));
         ret.setRemoteDeviceMin(readAsciiLong(8, 16));
         final long namesize = readAsciiLong(8, 16);
+        if (namesize < 0) {
+            throw new IOException("Found illegal entry with negative name length");
+        }
         ret.setChksum(readAsciiLong(8, 16));
         final String name = readCString((int) namesize);
         ret.setName(name);
         if (CpioUtil.fileType(mode) == 0 && !name.equals(CPIO_TRAILER)){
             throw new IOException("Mode 0 only allowed in the trailer. Found entry name: "
                                   + ArchiveUtils.sanitize(name)
-                                  + " Occured at byte: " + getBytesRead());
+                                  + " Occurred at byte: " + getBytesRead());
         }
         skip(ret.getHeaderPadCount(namesize - 1));
 
@@ -415,13 +434,19 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
         ret.setRemoteDevice(readAsciiLong(6, 8));
         ret.setTime(readAsciiLong(11, 8));
         final long namesize = readAsciiLong(6, 8);
+        if (namesize < 0) {
+            throw new IOException("Found illegal entry with negative name length");
+        }
         ret.setSize(readAsciiLong(11, 8));
+        if (ret.getSize() < 0) {
+            throw new IOException("Found illegal entry with negative length");
+        }
         final String name = readCString((int) namesize);
         ret.setName(name);
         if (CpioUtil.fileType(mode) == 0 && !name.equals(CPIO_TRAILER)){
             throw new IOException("Mode 0 only allowed in the trailer. Found entry: "
                                   + ArchiveUtils.sanitize(name)
-                                  + " Occured at byte: " + getBytesRead());
+                                  + " Occurred at byte: " + getBytesRead());
         }
 
         return ret;
@@ -443,13 +468,19 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
         ret.setRemoteDevice(readBinaryLong(2, swapHalfWord));
         ret.setTime(readBinaryLong(4, swapHalfWord));
         final long namesize = readBinaryLong(2, swapHalfWord);
+        if (namesize < 0) {
+            throw new IOException("Found illegal entry with negative name length");
+        }
         ret.setSize(readBinaryLong(4, swapHalfWord));
+        if (ret.getSize() < 0) {
+            throw new IOException("Found illegal entry with negative length");
+        }
         final String name = readCString((int) namesize);
         ret.setName(name);
         if (CpioUtil.fileType(mode) == 0 && !name.equals(CPIO_TRAILER)){
             throw new IOException("Mode 0 only allowed in the trailer. Found entry: "
                                   + ArchiveUtils.sanitize(name)
-                                  + "Occured at byte: " + getBytesRead());
+                                  + "Occurred at byte: " + getBytesRead());
         }
         skip(ret.getHeaderPadCount(namesize - 1));
 
@@ -458,9 +489,10 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
 
     private String readCString(final int length) throws IOException {
         // don't include trailing NUL in file name to decode
-        final byte tmpBuffer[] = new byte[length - 1];
-        readFully(tmpBuffer, 0, tmpBuffer.length);
-        this.in.read();
+        final byte[] tmpBuffer = readRange(length - 1);
+        if (this.in.read() == -1) {
+            throw new EOFException();
+        }
         return zipEncoding.decode(tmpBuffer);
     }
 
@@ -478,7 +510,7 @@ public class CpioArchiveInputStream extends ArchiveInputStream implements
     @Override
     public long skip(final long n) throws IOException {
         if (n < 0) {
-            throw new IllegalArgumentException("negative skip length");
+            throw new IllegalArgumentException("Negative skip length");
         }
         ensureOpen();
         final int max = (int) Math.min(n, Integer.MAX_VALUE);
