@@ -157,9 +157,9 @@ import org.apache.commons.compress.archivers.ArchiveEntry;
  * </pre>
  * <p>
  * It is important to note that the header DOES NOT have the name of the
- * file. It can't since hard links mean that you may have multiple filenames
+ * file. It can't since hard links mean that you may have multiple file names
  * for a single physical file. You must read the contents of the directory
- * entries to learn the mapping(s) from filename to inode.
+ * entries to learn the mapping(s) from file name to inode.
  * </p>
  *
  * <p>
@@ -215,11 +215,11 @@ public class DumpArchiveEntry implements ArchiveEntry {
     }
 
     /**
-     * Constructor taking only filename.
+     * Constructor taking only file name.
      * @param name pathname
-     * @param simpleName actual filename.
+     * @param simpleName actual file name.
      */
-    public DumpArchiveEntry(String name, String simpleName) {
+    public DumpArchiveEntry(final String name, final String simpleName) {
         setName(name);
         this.simpleName = simpleName;
     }
@@ -227,13 +227,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
     /**
      * Constructor taking name, inode and type.
      *
-     * @param name
-     * @param simpleName
-     * @param ino
-     * @param type
+     * @param name the name
+     * @param simpleName the simple name
+     * @param ino the ino
+     * @param type the type
      */
-    protected DumpArchiveEntry(String name, String simpleName, int ino,
-                               TYPE type) {
+    protected DumpArchiveEntry(final String name, final String simpleName, final int ino,
+                               final TYPE type) {
         setType(type);
         setName(name);
         this.simpleName = simpleName;
@@ -241,12 +241,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
         this.offset = 0;
     }
 
-    /**
-     * Constructor taking tape buffer.
-     * @param buffer
-     * @param offset
-     */
-
     /**
      * Returns the path of the entry.
      * @return the path of the entry.
@@ -257,13 +251,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Sets the path of the entry.
+     * @param simpleName the simple name
      */
-    protected void setSimpleName(String simpleName) {
+    protected void setSimpleName(final String simpleName) {
         this.simpleName = simpleName;
     }
 
     /**
      * Returns the ino of the entry.
+     * @return the ino
      */
     public int getIno() {
         return header.getIno();
@@ -271,6 +267,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Return the number of hard links to the entry.
+     * @return the number of hard links
      */
     public int getNlink() {
         return nlink;
@@ -278,13 +275,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the number of hard links.
+     * @param nlink the number of hard links
      */
-    public void setNlink(int nlink) {
+    public void setNlink(final int nlink) {
         this.nlink = nlink;
     }
 
     /**
      * Get file creation time.
+     * @return the creation time
      */
     public Date getCreationTime() {
         return new Date(ctime);
@@ -292,13 +291,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the file creation time.
+     * @param ctime the creation time
      */
-    public void setCreationTime(Date ctime) {
+    public void setCreationTime(final Date ctime) {
         this.ctime = ctime.getTime();
     }
 
     /**
      * Return the generation of the file.
+     * @return the generation
      */
     public int getGeneration() {
         return generation;
@@ -306,13 +307,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the generation of the file.
+     * @param generation the generation
      */
-    public void setGeneration(int generation) {
+    public void setGeneration(final int generation) {
         this.generation = generation;
     }
 
     /**
      * Has this file been deleted? (On valid on incremental dumps.)
+     * @return whether the file has been deleted
      */
     public boolean isDeleted() {
         return isDeleted;
@@ -320,13 +323,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set whether this file has been deleted.
+     * @param isDeleted whether the file has been deleted
      */
-    public void setDeleted(boolean isDeleted) {
+    public void setDeleted(final boolean isDeleted) {
         this.isDeleted = isDeleted;
     }
 
     /**
      * Return the offset within the archive
+     * @return the offset
      */
     public long getOffset() {
         return offset;
@@ -334,13 +339,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the offset within the archive.
+     * @param offset the offset
      */
-    public void setOffset(long offset) {
+    public void setOffset(final long offset) {
         this.offset = offset;
     }
 
     /**
      * Return the tape volume where this file is located.
+     * @return the volume
      */
     public int getVolume() {
         return volume;
@@ -348,13 +355,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the tape volume.
+     * @param volume the volume
      */
-    public void setVolume(int volume) {
+    public void setVolume(final int volume) {
         this.volume = volume;
     }
 
     /**
      * Return the type of the tape segment header.
+     * @return the segment header
      */
     public DumpArchiveConstants.SEGMENT_TYPE getHeaderType() {
         return header.getType();
@@ -362,6 +371,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Return the number of records in this segment.
+     * @return the number of records
      */
     public int getHeaderCount() {
         return header.getCount();
@@ -369,6 +379,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Return the number of sparse records in this segment.
+     * @return the number of sparse records
      */
     public int getHeaderHoles() {
         return header.getHoles();
@@ -376,33 +387,30 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a sparse record?
+     * @param idx index of the record to check
+     * @return whether this is a sparse record
      */
-    public boolean isSparseRecord(int idx) {
+    public boolean isSparseRecord(final int idx) {
         return (header.getCdata(idx) & 0x01) == 0;
     }
 
-    /**
-     * @see java.lang.Object#hashCode()
-     */
     @Override
     public int hashCode() {
         return ino;
     }
 
-    /**
-     * @see java.lang.Object#equals(Object o)
-     */
     @Override
-    public boolean equals(Object o) {
+    public boolean equals(final Object o) {
         if (o == this) {
             return true;
-        } else if (o == null || !o.getClass().equals(getClass())) {
+        }
+        if (o == null || !o.getClass().equals(getClass())) {
             return false;
         }
 
-        DumpArchiveEntry rhs = (DumpArchiveEntry) o;
+        final DumpArchiveEntry rhs = (DumpArchiveEntry) o;
 
-        if ((header == null) || (rhs.header == null)) {
+        if (rhs.header == null) {
             return false;
         }
 
@@ -410,17 +418,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
             return false;
         }
 
-        if ((summary == null && rhs.summary != null)
-            || (summary != null && !summary.equals(rhs.summary))) {
+        // summary is always null right now, but this may change some day
+        if ((summary == null && rhs.summary != null) // NOSONAR
+                || (summary != null && !summary.equals(rhs.summary))) { // NOSONAR
             return false;
         }
 
         return true;
     }
 
-    /**
-     * @see java.lang.Object#toString()
-     */
     @Override
     public String toString() {
         return getName();
@@ -430,12 +436,11 @@ public class DumpArchiveEntry implements ArchiveEntry {
      * Populate the dump archive entry and tape segment header with
      * the contents of the buffer.
      *
-     * @param buffer
-     * @throws Exception
+     * @param buffer buffer to read content from
      */
-    static DumpArchiveEntry parse(byte[] buffer) {
-        DumpArchiveEntry entry = new DumpArchiveEntry();
-        TapeSegmentHeader header = entry.header;
+    static DumpArchiveEntry parse(final byte[] buffer) {
+        final DumpArchiveEntry entry = new DumpArchiveEntry();
+        final TapeSegmentHeader header = entry.header;
 
         header.type = DumpArchiveConstants.SEGMENT_TYPE.find(DumpArchiveUtil.convert32(
                     buffer, 0));
@@ -449,7 +454,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
         //header.magic = DumpArchiveUtil.convert32(buffer, 24);
         //header.checksum = DumpArchiveUtil.convert32(buffer, 28);
-        int m = DumpArchiveUtil.convert16(buffer, 32);
+        final int m = DumpArchiveUtil.convert16(buffer, 32);
 
         // determine the type of the file.
         entry.setType(TYPE.find((m >> 12) & 0x0F));
@@ -500,7 +505,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
     /**
      * Update entry with information from next tape segment header.
      */
-    void update(byte[] buffer) {
+    void update(final byte[] buffer) {
         header.volume = DumpArchiveUtil.convert32(buffer, 16);
         header.count = DumpArchiveUtil.convert32(buffer, 160);
 
@@ -539,7 +544,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
             return ino;
         }
 
-        void setIno(int ino) {
+        void setIno(final int ino) {
             this.ino = ino;
         }
 
@@ -551,15 +556,19 @@ public class DumpArchiveEntry implements ArchiveEntry {
             return holes;
         }
 
-        public int getCdata(int idx) {
+        public int getCdata(final int idx) {
             return cdata[idx];
         }
     }
 
     /**
      * Returns the name of the entry.
+     *
+     * <p>This method returns the raw name as it is stored inside of the archive.</p>
+     *
      * @return the name of the entry.
      */
+    @Override
     public String getName() {
         return name;
     }
@@ -574,6 +583,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Sets the name of the entry.
+     * @param name the name
      */
     public final void setName(String name) {
         this.originalName = name;
@@ -588,19 +598,27 @@ public class DumpArchiveEntry implements ArchiveEntry {
         this.name = name;
     }
 
+    /**
+     * The last modified date.
+     * @return the last modified date
+     */
+    @Override
     public Date getLastModifiedDate() {
         return new Date(mtime);
     }
 
     /**
      * Is this a directory?
+     * @return whether this is a directory
      */
+    @Override
     public boolean isDirectory() {
         return type == TYPE.DIRECTORY;
     }
 
     /**
      * Is this a regular file?
+     * @return whether this is a regular file
      */
     public boolean isFile() {
         return type == TYPE.FILE;
@@ -608,6 +626,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a network device?
+     * @return whether this is a socket
      */
     public boolean isSocket() {
         return type == TYPE.SOCKET;
@@ -615,6 +634,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a character device?
+     * @return whether this is a character device
      */
     public boolean isChrDev() {
         return type == TYPE.CHRDEV;
@@ -622,6 +642,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a block device?
+     * @return whether this is a block device
      */
     public boolean isBlkDev() {
         return type == TYPE.BLKDEV;
@@ -629,6 +650,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a fifo/pipe?
+     * @return whether this is a fifo
      */
     public boolean isFifo() {
         return type == TYPE.FIFO;
@@ -636,6 +658,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Get the type of the entry.
+     * @return the type
      */
     public TYPE getType() {
         return type;
@@ -643,13 +666,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the type of the entry.
+     * @param type the type
      */
-    public void setType(TYPE type) {
+    public void setType(final TYPE type) {
         this.type = type;
     }
 
     /**
      * Return the access permissions on the entry.
+     * @return the access permissions
      */
     public int getMode() {
         return mode;
@@ -657,14 +682,16 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the access permissions on the entry.
+     * @param mode the access permissions
      */
-    public void setMode(int mode) {
+    public void setMode(final int mode) {
         this.mode = mode & 07777;
         this.permissions = PERMISSION.find(mode);
     }
 
     /**
      * Returns the permissions on the entry.
+     * @return the permissions
      */
     public Set<PERMISSION> getPermissions() {
         return permissions;
@@ -672,7 +699,9 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Returns the size of the entry.
+     * @return the size
      */
+    @Override
     public long getSize() {
         return isDirectory() ? SIZE_UNKNOWN : size;
     }
@@ -686,20 +715,23 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the size of the entry.
+     * @param size the size
      */
-    public void setSize(long size) {
+    public void setSize(final long size) {
         this.size = size;
     }
 
     /**
      * Set the time the file was last modified.
+     * @param mtime the last modified time
      */
-    public void setLastModifiedDate(Date mtime) {
+    public void setLastModifiedDate(final Date mtime) {
         this.mtime = mtime.getTime();
     }
 
     /**
      * Returns the time the file was last accessed.
+     * @return the access time
      */
     public Date getAccessTime() {
         return new Date(atime);
@@ -707,13 +739,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the time the file was last accessed.
+     * @param atime the access time
      */
-    public void setAccessTime(Date atime) {
+    public void setAccessTime(final Date atime) {
         this.atime = atime.getTime();
     }
 
     /**
      * Return the user id.
+     * @return the user id
      */
     public int getUserId() {
         return uid;
@@ -721,13 +755,15 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the user id.
+     * @param uid the user id
      */
-    public void setUserId(int uid) {
+    public void setUserId(final int uid) {
         this.uid = uid;
     }
 
     /**
      * Return the group id
+     * @return the group id
      */
     public int getGroupId() {
         return gid;
@@ -735,8 +771,9 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the group id.
+     * @param gid the group id
      */
-    public void setGroupId(int gid) {
+    public void setGroupId(final int gid) {
         this.gid = gid;
     }
 
@@ -751,16 +788,16 @@ public class DumpArchiveEntry implements ArchiveEntry {
         FIFO(1),
         UNKNOWN(15);
 
-        private int code;
+        private final int code;
 
-        private TYPE(int code) {
+        TYPE(final int code) {
             this.code = code;
         }
 
-        public static TYPE find(int code) {
+        public static TYPE find(final int code) {
             TYPE type = UNKNOWN;
 
-            for (TYPE t : TYPE.values()) {
+            for (final TYPE t : TYPE.values()) {
                 if (code == t.code) {
                     type = t;
                 }
@@ -784,16 +821,16 @@ public class DumpArchiveEntry implements ArchiveEntry {
         WORLD_WRITE(00002),
         WORLD_EXEC(00001);
 
-        private int code;
+        private final int code;
 
-        private PERMISSION(int code) {
+        PERMISSION(final int code) {
             this.code = code;
         }
 
-        public static Set<PERMISSION> find(int code) {
-            Set<PERMISSION> set = new HashSet<PERMISSION>();
+        public static Set<PERMISSION> find(final int code) {
+            final Set<PERMISSION> set = new HashSet<>();
 
-            for (PERMISSION p : PERMISSION.values()) {
+            for (final PERMISSION p : PERMISSION.values()) {
                 if ((code & p.code) == p.code) {
                     set.add(p);
                 }
