@@ -22,13 +22,13 @@ import org.apache.commons.compress.archivers.ArchiveException;
 import org.apache.commons.compress.archivers.ArchiveInputStream;
 import org.apache.commons.compress.archivers.zip.ZipEncoding;
 import org.apache.commons.compress.archivers.zip.ZipEncodingHelper;
+import org.apache.commons.compress.utils.IOUtils;
 
 import java.io.EOFException;
 import java.io.IOException;
 import java.io.InputStream;
 
 import java.util.Arrays;
-import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.PriorityQueue;
@@ -49,7 +49,7 @@ import java.util.Stack;
  * @NotThreadSafe
  */
 public class DumpArchiveInputStream extends ArchiveInputStream {
-    private DumpArchiveSummary summary;
+    private final DumpArchiveSummary summary;
     private DumpArchiveEntry active;
     private boolean isClosed;
     private boolean hasHitEOF;
@@ -69,10 +69,10 @@ public class DumpArchiveInputStream extends ArchiveInputStream {
     private final Map<Integer, DumpArchiveEntry> pending = new HashMap<>();
 
     // queue of (directory) entries where we now have the full path.
-    private Queue<DumpArchiveEntry> queue;
+    private final Queue<DumpArchiveEntry> queue;
 
     /**
-     * The encoding to use for filenames and labels.
+     * The encoding to use for file names and labels.
      */
     private final ZipEncoding zipEncoding;
 
@@ -137,15 +137,12 @@ public class DumpArchiveInputStream extends ArchiveInputStream {
         // use priority based on queue to ensure parent directories are
         // released first.
         queue = new PriorityQueue<>(10,
-                new Comparator<DumpArchiveEntry>() {
-                    @Override
-                    public int compare(final DumpArchiveEntry p, final DumpArchiveEntry q) {
-                        if (p.getOriginalName() == null || q.getOriginalName() == null) {
-                            return Integer.MAX_VALUE;
-                        }
-
-                        return p.getOriginalName().compareTo(q.getOriginalName());
+                (p, q) -> {
+                    if (p.getOriginalName() == null || q.getOriginalName() == null) {
+                        return Integer.MAX_VALUE;
                     }
+
+                    return p.getOriginalName().compareTo(q.getOriginalName());
                 });
     }
 
@@ -341,10 +338,11 @@ public class DumpArchiveInputStream extends ArchiveInputStream {
             final int datalen = DumpArchiveConstants.TP_SIZE * entry.getHeaderCount();
 
             if (blockBuffer.length < datalen) {
-                blockBuffer = new byte[datalen];
-            }
-
-            if (raw.read(blockBuffer, 0, datalen) != datalen) {
+                blockBuffer = IOUtils.readRange(raw, datalen);
+                if (blockBuffer.length != datalen) {
+                    throw new EOFException();
+                }
+            } else if (raw.read(blockBuffer, 0, datalen) != datalen) {
                 throw new EOFException();
             }
 
@@ -413,7 +411,7 @@ public class DumpArchiveInputStream extends ArchiveInputStream {
      * @return  full path for specified archive entry, or null if there's a gap.
      */
     private String getPath(final DumpArchiveEntry entry) {
-        // build the stack of elements. It's possible that we're 
+        // build the stack of elements. It's possible that we're
         // still missing an intermediate value and if so we
         final Stack<String> elements = new Stack<>();
         Dirent dirent = null;
@@ -465,6 +463,9 @@ public class DumpArchiveInputStream extends ArchiveInputStream {
      */
     @Override
     public int read(final byte[] buf, int off, int len) throws IOException {
+        if (len == 0) {
+            return 0;
+        }
         int totalRead = 0;
 
         if (hasHitEOF || isClosed || entryOffset >= entrySize) {
