@@ -32,15 +32,21 @@ import org.apache.commons.compress.PasswordRequiredException;
 class AES256SHA256Decoder extends CoderBase {
     @Override
     InputStream decode(final String archiveName, final InputStream in, final long uncompressedLength,
-            final Coder coder, final byte[] passwordBytes) throws IOException {
+            final Coder coder, final byte[] passwordBytes, final int maxMemoryLimitInKb) throws IOException {
         return new InputStream() {
-            private boolean isInitialized = false;
-            private CipherInputStream cipherInputStream = null;
+            private boolean isInitialized;
+            private CipherInputStream cipherInputStream;
 
             private CipherInputStream init() throws IOException {
                 if (isInitialized) {
                     return cipherInputStream;
                 }
+                if (coder.properties == null) {
+                    throw new IOException("Missing AES256 properties in " + archiveName);
+                }
+                if (coder.properties.length < 2) {
+                    throw new IOException("AES256 properties too short in " + archiveName);
+                }
                 final int byte0 = 0xff & coder.properties[0];
                 final int numCyclesPower = byte0 & 0x3f;
                 final int byte1 = 0xff & coder.properties[1];
@@ -111,7 +117,10 @@ class AES256SHA256Decoder extends CoderBase {
             }
 
             @Override
-            public void close() {
+            public void close() throws IOException {
+                if (cipherInputStream != null) {
+                    cipherInputStream.close();
+                }
             }
         };
     }
