@@ -24,11 +24,10 @@
 package org.apache.commons.compress.archivers.tar;
 
 import java.io.ByteArrayOutputStream;
+import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
-import java.util.Collections;
-import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -39,7 +38,6 @@ import org.apache.commons.compress.archivers.zip.ZipEncoding;
 import org.apache.commons.compress.archivers.zip.ZipEncodingHelper;
 import org.apache.commons.compress.utils.ArchiveUtils;
 import org.apache.commons.compress.utils.BoundedInputStream;
-import org.apache.commons.compress.utils.CharsetNames;
 import org.apache.commons.compress.utils.IOUtils;
 
 /**
@@ -58,6 +56,9 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     /** The size the TAR header */
     private final int recordSize;
 
+    /** The buffer to store the TAR header **/
+    private final byte[] recordBuffer;
+
     /** The size of a block */
     private final int blockSize;
 
@@ -112,7 +113,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * exception instead.
      * @since 1.19
      */
-    public TarArchiveInputStream(final InputStream is, boolean lenient) {
+    public TarArchiveInputStream(final InputStream is, final boolean lenient) {
         this(is, TarConstants.DEFAULT_BLKSIZE, TarConstants.DEFAULT_RCDSIZE, null, lenient);
     }
 
@@ -183,12 +184,13 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @since 1.19
      */
     public TarArchiveInputStream(final InputStream is, final int blockSize, final int recordSize,
-                                 final String encoding, boolean lenient) {
+                                 final String encoding, final boolean lenient) {
         this.inputStream = is;
         this.hasHitEOF = false;
         this.encoding = encoding;
         this.zipEncoding = ZipEncodingHelper.getZipEncoding(encoding);
         this.recordSize = recordSize;
+        this.recordBuffer = new byte[recordSize];
         this.blockSize = blockSize;
         this.lenient = lenient;
     }
@@ -201,7 +203,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     public void close() throws IOException {
         // Close all the input streams in sparseInputStreams
         if(sparseInputStreams != null) {
-            for (InputStream inputStream : sparseInputStreams) {
+            for (final InputStream inputStream : sparseInputStreams) {
                 inputStream.close();
             }
         }
@@ -256,8 +258,8 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @param n
      *            the number of bytes to be skipped.
      * @return the actual number of bytes skipped.
-     * @throws IOException
-     *                if some other I/O error occurs.
+     * @throws IOException if a truncated tar archive is detected
+     *                     or some other I/O error occurs
      */
     @Override
     public long skip(final long n) throws IOException {
@@ -265,13 +267,21 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             return 0;
         }
 
+        final long availableOfInputStream = inputStream.available();
         final long available = currEntry.getRealSize() - entryOffset;
-        final long skipped;
+        final long numToSkip = Math.min(n, available);
+        long skipped;
+
         if (!currEntry.isSparse()) {
-            skipped = IOUtils.skip(inputStream, Math.min(n, available));
+            skipped = IOUtils.skip(inputStream, numToSkip);
+            // for non-sparse entry, we should get the bytes actually skipped bytes along with
+            // inputStream.available() if inputStream is instance of FileInputStream
+            skipped = getActuallySkipped(availableOfInputStream, skipped, numToSkip);
         } else {
-            skipped = skipSparse(Math.min(n, available));
+            skipped = skipSparse(numToSkip);
         }
+
+
         count(skipped);
         entryOffset += skipped;
         return skipped;
@@ -287,7 +297,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @throws IOException
      */
     private long skipSparse(final long n) throws IOException {
-        if (sparseInputStreams == null || sparseInputStreams.size() == 0) {
+        if (sparseInputStreams == null || sparseInputStreams.isEmpty()) {
             return inputStream.skip(n);
         }
 
@@ -393,17 +403,27 @@ public class TarArchiveInputStream extends ArchiveInputStream {
                 // entry
                 return null;
             }
-            currEntry.setName(zipEncoding.decode(longNameData));
+
+            // COMPRESS-509 : the name of directories should end with '/'
+            final String name = zipEncoding.decode(longNameData);
+            currEntry.setName(name);
+            if (currEntry.isDirectory() && !name.endsWith("/")) {
+                currEntry.setName(name + "/");
+            }
         }
 
         if (currEntry.isGlobalPaxHeader()){ // Process Global Pax headers
             readGlobalPaxHeaders();
         }
 
-        if (currEntry.isPaxHeader()){ // Process Pax headers
-            paxHeaders();
-        } else if (!globalPaxHeaders.isEmpty()) {
-            applyPaxHeadersToCurrentEntry(globalPaxHeaders, globalSparseHeaders);
+        try {
+            if (currEntry.isPaxHeader()){ // Process Pax headers
+                paxHeaders();
+            } else if (!globalPaxHeaders.isEmpty()) {
+                applyPaxHeadersToCurrentEntry(globalPaxHeaders, globalSparseHeaders);
+            }
+        } catch (final NumberFormatException e) {
+            throw new IOException("Error detected parsing the pax header", e);
         }
 
         if (currEntry.isOldGNUSparse()){ // Process sparse files
@@ -421,17 +441,46 @@ public class TarArchiveInputStream extends ArchiveInputStream {
 
     /**
      * The last record block should be written at the full size, so skip any
-     * additional space used to fill a record after an entry
+     * additional space used to fill a record after an entry.
+     *
+     * @throws IOException if a truncated tar archive is detected
      */
     private void skipRecordPadding() throws IOException {
         if (!isDirectory() && this.entrySize > 0 && this.entrySize % this.recordSize != 0) {
+            final long available = inputStream.available();
             final long numRecords = (this.entrySize / this.recordSize) + 1;
             final long padding = (numRecords * this.recordSize) - this.entrySize;
-            final long skipped = IOUtils.skip(inputStream, padding);
+            long skipped = IOUtils.skip(inputStream, padding);
+
+            skipped = getActuallySkipped(available, skipped, padding);
+
             count(skipped);
         }
     }
 
+    /**
+     * For FileInputStream, the skip always return the number you input, so we
+     * need the available bytes to determine how many bytes are actually skipped
+     *
+     * @param available available bytes returned by inputStream.available()
+     * @param skipped   skipped bytes returned by inputStream.skip()
+     * @param expected  bytes expected to skip
+     * @return number of bytes actually skipped
+     * @throws IOException if a truncated tar archive is detected
+     */
+    private long getActuallySkipped(final long available, final long skipped, final long expected) throws IOException {
+        long actuallySkipped = skipped;
+        if (inputStream instanceof FileInputStream) {
+            actuallySkipped = Math.min(skipped, available);
+        }
+
+        if (actuallySkipped != expected) {
+            throw new IOException("Truncated TAR archive");
+        }
+
+        return actuallySkipped;
+    }
+
     /**
      * Get the next entry in this tar archive as longname data.
      *
@@ -508,21 +557,22 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @throws IOException on error
      */
     protected byte[] readRecord() throws IOException {
-
-        final byte[] record = new byte[recordSize];
-
-        final int readNow = IOUtils.readFully(inputStream, record);
+        final int readNow = IOUtils.readFully(inputStream, recordBuffer);
         count(readNow);
         if (readNow != recordSize) {
             return null;
         }
 
-        return record;
+        return recordBuffer;
     }
 
     private void readGlobalPaxHeaders() throws IOException {
-        globalPaxHeaders = parsePaxHeaders(this, globalSparseHeaders);
+        globalPaxHeaders = TarUtils.parsePaxHeaders(this, globalSparseHeaders, globalPaxHeaders, entrySize);
         getNextEntry(); // Get the actual file entry
+
+        if (currEntry == null) {
+            throw new IOException("Error detected parsing the pax header");
+        }
     }
 
     /**
@@ -550,20 +600,23 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * giving the offset and size of the data block it describes.
      * @throws IOException
      */
-    private void paxHeaders() throws IOException{
+    private void paxHeaders() throws IOException {
         List<TarArchiveStructSparse> sparseHeaders = new ArrayList<>();
-        final Map<String, String> headers = parsePaxHeaders(this, sparseHeaders);
+        final Map<String, String> headers = TarUtils.parsePaxHeaders(this, sparseHeaders, globalPaxHeaders, entrySize);
 
         // for 0.1 PAX Headers
         if (headers.containsKey("GNU.sparse.map")) {
-            sparseHeaders = parsePAX01SparseHeaders(headers.get("GNU.sparse.map"));
+            sparseHeaders = new ArrayList<>(TarUtils.parseFromPAX01SparseHeaders(headers.get("GNU.sparse.map")));
         }
         getNextEntry(); // Get the actual file entry
+        if (currEntry == null) {
+            throw new IOException("premature end of tar archive. Didn't find any entry after PAX header.");
+        }
         applyPaxHeadersToCurrentEntry(headers, sparseHeaders);
 
         // for 1.0 PAX Format, the sparse map is stored in the file data block
         if (currEntry.isPaxGNU1XSparse()) {
-            sparseHeaders = parsePAX1XSparseHeaders();
+            sparseHeaders = TarUtils.parsePAX1XSparseHeaders(inputStream, recordSize);
             currEntry.setSparseHeaders(sparseHeaders);
         }
 
@@ -572,188 +625,8 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         buildSparseInputStreams();
     }
 
-    /**
-     * For PAX Format 0.1, the sparse headers are stored in a single variable : GNU.sparse.map
-     * GNU.sparse.map
-     *    Map of non-null data chunks. It is a string consisting of comma-separated values "offset,size[,offset-1,size-1...]"
-     *
-     * @param sparseMap the sparse map string consisting of comma-separated values "offset,size[,offset-1,size-1...]"
-     * @return sparse headers parsed from sparse map
-     * @throws IOException
-     */
-    private List<TarArchiveStructSparse> parsePAX01SparseHeaders(String sparseMap) throws IOException {
-        List<TarArchiveStructSparse> sparseHeaders = new ArrayList<>();
-        String[] sparseHeaderStrings = sparseMap.split(",");
-
-        for (int i = 0; i < sparseHeaderStrings.length;i += 2) {
-            long sparseOffset = Long.parseLong(sparseHeaderStrings[i]);
-            long sparseNumbytes = Long.parseLong(sparseHeaderStrings[i + 1]);
-            sparseHeaders.add(new TarArchiveStructSparse(sparseOffset, sparseNumbytes));
-        }
-
-        return sparseHeaders;
-    }
-
-    /**
-     * For PAX Format 1.X:
-     * The sparse map itself is stored in the file data block, preceding the actual file data.
-     * It consists of a series of decimal numbers delimited by newlines. The map is padded with nulls to the nearest block boundary.
-     * The first number gives the number of entries in the map. Following are map entries, each one consisting of two numbers
-     * giving the offset and size of the data block it describes.
-     * @return sparse headers
-     * @throws IOException
-     */
-    private List<TarArchiveStructSparse> parsePAX1XSparseHeaders() throws IOException {
-        // for 1.X PAX Headers
-        List<TarArchiveStructSparse> sparseHeaders = new ArrayList<>();
-        long bytesRead = 0;
-
-        long[] readResult = readLineOfNumberForPax1X(inputStream);
-        long sparseHeadersCount = readResult[0];
-        bytesRead += readResult[1];
-        while (sparseHeadersCount-- > 0) {
-            readResult = readLineOfNumberForPax1X(inputStream);
-            long sparseOffset = readResult[0];
-            bytesRead += readResult[1];
-
-            readResult = readLineOfNumberForPax1X(inputStream);
-            long sparseNumbytes = readResult[0];
-            bytesRead += readResult[1];
-            sparseHeaders.add(new TarArchiveStructSparse(sparseOffset, sparseNumbytes));
-        }
-
-        // skip the rest of this record data
-        long bytesToSkip = recordSize - bytesRead % recordSize;
-        IOUtils.skip(inputStream, bytesToSkip);
-        return sparseHeaders;
-    }
-
-    /**
-     * For 1.X PAX Format, the sparse headers are stored in the file data block, preceding the actual file data.
-     * It consists of a series of decimal numbers delimited by newlines.
-     *
-     * @param inputStream the input stream of the tar file
-     * @return the decimal number delimited by '\n', and the bytes read from input stream
-     * @throws IOException
-     */
-    private long[] readLineOfNumberForPax1X(InputStream inputStream) throws IOException {
-        int number;
-        long result = 0;
-        long bytesRead = 0;
-
-        while((number = inputStream.read()) != '\n') {
-            bytesRead += 1;
-            if(number == -1) {
-                throw new IOException("Unexpected EOF when reading parse information of 1.X PAX format");
-            }
-            result = result * 10 + (number - '0');
-        }
-        bytesRead += 1;
-
-        return new long[] {result, bytesRead};
-    }
-
-    /**
-     * For PAX Format 0.0, the sparse headers(GNU.sparse.offset and GNU.sparse.numbytes)
-     * may appear multi times, and they look like:
-     *
-     * GNU.sparse.size=size
-     * GNU.sparse.numblocks=numblocks
-     * repeat numblocks times
-     *   GNU.sparse.offset=offset
-     *   GNU.sparse.numbytes=numbytes
-     * end repeat
-     *
-     * For PAX Format 0.1, the sparse headers are stored in a single variable : GNU.sparse.map
-     *
-     * GNU.sparse.map
-     *    Map of non-null data chunks. It is a string consisting of comma-separated values "offset,size[,offset-1,size-1...]"
-     *
-     * @param inputstream inputstream to read keys and values
-     * @param sparseHeaders used in PAX Format 0.0 &amp; 0.1, as it may appear multi times,
-     *                      the sparse headers need to be stored in an array, not a map
-     * @return map of PAX headers values found inside of the current (local or global) PAX headers tar entry.
-     * @throws IOException
-     */
-    Map<String, String> parsePaxHeaders(final InputStream inputStream, List<TarArchiveStructSparse> sparseHeaders)
+    private void applyPaxHeadersToCurrentEntry(final Map<String, String> headers, final List<TarArchiveStructSparse> sparseHeaders)
         throws IOException {
-        final Map<String, String> headers = new HashMap<>(globalPaxHeaders);
-        Long offset = null;
-        // Format is "length keyword=value\n";
-        while(true) { // get length
-            int ch;
-            int len = 0;
-            int read = 0;
-            while((ch = inputStream.read()) != -1) {
-                read++;
-                if (ch == '\n') { // blank line in header
-                    break;
-                } else if (ch == ' '){ // End of length string
-                    // Get keyword
-                    final ByteArrayOutputStream coll = new ByteArrayOutputStream();
-                    while((ch = inputStream.read()) != -1) {
-                        read++;
-                        if (ch == '='){ // end of keyword
-                            final String keyword = coll.toString(CharsetNames.UTF_8);
-                            // Get rest of entry
-                            final int restLen = len - read;
-                            if (restLen == 1) { // only NL
-                                headers.remove(keyword);
-                            } else {
-                                final byte[] rest = new byte[restLen];
-                                final int got = IOUtils.readFully(inputStream, rest);
-                                if (got != restLen) {
-                                    throw new IOException("Failed to read "
-                                                          + "Paxheader. Expected "
-                                                          + restLen
-                                                          + " bytes, read "
-                                                          + got);
-                                }
-                                // Drop trailing NL
-                                final String value = new String(rest, 0,
-                                                          restLen - 1, CharsetNames.UTF_8);
-                                headers.put(keyword, value);
-
-                                // for 0.0 PAX Headers
-                                if (keyword.equals("GNU.sparse.offset")) {
-                                    if (offset != null) {
-                                        // previous GNU.sparse.offset header but but no numBytes
-                                        sparseHeaders.add(new TarArchiveStructSparse(offset, 0));
-                                    }
-                                    offset = Long.valueOf(value);
-                                }
-
-                                // for 0.0 PAX Headers
-                                if (keyword.equals("GNU.sparse.numbytes")) {
-                                    if (offset == null) {
-                                        throw new IOException("Failed to read Paxheader." +
-                                                "GNU.sparse.offset is expected before GNU.sparse.numbytes shows up.");
-                                    }
-                                    sparseHeaders.add(new TarArchiveStructSparse(offset, Long.parseLong(value)));
-                                    offset = null;
-                                }
-                            }
-                            break;
-                        }
-                        coll.write((byte) ch);
-                    }
-                    break; // Processed single header
-                }
-                len *= 10;
-                len += ch - '0';
-            }
-            if (ch == -1){ // EOF
-                break;
-            }
-        }
-        if (offset != null) {
-            // offset but no numBytes
-            sparseHeaders.add(new TarArchiveStructSparse(offset, 0));
-        }
-        return headers;
-    }
-
-    private void applyPaxHeadersToCurrentEntry(final Map<String, String> headers, final List<TarArchiveStructSparse> sparseHeaders) {
         currEntry.updateEntryFromPaxHeaders(headers);
         currEntry.setSparseHeaders(sparseHeaders);
     }
@@ -770,8 +643,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             do {
                 final byte[] headerBuf = getRecord();
                 if (headerBuf == null) {
-                    currEntry = null;
-                    break;
+                    throw new IOException("premature end of tar archive. Didn't find extended_header after header with extended flag.");
                 }
                 entry = new TarArchiveSparseEntry(headerBuf);
                 currEntry.getSparseHeaders().addAll(entry.getSparseHeaders());
@@ -853,15 +725,8 @@ public class TarArchiveInputStream extends ArchiveInputStream {
             throw new IllegalStateException("No current tar entry");
         }
 
-        if (!currEntry.isSparse()) {
-            if (entryOffset >= entrySize) {
-                return -1;
-            }
-        } else {
-            // for sparse entries, there are actually currEntry.getRealSize() bytes to read
-            if (entryOffset >= currEntry.getRealSize()) {
-                return -1;
-            }
+        if (entryOffset >= currEntry.getRealSize()) {
+            return -1;
         }
 
         numToRead = Math.min(numToRead, available());
@@ -900,18 +765,18 @@ public class TarArchiveInputStream extends ArchiveInputStream {
      * @return The number of bytes read, or -1 at EOF.
      * @throws IOException on error
      */
-    private int readSparse(final byte[] buf, final int offset, int numToRead) throws IOException {
+    private int readSparse(final byte[] buf, final int offset, final int numToRead) throws IOException {
         // if there are no actual input streams, just read from the original input stream
-        if (sparseInputStreams == null || sparseInputStreams.size() == 0) {
+        if (sparseInputStreams == null || sparseInputStreams.isEmpty()) {
             return inputStream.read(buf, offset, numToRead);
         }
 
-        if(currentSparseInputStreamIndex >= sparseInputStreams.size()) {
+        if (currentSparseInputStreamIndex >= sparseInputStreams.size()) {
             return -1;
         }
 
-        InputStream currentInputStream = sparseInputStreams.get(currentSparseInputStreamIndex);
-        int readLen = currentInputStream.read(buf, offset, numToRead);
+        final InputStream currentInputStream = sparseInputStreams.get(currentSparseInputStreamIndex);
+        final int readLen = currentInputStream.read(buf, offset, numToRead);
 
         // if the current input stream is the last input stream,
         // just return the number of bytes read from current input stream
@@ -929,7 +794,7 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         // and recursively call read
         if (readLen < numToRead) {
             currentSparseInputStreamIndex++;
-            int readLenOfNext = readSparse(buf, offset + readLen, numToRead - readLen);
+            final int readLenOfNext = readSparse(buf, offset + readLen, numToRead - readLen);
             if (readLenOfNext == -1) {
                 return readLen;
             }
@@ -944,15 +809,11 @@ public class TarArchiveInputStream extends ArchiveInputStream {
     /**
      * Whether this class is able to read the given entry.
      *
-     * <p>May return false if the current entry is a sparse file.</p>
+     * @return The implementation will return true if the {@link ArchiveEntry} is an instance of {@link TarArchiveEntry}
      */
     @Override
     public boolean canReadEntryData(final ArchiveEntry ae) {
-        if (ae instanceof TarArchiveEntry) {
-            final TarArchiveEntry te = (TarArchiveEntry) ae;
-            return !te.isSparse();
-        }
-        return false;
+        return ae instanceof TarArchiveEntry;
     }
 
     /**
@@ -1044,76 +905,34 @@ public class TarArchiveInputStream extends ArchiveInputStream {
         currentSparseInputStreamIndex = -1;
         sparseInputStreams = new ArrayList<>();
 
-        final List<TarArchiveStructSparse> sparseHeaders = currEntry.getSparseHeaders();
-        // sort the sparse headers in case they are written in wrong order
-        if (sparseHeaders != null && sparseHeaders.size() > 1) {
-            final Comparator<TarArchiveStructSparse> sparseHeaderComparator = new Comparator<TarArchiveStructSparse>() {
-                @Override
-                public int compare(final TarArchiveStructSparse p, final TarArchiveStructSparse q) {
-                    Long pOffset = p.getOffset();
-                    Long qOffset = q.getOffset();
-                    return pOffset.compareTo(qOffset);
-                }
-            };
-            Collections.sort(sparseHeaders, sparseHeaderComparator);
-        }
+        final List<TarArchiveStructSparse> sparseHeaders = currEntry.getOrderedSparseHeaders();
 
-        if (sparseHeaders != null) {
-            // Stream doesn't need to be closed at all as it doesn't use any resources
-            final InputStream zeroInputStream = new TarArchiveSparseZeroInputStream(); //NOSONAR
-            long offset = 0;
-            for (TarArchiveStructSparse sparseHeader : sparseHeaders) {
-                if (sparseHeader.getOffset() == 0 && sparseHeader.getNumbytes() == 0) {
-                    break;
-                }
-
-                if ((sparseHeader.getOffset() - offset) < 0) {
-                    throw new IOException("Corrupted struct sparse detected");
-                }
-
-                // only store the input streams with non-zero size
-                if ((sparseHeader.getOffset() - offset) > 0) {
-                    sparseInputStreams.add(new BoundedInputStream(zeroInputStream, sparseHeader.getOffset() - offset));
-                }
-
-                // only store the input streams with non-zero size
-                if (sparseHeader.getNumbytes() > 0) {
-                    sparseInputStreams.add(new BoundedInputStream(inputStream, sparseHeader.getNumbytes()));
-                }
-
-                offset = sparseHeader.getOffset() + sparseHeader.getNumbytes();
+        // Stream doesn't need to be closed at all as it doesn't use any resources
+        final InputStream zeroInputStream = new TarArchiveSparseZeroInputStream(); //NOSONAR
+        // logical offset into the extracted entry
+        long offset = 0;
+        for (final TarArchiveStructSparse sparseHeader : sparseHeaders) {
+            final long zeroBlockSize = sparseHeader.getOffset() - offset;
+            if (zeroBlockSize < 0) {
+                // sparse header says to move backwards inside of the extracted entry
+                throw new IOException("Corrupted struct sparse detected");
             }
+
+            // only store the zero block if it is not empty
+            if (zeroBlockSize > 0) {
+                sparseInputStreams.add(new BoundedInputStream(zeroInputStream, sparseHeader.getOffset() - offset));
+            }
+
+            // only store the input streams with non-zero size
+            if (sparseHeader.getNumbytes() > 0) {
+                sparseInputStreams.add(new BoundedInputStream(inputStream, sparseHeader.getNumbytes()));
+            }
+
+            offset = sparseHeader.getOffset() + sparseHeader.getNumbytes();
         }
 
-        if (sparseInputStreams.size() > 0) {
+        if (!sparseInputStreams.isEmpty()) {
             currentSparseInputStreamIndex = 0;
         }
     }
-
-    /**
-     * This is an inputstream that always return 0,
-     * this is used when reading the "holes" of a sparse file
-     */
-    private static class TarArchiveSparseZeroInputStream extends InputStream {
-        /**
-         * Just return 0
-         * @return
-         * @throws IOException
-         */
-        @Override
-        public int read() throws IOException {
-            return 0;
-        }
-
-        /**
-         * these's nothing need to do when skipping
-         *
-         * @param n bytes to skip
-         * @return bytes actually skipped
-         */
-        @Override
-        public long skip(final long n) {
-            return n;
-        }
-    }
 }
