@@ -22,17 +22,19 @@ import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.StringWriter;
-import java.io.UnsupportedEncodingException;
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.LinkOption;
+import java.nio.file.Path;
 import java.util.Arrays;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
+
 import org.apache.commons.compress.archivers.ArchiveEntry;
 import org.apache.commons.compress.archivers.ArchiveOutputStream;
 import org.apache.commons.compress.archivers.zip.ZipEncoding;
 import org.apache.commons.compress.archivers.zip.ZipEncodingHelper;
-import org.apache.commons.compress.utils.CharsetNames;
 import org.apache.commons.compress.utils.CountingOutputStream;
 import org.apache.commons.compress.utils.FixedLengthBlockOutputStream;
 
@@ -98,17 +100,17 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     private int recordsWritten;
     private final int recordsPerBlock;
 
-    private boolean closed = false;
+    private boolean closed;
 
     /**
      * Indicates if putArchiveEntry has been called without closeArchiveEntry
      */
-    private boolean haveUnclosedEntry = false;
+    private boolean haveUnclosedEntry;
 
     /**
      * indicates if this archive is finished
      */
-    private boolean finished = false;
+    private boolean finished;
 
     private final FixedLengthBlockOutputStream out;
     private final CountingOutputStream countingOut;
@@ -118,7 +120,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     // the provided encoding (for unit tests)
     final String encoding;
 
-    private boolean addPaxHeadersForNonAsciiNames = false;
+    private boolean addPaxHeadersForNonAsciiNames;
     private static final ZipEncoding ASCII =
         ZipEncodingHelper.getZipEncoding("ASCII");
 
@@ -206,7 +208,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      */
     public TarArchiveOutputStream(final OutputStream os, final int blockSize,
         final String encoding) {
-        int realBlockSize;
+        final int realBlockSize;
         if (BLOCK_SIZE_UNSPECIFIED == blockSize) {
             realBlockSize = RECORD_SIZE;
         } else {
@@ -335,6 +337,12 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      * @param archiveEntry The TarEntry to be written to the archive.
      * @throws IOException on error
      * @throws ClassCastException if archiveEntry is not an instance of TarArchiveEntry
+     * @throws IllegalArgumentException if the {@link TarArchiveOutputStream#longFileMode} equals
+     *                                  {@link TarArchiveOutputStream#LONGFILE_ERROR} and the file
+     *                                  name is too long
+     * @throws IllegalArgumentException if the {@link TarArchiveOutputStream#bigNumberMode} equals
+     *         {@link TarArchiveOutputStream#BIGNUMBER_ERROR} and one of the numeric values
+     *         exceeds the limits of a traditional tar header.
      */
     @Override
     public void putArchiveEntry(final ArchiveEntry archiveEntry) throws IOException {
@@ -359,7 +367,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
                 TarConstants.LF_GNUTYPE_LONGNAME, "file name");
 
             final String linkName = entry.getLinkName();
-            final boolean paxHeaderContainsLinkPath = linkName != null && linkName.length() > 0
+            final boolean paxHeaderContainsLinkPath = linkName != null && !linkName.isEmpty()
                 && handleLongName(entry, linkName, paxHeaders, "linkpath",
                 TarConstants.LF_GNUTYPE_LONGLINK, "link name");
 
@@ -381,7 +389,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
             }
             paxHeaders.putAll(entry.getExtraPaxHeaders());
 
-            if (paxHeaders.size() > 0) {
+            if (!paxHeaders.isEmpty()) {
                 writePaxHeaders(entry, entryName, paxHeaders);
             }
 
@@ -418,7 +426,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
         }
         out.flushBlock();
         if (currBytes < currSize) {
-            throw new IOException("entry '" + currName + "' closed at '"
+            throw new IOException("Entry '" + currName + "' closed at '"
                 + currBytes
                 + "' before the '" + currSize
                 + "' bytes specified in the header were written");
@@ -441,12 +449,12 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      * @throws IOException on error
      */
     @Override
-    public void write(final byte[] wBuf, int wOffset, int numToWrite) throws IOException {
+    public void write(final byte[] wBuf, final int wOffset, final int numToWrite) throws IOException {
         if (!haveUnclosedEntry) {
             throw new IllegalStateException("No current tar entry");
         }
         if (currBytes + numToWrite > currSize) {
-            throw new IOException("request to write '" + numToWrite
+            throw new IOException("Request to write '" + numToWrite
                 + "' bytes exceeds size in header of '"
                 + currSize + "' bytes for entry '"
                 + currName + "'");
@@ -478,8 +486,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
         closeArchiveEntry();
     }
 
-    private byte[] encodeExtendedPaxHeadersContents(Map<String, String> headers)
-        throws UnsupportedEncodingException {
+    private byte[] encodeExtendedPaxHeadersContents(final Map<String, String> headers) {
         final StringWriter w = new StringWriter();
         for (final Map.Entry<String, String> h : headers.entrySet()) {
             final String key = h.getKey();
@@ -488,7 +495,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
                 + 3 /* blank, equals and newline */
                 + 2 /* guess 9 < actual length < 100 */;
             String line = len + " " + key + "=" + value + "\n";
-            int actualLength = line.getBytes(CharsetNames.UTF_8).length;
+            int actualLength = line.getBytes(StandardCharsets.UTF_8).length;
             while (len != actualLength) {
                 // Adjust for cases where length < 10 or > 100
                 // or where UTF-8 encoding isn't a single octet
@@ -497,11 +504,11 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
                 // first pass so we'd need a second.
                 len = actualLength;
                 line = len + " " + key + "=" + value + "\n";
-                actualLength = line.getBytes(CharsetNames.UTF_8).length;
+                actualLength = line.getBytes(StandardCharsets.UTF_8).length;
             }
             w.write(line);
         }
-        return w.toString().getBytes(CharsetNames.UTF_8);
+        return w.toString().getBytes(StandardCharsets.UTF_8);
     }
 
     private String stripTo7Bits(final String name) {
@@ -551,6 +558,14 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
         return new TarArchiveEntry(inputFile, entryName);
     }
 
+    @Override
+    public ArchiveEntry createArchiveEntry(final Path inputPath, final String entryName, final LinkOption... options) throws IOException {
+        if (finished) {
+            throw new IOException("Stream has already been finished");
+        }
+        return new TarArchiveEntry(inputPath, entryName, options);
+    }
+
     /**
      * Write an archive record to the archive.
      *
@@ -559,7 +574,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      */
     private void writeRecord(final byte[] record) throws IOException {
         if (record.length != RECORD_SIZE) {
-            throw new IOException("record to write has length '"
+            throw new IOException("Record to write has length '"
                 + record.length
                 + "' which is not the record size of '"
                 + RECORD_SIZE + "'");
@@ -633,7 +648,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     private void failForBigNumber(final String field, final long value, final long maxValue,
         final String additionalMsg) {
         if (value < 0 || value > maxValue) {
-            throw new RuntimeException(field + " '" + value //NOSONAR
+            throw new IllegalArgumentException(field + " '" + value //NOSONAR
                 + "' is too big ( > "
                 + maxValue + " )." + additionalMsg);
         }
@@ -654,6 +669,9 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      * @param paxHeaderName name of the pax header to write
      * @param linkType type of the GNU entry to write
      * @param fieldName the name of the field
+     * @throws IllegalArgumentException if the {@link TarArchiveOutputStream#longFileMode} equals
+     *                                  {@link TarArchiveOutputStream#LONGFILE_ERROR} and the file
+     *                                  name is too long
      * @return whether a pax header has been written.
      */
     private boolean handleLongName(final TarArchiveEntry entry, final String name,
@@ -667,7 +685,8 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
             if (longFileMode == LONGFILE_POSIX) {
                 paxHeaders.put(paxHeaderName, name);
                 return true;
-            } else if (longFileMode == LONGFILE_GNU) {
+            }
+            if (longFileMode == LONGFILE_GNU) {
                 // create a TarEntry for the LongLink, the contents
                 // of which are the link's name
                 final TarArchiveEntry longLinkEntry = new TarArchiveEntry(TarConstants.GNU_LONGLINK,
@@ -680,7 +699,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
                 write(0); // NUL terminator
                 closeArchiveEntry();
             } else if (longFileMode != LONGFILE_TRUNCATE) {
-                throw new RuntimeException(fieldName + " '" + name //NOSONAR
+                throw new IllegalArgumentException(fieldName + " '" + name //NOSONAR
                     + "' is too long ( > "
                     + TarConstants.NAMELEN + " bytes)");
             }
