@@ -66,9 +66,9 @@ class ExplodingInputStream extends InputStream implements InputStreamStatistics
     /** Output buffer holding the decompressed data */
     private final CircularBuffer buffer = new CircularBuffer(32 * 1024);
 
-    private long uncompressedCount = 0;
+    private long uncompressedCount;
 
-    private long treeSizes = 0;
+    private long treeSizes;
 
     /**
      * Create a new stream decompressing the content of the specified stream
@@ -116,7 +116,11 @@ class ExplodingInputStream extends InputStream implements InputStreamStatistics
     @Override
     public int read() throws IOException {
         if (!buffer.available()) {
-            fillBuffer();
+            try {
+                fillBuffer();
+            } catch (final IllegalArgumentException ex) {
+                throw new IOException("bad IMPLODE stream", ex);
+            }
         }
 
         final int ret = buffer.get();
@@ -161,9 +165,10 @@ class ExplodingInputStream extends InputStream implements InputStreamStatistics
         if (bit == -1) {
             // EOF
             return;
-        } else if (bit == 1) {
+        }
+        if (bit == 1) {
             // literal value
-            int literal;
+            final int literal;
             if (literalTree != null) {
                 literal = literalTree.read(bits);
             } else {
