@@ -99,26 +99,36 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
     private final ByteBuffer buf = ByteBuffer.allocate(ZipArchiveOutputStream.BUFFER_SIZE);
 
     /** The entry that is currently being read. */
-    private CurrentEntry current = null;
+    private CurrentEntry current;
 
     /** Whether the stream has been closed. */
-    private boolean closed = false;
+    private boolean closed;
 
     /** Whether the stream has reached the central directory - and thus found all entries. */
-    private boolean hitCentralDirectory = false;
+    private boolean hitCentralDirectory;
 
     /**
      * When reading a stored entry that uses the data descriptor this
      * stream has to read the full entry and caches it.  This is the
      * cache.
      */
-    private ByteArrayInputStream lastStoredEntry = null;
+    private ByteArrayInputStream lastStoredEntry;
 
-    /** Whether the stream will try to read STORED entries that use a data descriptor. */
-    private boolean allowStoredEntriesWithDataDescriptor = false;
+    /**
+     * Whether the stream will try to read STORED entries that use a data descriptor.
+     * Setting it to true means we will not stop reading a entry with the compressed
+     * size, instead we will stoping reading a entry when a data descriptor is met(by
+     * finding the Data Descriptor Signature). This will completely break down in some
+     * cases - like JARs in WARs.
+     * <p>
+     * See also :
+     * https://issues.apache.org/jira/projects/COMPRESS/issues/COMPRESS-555
+     * https://github.com/apache/commons-compress/pull/137#issuecomment-690835644
+     */
+    private boolean allowStoredEntriesWithDataDescriptor;
 
     /** Count decompressed bytes for current entry */
-    private long uncompressedCount = 0;
+    private long uncompressedCount;
 
     /** Whether the stream will try to skip the zip split signature(08074B50) at the beginning **/
     private final boolean skipSplitSig;
@@ -168,7 +178,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
     private final byte[] wordBuf = new byte[WORD];
     private final byte[] twoDwordBuf = new byte[2 * DWORD];
 
-    private int entriesRead = 0;
+    private int entriesRead;
 
     /**
      * Create an instance using UTF-8 encoding
@@ -261,14 +271,14 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
             firstEntry = false;
         }
 
-        long currentHeaderOffset = getBytesRead();
+        final long currentHeaderOffset = getBytesRead();
         try {
             if (firstEntry) {
                 // split archives have a special signature before the
                 // first local file header - look for it and fail with
                 // the appropriate error message if this is a split
                 // archive.
-                readFirstLocalFileHeader(lfhBuf);
+                readFirstLocalFileHeader();
             } else {
                 readFully(lfhBuf);
             }
@@ -329,16 +339,20 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         final int extraLen = ZipShort.getValue(lfhBuf, off);
         off += SHORT; // NOSONAR - assignment as documentation
 
-        final byte[] fileName = new byte[fileNameLen];
-        readFully(fileName);
+        final byte[] fileName = readRange(fileNameLen);
         current.entry.setName(entryEncoding.decode(fileName), fileName);
         if (hasUTF8Flag) {
             current.entry.setNameSource(ZipArchiveEntry.NameSource.NAME_WITH_EFS_FLAG);
         }
 
-        final byte[] extraData = new byte[extraLen];
-        readFully(extraData);
-        current.entry.setExtra(extraData);
+        final byte[] extraData = readRange(extraLen);
+        try {
+            current.entry.setExtra(extraData);
+        } catch (RuntimeException ex) {
+            final ZipException z = new ZipException("Invalid extra data in entry " + current.entry.getName());
+            z.initCause(ex);
+            throw z;
+        }
 
         if (!hasUTF8Flag && useUnicodeExtraFields) {
             ZipUtil.setNameAndCommentFromExtraFields(current.entry, fileName, null);
@@ -350,19 +364,23 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         current.entry.setDataOffset(getBytesRead());
         current.entry.setStreamContiguous(true);
 
-        ZipMethod m = ZipMethod.getMethodByCode(current.entry.getMethod());
+        final ZipMethod m = ZipMethod.getMethodByCode(current.entry.getMethod());
         if (current.entry.getCompressedSize() != ArchiveEntry.SIZE_UNKNOWN) {
             if (ZipUtil.canHandleEntryData(current.entry) && m != ZipMethod.STORED && m != ZipMethod.DEFLATED) {
-                InputStream bis = new BoundedInputStream(in, current.entry.getCompressedSize());
+                final InputStream bis = new BoundedInputStream(in, current.entry.getCompressedSize());
                 switch (m) {
                 case UNSHRINKING:
                     current.in = new UnshrinkingInputStream(bis);
                     break;
                 case IMPLODING:
-                    current.in = new ExplodingInputStream(
-                        current.entry.getGeneralPurposeBit().getSlidingDictionarySize(),
-                        current.entry.getGeneralPurposeBit().getNumberOfShannonFanoTrees(),
-                        bis);
+                    try {
+                        current.in = new ExplodingInputStream(
+                            current.entry.getGeneralPurposeBit().getSlidingDictionarySize(),
+                            current.entry.getGeneralPurposeBit().getNumberOfShannonFanoTrees(),
+                            bis);
+                    } catch (final IllegalArgumentException ex) {
+                        throw new IOException("bad IMPLODE data", ex);
+                    }
                     break;
                 case BZIP2:
                     current.in = new BZip2CompressorInputStream(bis);
@@ -390,9 +408,9 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
      * deals with splitting/spanning markers that may prefix the first
      * LFH.
      */
-    private void readFirstLocalFileHeader(final byte[] lfh) throws IOException {
-        readFully(lfh);
-        final ZipLong sig = new ZipLong(lfh);
+    private void readFirstLocalFileHeader() throws IOException {
+        readFully(lfhBuf);
+        final ZipLong sig = new ZipLong(lfhBuf);
 
         if (!skipSplitSig && sig.equals(ZipLong.DD_SIG)) {
             throw new UnsupportedZipFeatureException(UnsupportedZipFeatureException.Feature.SPLITTING);
@@ -403,8 +421,8 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
             // Just skip over the marker.
             final byte[] missedLfhBytes = new byte[4];
             readFully(missedLfhBytes);
-            System.arraycopy(lfh, 4, lfh, 0, LFH_LEN - 4);
-            System.arraycopy(missedLfhBytes, 0, lfh, LFH_LEN - 4, 4);
+            System.arraycopy(lfhBuf, 4, lfhBuf, 0, LFH_LEN - 4);
+            System.arraycopy(missedLfhBytes, 0, lfhBuf, LFH_LEN - 4, 4);
         }
     }
 
@@ -413,18 +431,40 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
      * information from it if sizes are 0xFFFFFFFF and the entry
      * doesn't use a data descriptor.
      */
-    private void processZip64Extra(final ZipLong size, final ZipLong cSize) {
-        final Zip64ExtendedInformationExtraField z64 =
-            (Zip64ExtendedInformationExtraField)
+    private void processZip64Extra(final ZipLong size, final ZipLong cSize) throws ZipException {
+        final ZipExtraField extra =
             current.entry.getExtraField(Zip64ExtendedInformationExtraField.HEADER_ID);
+        if (extra != null && !(extra instanceof Zip64ExtendedInformationExtraField)) {
+            throw new ZipException("archive contains unparseable zip64 extra field");
+        }
+        final Zip64ExtendedInformationExtraField z64 =
+            (Zip64ExtendedInformationExtraField) extra;
         current.usesZip64 = z64 != null;
         if (!current.hasDataDescriptor) {
             if (z64 != null // same as current.usesZip64 but avoids NPE warning
                     && (ZipLong.ZIP64_MAGIC.equals(cSize) || ZipLong.ZIP64_MAGIC.equals(size)) ) {
-                current.entry.setCompressedSize(z64.getCompressedSize().getLongValue());
-                current.entry.setSize(z64.getSize().getLongValue());
+                if (z64.getCompressedSize() == null || z64.getSize() == null) {
+                    // avoid NPE if it's a corrupted zip archive
+                    throw new ZipException("archive contains corrupted zip64 extra field");
+                }
+                long s = z64.getCompressedSize().getLongValue();
+                if (s < 0) {
+                    throw new ZipException("broken archive, entry with negative compressed size");
+                }
+                current.entry.setCompressedSize(s);
+                s = z64.getSize().getLongValue();
+                if (s < 0) {
+                    throw new ZipException("broken archive, entry with negative size");
+                }
+                current.entry.setSize(s);
             } else if (cSize != null && size != null) {
+                if (cSize.getValue() < 0) {
+                    throw new ZipException("broken archive, entry with negative compressed size");
+                }
                 current.entry.setCompressedSize(cSize.getValue());
+                if (size.getValue() < 0) {
+                    throw new ZipException("broken archive, entry with negative size");
+                }
                 current.entry.setSize(size.getValue());
             }
         }
@@ -481,7 +521,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                     current.entry);
         }
 
-        int read;
+        final int read;
         if (current.entry.getMethod() == ZipArchiveOutputStream.STORED) {
             read = readStored(buffer, offset, length);
         } else if (current.entry.getMethod() == ZipArchiveOutputStream.DEFLATED) {
@@ -511,19 +551,23 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
     public long getCompressedCount() {
         if (current.entry.getMethod() == ZipArchiveOutputStream.STORED) {
             return current.bytesRead;
-        } else if (current.entry.getMethod() == ZipArchiveOutputStream.DEFLATED) {
-            return getBytesInflated();
-        } else if (current.entry.getMethod() == ZipMethod.UNSHRINKING.getCode()) {
-            return ((UnshrinkingInputStream) current.in).getCompressedCount();
-        } else if (current.entry.getMethod() == ZipMethod.IMPLODING.getCode()) {
-            return ((ExplodingInputStream) current.in).getCompressedCount();
-        } else if (current.entry.getMethod() == ZipMethod.ENHANCED_DEFLATED.getCode()) {
-            return ((Deflate64CompressorInputStream) current.in).getCompressedCount();
-        } else if (current.entry.getMethod() == ZipMethod.BZIP2.getCode()) {
-            return ((BZip2CompressorInputStream) current.in).getCompressedCount();
-        } else {
-            return -1;
         }
+        if (current.entry.getMethod() == ZipArchiveOutputStream.DEFLATED) {
+            return getBytesInflated();
+        }
+        if (current.entry.getMethod() == ZipMethod.UNSHRINKING.getCode()) {
+            return ((UnshrinkingInputStream) current.in).getCompressedCount();
+        }
+        if (current.entry.getMethod() == ZipMethod.IMPLODING.getCode()) {
+            return ((ExplodingInputStream) current.in).getCompressedCount();
+        }
+        if (current.entry.getMethod() == ZipMethod.ENHANCED_DEFLATED.getCode()) {
+            return ((Deflate64CompressorInputStream) current.in).getCompressedCount();
+        }
+        if (current.entry.getMethod() == ZipMethod.BZIP2.getCode()) {
+            return ((BZip2CompressorInputStream) current.in).getCompressedCount();
+        }
+        return -1;
     }
 
     /**
@@ -582,11 +626,13 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         if (read <= 0) {
             if (inf.finished()) {
                 return -1;
-            } else if (inf.needsDictionary()) {
+            }
+            if (inf.needsDictionary()) {
                 throw new ZipException("This archive needs a preset dictionary"
                                        + " which is not supported by Commons"
                                        + " Compress.");
-            } else if (read == -1) {
+            }
+            if (read == -1) {
                 throw new IOException("Truncated ZIP file");
             }
         }
@@ -834,6 +880,15 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         }
     }
 
+    private byte[] readRange(int len) throws IOException {
+        final byte[] ret = IOUtils.readRange(in, len);
+        count(ret.length);
+        if (ret.length < len) {
+            throw new EOFException();
+        }
+        return ret;
+    }
+
     private void readDataDescriptor() throws IOException {
         readFully(wordBuf);
         ZipLong val = new ZipLong(wordBuf);
@@ -859,11 +914,27 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         final ZipLong potentialSig = new ZipLong(twoDwordBuf, DWORD);
         if (potentialSig.equals(ZipLong.CFH_SIG) || potentialSig.equals(ZipLong.LFH_SIG)) {
             pushback(twoDwordBuf, DWORD, DWORD);
-            current.entry.setCompressedSize(ZipLong.getValue(twoDwordBuf));
-            current.entry.setSize(ZipLong.getValue(twoDwordBuf, WORD));
+            long size = ZipLong.getValue(twoDwordBuf);
+            if (size < 0) {
+                throw new ZipException("broken archive, entry with negative compressed size");
+            }
+            current.entry.setCompressedSize(size);
+            size = ZipLong.getValue(twoDwordBuf, WORD);
+            if (size < 0) {
+                throw new ZipException("broken archive, entry with negative size");
+            }
+            current.entry.setSize(size);
         } else {
-            current.entry.setCompressedSize(ZipEightByteInteger.getLongValue(twoDwordBuf));
-            current.entry.setSize(ZipEightByteInteger.getLongValue(twoDwordBuf, DWORD));
+            long size = ZipEightByteInteger.getLongValue(twoDwordBuf);
+            if (size < 0) {
+                throw new ZipException("broken archive, entry with negative compressed size");
+            }
+            current.entry.setCompressedSize(size);
+            size = ZipEightByteInteger.getLongValue(twoDwordBuf, DWORD);
+            if (size < 0) {
+                throw new ZipException("broken archive, entry with negative size");
+            }
+            current.entry.setSize(size);
         }
     }
 
@@ -977,7 +1048,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                 int expectDDPos = i;
                 if (i >= expectedDDLen &&
                     (buf.array()[i + 2] == LFH[2] && buf.array()[i + 3] == LFH[3])
-                    || (buf.array()[i] == CFH[2] && buf.array()[i + 3] == CFH[3])) {
+                    || (buf.array()[i + 2] == CFH[2] && buf.array()[i + 3] == CFH[3])) {
                     // found a LFH or CFH:
                     expectDDPos = i - expectedDDLen;
                     done = true;
@@ -1051,19 +1122,28 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         // skip over central directory. One LFH has been read too much
         // already.  The calculation discounts file names and extra
         // data so it will be too short.
-        realSkip((long) entriesRead * CFH_LEN - LFH_LEN);
-        findEocdRecord();
-        realSkip((long) ZipFile.MIN_EOCD_SIZE - WORD /* signature */ - SHORT /* comment len */);
-        readFully(shortBuf);
-        // file comment
-        realSkip(ZipShort.getValue(shortBuf));
+        if (entriesRead > 0) {
+            realSkip((long) entriesRead * CFH_LEN - LFH_LEN);
+            final boolean foundEocd = findEocdRecord();
+            if (foundEocd) {
+                realSkip((long) ZipFile.MIN_EOCD_SIZE - WORD /* signature */ - SHORT /* comment len */);
+                readFully(shortBuf);
+                // file comment
+                final int commentLen = ZipShort.getValue(shortBuf);
+                if (commentLen >= 0) {
+                    realSkip(commentLen);
+                    return;
+                }
+            }
+        }
+        throw new IOException("Truncated ZIP file");
     }
 
     /**
      * Reads forward until the signature of the &quot;End of central
      * directory&quot; record is found.
      */
-    private void findEocdRecord() throws IOException {
+    private boolean findEocdRecord() throws IOException {
         int currentByte = -1;
         boolean skipReadCall = false;
         while (skipReadCall || (currentByte = readOneByte()) > -1) {
@@ -1088,12 +1168,15 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                 continue;
             }
             currentByte = readOneByte();
-            if (currentByte == -1
-                || currentByte == ZipArchiveOutputStream.EOCD_SIG[3]) {
+            if (currentByte == -1) {
                 break;
             }
+            if (currentByte == ZipArchiveOutputStream.EOCD_SIG[3]) {
+                return true;
+            }
             skipReadCall = isFirstByteOfEocdSig(currentByte);
         }
+        return false;
     }
 
     /**
@@ -1157,24 +1240,24 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
      *
      * @see <a href="https://source.android.com/security/apksigning/v2">https://source.android.com/security/apksigning/v2</a>
      */
-    private boolean isApkSigningBlock(byte[] suspectLocalFileHeader) throws IOException {
+    private boolean isApkSigningBlock(final byte[] suspectLocalFileHeader) throws IOException {
         // length of block excluding the size field itself
-        BigInteger len = ZipEightByteInteger.getValue(suspectLocalFileHeader);
+        final BigInteger len = ZipEightByteInteger.getValue(suspectLocalFileHeader);
         // LFH has already been read and all but the first eight bytes contain (part of) the APK signing block,
         // also subtract 16 bytes in order to position us at the magic string
         BigInteger toSkip = len.add(BigInteger.valueOf(DWORD - suspectLocalFileHeader.length
             - (long) APK_SIGNING_BLOCK_MAGIC.length));
-        byte[] magic = new byte[APK_SIGNING_BLOCK_MAGIC.length];
+        final byte[] magic = new byte[APK_SIGNING_BLOCK_MAGIC.length];
 
         try {
             if (toSkip.signum() < 0) {
                 // suspectLocalFileHeader contains the start of suspect magic string
-                int off = suspectLocalFileHeader.length + toSkip.intValue();
+                final int off = suspectLocalFileHeader.length + toSkip.intValue();
                 // length was shorter than magic length
                 if (off < DWORD) {
                     return false;
                 }
-                int bytesInBuffer = Math.abs(toSkip.intValue());
+                final int bytesInBuffer = Math.abs(toSkip.intValue());
                 System.arraycopy(suspectLocalFileHeader, off, magic, 0, Math.min(bytesInBuffer, magic.length));
                 if (bytesInBuffer < magic.length) {
                     readFully(magic, bytesInBuffer);
@@ -1187,7 +1270,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                 realSkip(toSkip.longValue());
                 readFully(magic);
             }
-        } catch (EOFException ex) { //NOSONAR
+        } catch (final EOFException ex) { //NOSONAR
             // length was invalid
             return false;
         }
@@ -1253,7 +1336,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         private final long max;
 
         /** the number of bytes already returned */
-        private long pos = 0;
+        private long pos;
 
         /**
          * Creates a new <code>BoundedInputStream</code> that wraps the given input
