@@ -45,6 +45,8 @@ import java.util.zip.ZipException;
 import org.apache.commons.compress.archivers.EntryStreamOffsets;
 import org.apache.commons.compress.compressors.bzip2.BZip2CompressorInputStream;
 import org.apache.commons.compress.compressors.deflate64.Deflate64CompressorInputStream;
+import org.apache.commons.compress.utils.BoundedArchiveInputStream;
+import org.apache.commons.compress.utils.BoundedSeekableByteChannelInputStream;
 import org.apache.commons.compress.utils.CountingInputStream;
 import org.apache.commons.compress.utils.IOUtils;
 import org.apache.commons.compress.utils.InputStreamStatistics;
@@ -144,7 +146,7 @@ public class ZipFile implements Closeable {
     private volatile boolean closed = true;
 
     /**
-     * Whether the zip archive is a splite zip archive
+     * Whether the zip archive is a split zip archive
      */
     private final boolean isSplitZipArchive;
 
@@ -158,6 +160,9 @@ public class ZipFile implements Closeable {
     private final ByteBuffer cfhBbuf = ByteBuffer.wrap(cfhBuf);
     private final ByteBuffer shortBbuf = ByteBuffer.wrap(shortBuf);
 
+    private long centralDirectoryStartDiskNumber, centralDirectoryStartRelativeOffset;
+    private long centralDirectoryStartOffset;
+
     /**
      * Opens the given file for reading, assuming "UTF8" for file names.
      *
@@ -374,6 +379,8 @@ public class ZipFile implements Closeable {
             }
             fillNameMap();
             success = true;
+        } catch (final IOException e) {
+            throw new IOException("Error on ZipFile " + archiveName, e);
         } finally {
             closed = !success;
             if (!success && closeOnError) {
@@ -437,7 +444,7 @@ public class ZipFile implements Closeable {
      * @since 1.1
      */
     public Enumeration<ZipArchiveEntry> getEntriesInPhysicalOrder() {
-        final ZipArchiveEntry[] allEntries = entries.toArray(new ZipArchiveEntry[entries.size()]);
+        final ZipArchiveEntry[] allEntries = entries.toArray(ZipArchiveEntry.EMPTY_ZIP_ARCHIVE_ENTRY_ARRAY);
         Arrays.sort(allEntries, offsetComparator);
         return Collections.enumeration(Arrays.asList(allEntries));
     }
@@ -471,7 +478,7 @@ public class ZipFile implements Closeable {
     public Iterable<ZipArchiveEntry> getEntries(final String name) {
         final List<ZipArchiveEntry> entriesOfThatName = nameMap.get(name);
         return entriesOfThatName != null ? entriesOfThatName
-            : Collections.<ZipArchiveEntry>emptyList();
+            : Collections.emptyList();
     }
 
     /**
@@ -484,7 +491,7 @@ public class ZipFile implements Closeable {
      * @since 1.6
      */
     public Iterable<ZipArchiveEntry> getEntriesInPhysicalOrder(final String name) {
-        ZipArchiveEntry[] entriesOfThatName = new ZipArchiveEntry[0];
+        ZipArchiveEntry[] entriesOfThatName = ZipArchiveEntry.EMPTY_ZIP_ARCHIVE_ENTRY_ARRAY;
         if (nameMap.containsKey(name)) {
             entriesOfThatName = nameMap.get(name).toArray(entriesOfThatName);
             Arrays.sort(entriesOfThatName, offsetComparator);
@@ -575,8 +582,12 @@ public class ZipFile implements Closeable {
             case UNSHRINKING:
                 return new UnshrinkingInputStream(is);
             case IMPLODING:
-                return new ExplodingInputStream(ze.getGeneralPurposeBit().getSlidingDictionarySize(),
-                        ze.getGeneralPurposeBit().getNumberOfShannonFanoTrees(), is);
+                try {
+                    return new ExplodingInputStream(ze.getGeneralPurposeBit().getSlidingDictionarySize(),
+                            ze.getGeneralPurposeBit().getNumberOfShannonFanoTrees(), is);
+                } catch (final IllegalArgumentException ex) {
+                    throw new IOException("bad IMPLODE data", ex);
+                }
             case DEFLATED:
                 final Inflater inflater = new Inflater(true);
                 // Inflater with nowrap=true has this odd contract for a zero padding
@@ -700,6 +711,7 @@ public class ZipFile implements Closeable {
             new HashMap<>();
 
         positionAtCentralDirectory();
+        centralDirectoryStartOffset = archive.position();
 
         wordBbuf.rewind();
         IOUtils.readFully(archive, wordBbuf);
@@ -767,20 +779,37 @@ public class ZipFile implements Closeable {
         ze.setCrc(ZipLong.getValue(cfhBuf, off));
         off += WORD;
 
-        ze.setCompressedSize(ZipLong.getValue(cfhBuf, off));
+        long size = ZipLong.getValue(cfhBuf, off);
+        if (size < 0) {
+            throw new IOException("broken archive, entry with negative compressed size");
+        }
+        ze.setCompressedSize(size);
         off += WORD;
 
-        ze.setSize(ZipLong.getValue(cfhBuf, off));
+        size = ZipLong.getValue(cfhBuf, off);
+        if (size < 0) {
+            throw new IOException("broken archive, entry with negative size");
+        }
+        ze.setSize(size);
         off += WORD;
 
         final int fileNameLen = ZipShort.getValue(cfhBuf, off);
         off += SHORT;
+        if (fileNameLen < 0) {
+            throw new IOException("broken archive, entry with negative fileNameLen");
+        }
 
         final int extraLen = ZipShort.getValue(cfhBuf, off);
         off += SHORT;
+        if (extraLen < 0) {
+            throw new IOException("broken archive, entry with negative extraLen");
+        }
 
         final int commentLen = ZipShort.getValue(cfhBuf, off);
         off += SHORT;
+        if (commentLen < 0) {
+            throw new IOException("broken archive, entry with negative commentLen");
+        }
 
         ze.setDiskNumberStart(ZipShort.getValue(cfhBuf, off));
         off += SHORT;
@@ -791,8 +820,10 @@ public class ZipFile implements Closeable {
         ze.setExternalAttributes(ZipLong.getValue(cfhBuf, off));
         off += WORD;
 
-        final byte[] fileName = new byte[fileNameLen];
-        IOUtils.readFully(archive, ByteBuffer.wrap(fileName));
+        final byte[] fileName = IOUtils.readRange(archive, fileNameLen);
+        if (fileName.length < fileNameLen) {
+            throw new EOFException();
+        }
         ze.setName(entryEncoding.decode(fileName), fileName);
 
         // LFH offset,
@@ -800,14 +831,25 @@ public class ZipFile implements Closeable {
         // data offset will be filled later
         entries.add(ze);
 
-        final byte[] cdExtraData = new byte[extraLen];
-        IOUtils.readFully(archive, ByteBuffer.wrap(cdExtraData));
-        ze.setCentralDirectoryExtra(cdExtraData);
+        final byte[] cdExtraData = IOUtils.readRange(archive, extraLen);
+        if (cdExtraData.length < extraLen) {
+            throw new EOFException();
+        }
+        try {
+            ze.setCentralDirectoryExtra(cdExtraData);
+        } catch (RuntimeException ex) {
+            final ZipException z = new ZipException("Invalid extra data in entry " + ze.getName());
+            z.initCause(ex);
+            throw z;
+        }
 
         setSizesAndOffsetFromZip64Extra(ze);
+        sanityCheckLFHOffset(ze);
 
-        final byte[] comment = new byte[commentLen];
-        IOUtils.readFully(archive, ByteBuffer.wrap(comment));
+        final byte[] comment = IOUtils.readRange(archive, commentLen);
+        if (comment.length < commentLen) {
+            throw new EOFException();
+        }
         ze.setComment(entryEncoding.decode(comment));
 
         if (!hasUTF8Flag && useUnicodeExtraFields) {
@@ -817,6 +859,28 @@ public class ZipFile implements Closeable {
         ze.setStreamContiguous(true);
     }
 
+    private void sanityCheckLFHOffset(final ZipArchiveEntry ze) throws IOException {
+        if (ze.getDiskNumberStart() < 0) {
+            throw new IOException("broken archive, entry with negative disk number");
+        }
+        if (ze.getLocalHeaderOffset() < 0) {
+            throw new IOException("broken archive, entry with negative local file header offset");
+        }
+        if (isSplitZipArchive) {
+            if (ze.getDiskNumberStart() > centralDirectoryStartDiskNumber) {
+                throw new IOException("local file header for " + ze.getName() + " starts on a later disk than central directory");
+            }
+            if (ze.getDiskNumberStart() == centralDirectoryStartDiskNumber
+                && ze.getLocalHeaderOffset() > centralDirectoryStartRelativeOffset) {
+                throw new IOException("local file header for " + ze.getName() + " starts after central directory");
+            }
+        } else {
+            if (ze.getLocalHeaderOffset() > centralDirectoryStartOffset) {
+                throw new IOException("local file header for " + ze.getName() + " starts after central directory");
+            }
+        }
+    }
+
     /**
      * If the entry holds a Zip64 extended information extra field,
      * read sizes from there if the entry's sizes are set to
@@ -831,9 +895,13 @@ public class ZipFile implements Closeable {
      */
     private void setSizesAndOffsetFromZip64Extra(final ZipArchiveEntry ze)
         throws IOException {
-        final Zip64ExtendedInformationExtraField z64 =
-            (Zip64ExtendedInformationExtraField)
+        final ZipExtraField extra =
             ze.getExtraField(Zip64ExtendedInformationExtraField.HEADER_ID);
+        if (extra != null && !(extra instanceof Zip64ExtendedInformationExtraField)) {
+            throw new ZipException("archive contains unparseable zip64 extra field");
+        }
+        final Zip64ExtendedInformationExtraField z64 =
+            (Zip64ExtendedInformationExtraField) extra;
         if (z64 != null) {
             final boolean hasUncompressedSize = ze.getSize() == ZIP64_MAGIC;
             final boolean hasCompressedSize = ze.getCompressedSize() == ZIP64_MAGIC;
@@ -846,13 +914,21 @@ public class ZipFile implements Closeable {
                                             hasDiskStart);
 
             if (hasUncompressedSize) {
-                ze.setSize(z64.getSize().getLongValue());
+                final long size = z64.getSize().getLongValue();
+                if (size < 0) {
+                    throw new IOException("broken archive, entry with negative size");
+                }
+                ze.setSize(size);
             } else if (hasCompressedSize) {
                 z64.setSize(new ZipEightByteInteger(ze.getSize()));
             }
 
             if (hasCompressedSize) {
-                ze.setCompressedSize(z64.getCompressedSize().getLongValue());
+                final long size = z64.getCompressedSize().getLongValue();
+                if (size < 0) {
+                    throw new IOException("broken archive, entry with negative compressed size");
+                }
+                ze.setCompressedSize(size);
             } else if (hasUncompressedSize) {
                 z64.setCompressedSize(new ZipEightByteInteger(ze.getCompressedSize()));
             }
@@ -1081,21 +1157,23 @@ public class ZipFile implements Closeable {
                     - WORD /* signature has already been read */);
             wordBbuf.rewind();
             IOUtils.readFully(archive, wordBbuf);
-            final long diskNumberOfCFD = ZipLong.getValue(wordBuf);
+            centralDirectoryStartDiskNumber = ZipLong.getValue(wordBuf);
 
             skipBytes(ZIP64_EOCD_CFD_LOCATOR_RELATIVE_OFFSET);
 
             dwordBbuf.rewind();
             IOUtils.readFully(archive, dwordBbuf);
-            final long relativeOffsetOfCFD = ZipEightByteInteger.getLongValue(dwordBuf);
+            centralDirectoryStartRelativeOffset = ZipEightByteInteger.getLongValue(dwordBuf);
             ((ZipSplitReadOnlySeekableByteChannel) archive)
-                .position(diskNumberOfCFD, relativeOffsetOfCFD);
+                .position(centralDirectoryStartDiskNumber, centralDirectoryStartRelativeOffset);
         } else {
             skipBytes(ZIP64_EOCD_CFD_LOCATOR_OFFSET
                     - WORD /* signature has already been read */);
             dwordBbuf.rewind();
             IOUtils.readFully(archive, dwordBbuf);
-            archive.position(ZipEightByteInteger.getLongValue(dwordBuf));
+            centralDirectoryStartDiskNumber = 0;
+            centralDirectoryStartRelativeOffset = ZipEightByteInteger.getLongValue(dwordBuf);
+            archive.position(centralDirectoryStartRelativeOffset);
         }
     }
 
@@ -1112,20 +1190,22 @@ public class ZipFile implements Closeable {
             skipBytes(CFD_DISK_OFFSET);
             shortBbuf.rewind();
             IOUtils.readFully(archive, shortBbuf);
-            final int diskNumberOfCFD = ZipShort.getValue(shortBuf);
+            centralDirectoryStartDiskNumber = ZipShort.getValue(shortBuf);
 
             skipBytes(CFD_LOCATOR_RELATIVE_OFFSET);
 
             wordBbuf.rewind();
             IOUtils.readFully(archive, wordBbuf);
-            final long relativeOffsetOfCFD = ZipLong.getValue(wordBuf);
+            centralDirectoryStartRelativeOffset = ZipLong.getValue(wordBuf);
             ((ZipSplitReadOnlySeekableByteChannel) archive)
-                .position(diskNumberOfCFD, relativeOffsetOfCFD);
+                .position(centralDirectoryStartDiskNumber, centralDirectoryStartRelativeOffset);
         } else {
             skipBytes(CFD_LOCATOR_OFFSET);
             wordBbuf.rewind();
             IOUtils.readFully(archive, wordBbuf);
-            archive.position(ZipLong.getValue(wordBuf));
+            centralDirectoryStartDiskNumber = 0;
+            centralDirectoryStartRelativeOffset = ZipLong.getValue(wordBuf);
+            archive.position(centralDirectoryStartRelativeOffset);
         }
     }
 
@@ -1161,7 +1241,7 @@ public class ZipFile implements Closeable {
                     wordBbuf.rewind();
                     IOUtils.readFully(archive, wordBbuf);
                     wordBbuf.flip();
-                } catch (EOFException ex) { // NOSONAR
+                } catch (final EOFException ex) { // NOSONAR
                     break;
                 }
                 int curr = wordBbuf.get();
@@ -1191,8 +1271,8 @@ public class ZipFile implements Closeable {
      * skipping failed.
      */
     private void skipBytes(final int count) throws IOException {
-        long currentPosition = archive.position();
-        long newPosition = currentPosition + count;
+        final long currentPosition = archive.position();
+        final long newPosition = currentPosition + count;
         if (newPosition > archive.size()) {
             throw new EOFException();
         }
@@ -1228,13 +1308,21 @@ public class ZipFile implements Closeable {
             // entries is filled in populateFromCentralDirectory and
             // never modified
             final Entry ze = (Entry) zipArchiveEntry;
-            int[] lens = setDataOffset(ze);
+            final int[] lens = setDataOffset(ze);
             final int fileNameLen = lens[0];
             final int extraFieldLen = lens[1];
             skipBytes(fileNameLen);
-            final byte[] localExtraData = new byte[extraFieldLen];
-            IOUtils.readFully(archive, ByteBuffer.wrap(localExtraData));
-            ze.setExtra(localExtraData);
+            final byte[] localExtraData = IOUtils.readRange(archive, extraFieldLen);
+            if (localExtraData.length < extraFieldLen) {
+                throw new EOFException();
+            }
+            try {
+                ze.setExtra(localExtraData);
+            } catch (RuntimeException ex) {
+                final ZipException z = new ZipException("Invalid extra data in entry " + ze.getName());
+                z.initCause(ex);
+                throw z;
+            }
 
             if (entriesWithoutUTF8Flag.containsKey(ze)) {
                 final NameAndComment nc = entriesWithoutUTF8Flag.get(ze);
@@ -1249,16 +1337,12 @@ public class ZipFile implements Closeable {
             // entries is filled in populateFromCentralDirectory and
             // never modified
             final String name = ze.getName();
-            LinkedList<ZipArchiveEntry> entriesOfThatName = nameMap.get(name);
-            if (entriesOfThatName == null) {
-                entriesOfThatName = new LinkedList<>();
-                nameMap.put(name, entriesOfThatName);
-            }
+            LinkedList<ZipArchiveEntry> entriesOfThatName = nameMap.computeIfAbsent(name, k -> new LinkedList<>());
             entriesOfThatName.addLast(ze);
         }
     }
 
-    private int[] setDataOffset(ZipArchiveEntry ze) throws IOException {
+    private int[] setDataOffset(final ZipArchiveEntry ze) throws IOException {
         long offset = ze.getLocalHeaderOffset();
         if (isSplitZipArchive) {
             ((ZipSplitReadOnlySeekableByteChannel) archive)
@@ -1277,11 +1361,14 @@ public class ZipFile implements Closeable {
         final int extraFieldLen = ZipShort.getValue(shortBuf);
         ze.setDataOffset(offset + LFH_OFFSET_FOR_FILENAME_LENGTH
                          + SHORT + SHORT + fileNameLen + extraFieldLen);
+        if (ze.getDataOffset() + ze.getCompressedSize() > centralDirectoryStartOffset) {
+            throw new IOException("data for " + ze.getName() + " overlaps with central directory.");
+        }
         return new int[] { fileNameLen, extraFieldLen };
     }
 
-    private long getDataOffset(ZipArchiveEntry ze) throws IOException {
-        long s = ze.getDataOffset();
+    private long getDataOffset(final ZipArchiveEntry ze) throws IOException {
+        final long s = ze.getDataOffset();
         if (s == EntryStreamOffsets.OFFSET_UNKNOWN) {
             setDataOffset(ze);
             return ze.getDataOffset();
@@ -1304,82 +1391,14 @@ public class ZipFile implements Closeable {
      * Creates new BoundedInputStream, according to implementation of
      * underlying archive channel.
      */
-    private BoundedInputStream createBoundedInputStream(long start, long remaining) {
+    private BoundedArchiveInputStream createBoundedInputStream(final long start, final long remaining) {
+        if (start < 0 || remaining < 0 || start + remaining < start) {
+            throw new IllegalArgumentException("Corrupted archive, stream boundaries"
+                + " are out of range");
+        }
         return archive instanceof FileChannel ?
             new BoundedFileChannelInputStream(start, remaining) :
-            new BoundedInputStream(start, remaining);
-    }
-
-    /**
-     * InputStream that delegates requests to the underlying
-     * SeekableByteChannel, making sure that only bytes from a certain
-     * range can be read.
-     */
-    private class BoundedInputStream extends InputStream {
-        private ByteBuffer singleByteBuffer;
-        private final long end;
-        private long loc;
-
-        BoundedInputStream(final long start, final long remaining) {
-            this.end = start+remaining;
-            if (this.end < start) {
-                // check for potential vulnerability due to overflow
-                throw new IllegalArgumentException("Invalid length of stream at offset="+start+", length="+remaining);
-            }
-            loc = start;
-        }
-
-        @Override
-        public synchronized int read() throws IOException {
-            if (loc >= end) {
-                return -1;
-            }
-            if (singleByteBuffer == null) {
-                singleByteBuffer = ByteBuffer.allocate(1);
-            }
-            else {
-                singleByteBuffer.rewind();
-            }
-            int read = read(loc, singleByteBuffer);
-            if (read < 0) {
-                return read;
-            }
-            loc++;
-            return singleByteBuffer.get() & 0xff;
-        }
-
-        @Override
-        public synchronized int read(final byte[] b, final int off, int len) throws IOException {
-            if (len <= 0) {
-                return 0;
-            }
-
-            if (len > end-loc) {
-                if (loc >= end) {
-                    return -1;
-                }
-                len = (int)(end-loc);
-            }
-
-            ByteBuffer buf;
-            buf = ByteBuffer.wrap(b, off, len);
-            int ret = read(loc, buf);
-            if (ret > 0) {
-                loc += ret;
-                return ret;
-            }
-            return ret;
-        }
-
-        protected int read(long pos, ByteBuffer buf) throws IOException {
-            int read;
-            synchronized (archive) {
-                archive.position(pos);
-                read = archive.read(buf);
-            }
-            buf.flip();
-            return read;
-        }
+            new BoundedSeekableByteChannelInputStream(start, remaining, archive);
     }
 
     /**
@@ -1388,17 +1407,17 @@ public class ZipFile implements Closeable {
      * file channel and therefore performs significantly faster in
      * concurrent environment.
      */
-    private class BoundedFileChannelInputStream extends BoundedInputStream {
+    private class BoundedFileChannelInputStream extends BoundedArchiveInputStream {
         private final FileChannel archive;
 
         BoundedFileChannelInputStream(final long start, final long remaining) {
             super(start, remaining);
-            archive = (FileChannel)ZipFile.this.archive;
+            archive = (FileChannel) ZipFile.this.archive;
         }
 
         @Override
-        protected int read(long pos, ByteBuffer buf) throws IOException {
-            int read = archive.read(buf, pos);
+        protected int read(final long pos, final ByteBuffer buf) throws IOException {
+            final int read = archive.read(buf, pos);
             buf.flip();
             return read;
         }
@@ -1422,32 +1441,8 @@ public class ZipFile implements Closeable {
      * @since 1.1
      */
     private final Comparator<ZipArchiveEntry> offsetComparator =
-        new Comparator<ZipArchiveEntry>() {
-        @Override
-        public int compare(final ZipArchiveEntry e1, final ZipArchiveEntry e2) {
-            if (e1 == e2) {
-                return 0;
-            }
-
-            final Entry ent1 = e1 instanceof Entry ? (Entry) e1 : null;
-            final Entry ent2 = e2 instanceof Entry ? (Entry) e2 : null;
-            if (ent1 == null) {
-                return 1;
-            }
-            if (ent2 == null) {
-                return -1;
-            }
-
-            // disk number is prior to relative offset
-            final long diskNumberStartVal = ent1.getDiskNumberStart() - ent2.getDiskNumberStart();
-            if (diskNumberStartVal != 0) {
-                return diskNumberStartVal < 0 ? -1 : +1;
-            }
-            final long val = (ent1.getLocalHeaderOffset()
-                        - ent2.getLocalHeaderOffset());
-            return val == 0 ? 0 : val < 0 ? -1 : +1;
-        }
-    };
+        Comparator.comparingLong(ZipArchiveEntry::getDiskNumberStart)
+            .thenComparingLong(ZipArchiveEntry::getLocalHeaderOffset);
 
     /**
      * Extends ZipArchiveEntry to store the offset within the archive.
@@ -1480,7 +1475,7 @@ public class ZipFile implements Closeable {
     }
 
     private static class StoredStatisticsStream extends CountingInputStream implements InputStreamStatistics {
-        StoredStatisticsStream(InputStream in) {
+        StoredStatisticsStream(final InputStream in) {
             super(in);
         }
 
