@@ -25,24 +25,24 @@ import java.util.List;
  * Stores the results of an performed ChangeSet operation.
  */
 public class ChangeSetResults {
-    private final List<String> addedFromChangeSet = new ArrayList<>();
-    private final List<String> addedFromStream = new ArrayList<>();
-    private final List<String> deleted = new ArrayList<>();
+    private final List<String> addedFromChangeSet = new ArrayList<String>();
+    private final List<String> addedFromStream = new ArrayList<String>();
+    private final List<String> deleted = new ArrayList<String>();
 
     /**
      * Adds the filename of a recently deleted file to the result list.
      * @param fileName the file which has been deleted
      */
-    void deleted(final String fileName) {
+    void deleted(String fileName) {
         deleted.add(fileName);
     }
 
     /**
-     * Adds the name of a file to the result list which has been
+     * Adds the name of a file to the result list which has been 
      * copied from the source stream to the target stream.
      * @param fileName the file name which has been added from the original stream
      */
-    void addedFromStream(final String fileName) {
+    void addedFromStream(String fileName) {
         addedFromStream.add(fileName);
     }
 
@@ -51,7 +51,7 @@ public class ChangeSetResults {
      * copied from the changeset to the target stream
      * @param fileName the name of the file
      */
-    void addedFromChangeSet(final String fileName) {
+    void addedFromChangeSet(String fileName) {
         addedFromChangeSet.add(fileName);
     }
 
@@ -84,7 +84,10 @@ public class ChangeSetResults {
      * @param filename the filename to check
      * @return true, if this filename already has been added
      */
-    boolean hasBeenAdded(final String filename) {
-        return addedFromChangeSet.contains(filename) || addedFromStream.contains(filename);
+    boolean hasBeenAdded(String filename) {
+        if(addedFromChangeSet.contains(filename) || addedFromStream.contains(filename)) {
+            return true;
+        } 
+        return false;
     }
 }
