@@ -100,7 +100,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
 
     /**
      * Constructs a new BZip2CompressorInputStream which decompresses bytes
-     * read from the specified stream. This doesn't suppprt decompressing
+     * read from the specified stream. This doesn't support decompressing
      * concatenated .bz2 files.
      *
      * @param in the InputStream from which this object should be created
@@ -236,8 +236,8 @@ public class BZip2CompressorInputStream extends CompressorInputStream
         }
     }
 
-    private int readNextByte(BitInputStream in) throws IOException {
-        long b = in.readBits(8);
+    private int readNextByte(final BitInputStream in) throws IOException {
+        final long b = in.readBits(8);
         return (int) b;
     }
 
@@ -276,7 +276,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
     }
 
     private void initBlock() throws IOException {
-        BitInputStream bin = this.bin;
+        final BitInputStream bin = this.bin;
         char magic0;
         char magic1;
         char magic2;
@@ -320,7 +320,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
         this.storedBlockCRC = bsGetInt(bin);
         this.blockRandomised = bsR(bin, 1) == 1;
 
-        /**
+        /*
          * Allocate data here instead in constructor, so we do not allocate
          * it if the input file is empty.
          */
@@ -331,7 +331,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
         // currBlockNo++;
         getAndMoveToFrontDecode();
 
-        this.crc.initialiseCRC();
+        this.crc.initializeCRC();
         this.currentState = START_BLOCK_STATE;
     }
 
@@ -387,27 +387,27 @@ public class BZip2CompressorInputStream extends CompressorInputStream
      * @return the requested bits combined into an int
      * @throws IOException
      */
-    private static int bsR(BitInputStream bin, final int n) throws IOException {
-        long thech = bin.readBits(n);
+    private static int bsR(final BitInputStream bin, final int n) throws IOException {
+        final long thech = bin.readBits(n);
         if (thech < 0) {
             throw new IOException("Unexpected end of stream");
         }
         return (int) thech;
     }
 
-    private static boolean bsGetBit(BitInputStream bin) throws IOException {
+    private static boolean bsGetBit(final BitInputStream bin) throws IOException {
         return bsR(bin, 1) != 0;
     }
 
-    private static char bsGetUByte(BitInputStream bin) throws IOException {
+    private static char bsGetUByte(final BitInputStream bin) throws IOException {
         return (char) bsR(bin, 8);
     }
 
-    private static int bsGetInt(BitInputStream bin) throws IOException {
+    private static int bsGetInt(final BitInputStream bin) throws IOException {
         return bsR(bin, 32);
     }
 
-    private static void checkBounds(final int checkVal, final int limitExclusive, String name)
+    private static void checkBounds(final int checkVal, final int limitExclusive, final String name)
         throws IOException {
         if (checkVal < 0) {
             throw new IOException("Corrupted input, " + name + " value negative");
@@ -659,6 +659,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
                     checkBounds(tmp, MAX_ALPHA_SIZE, "zvec");
                     nextSym = perm_zt[tmp];
                 }
+                checkBounds(s, this.data.ll8.length, "s");
 
                 final int yy0 = yy[0];
                 checkBounds(yy0, 256, "yy");
@@ -667,6 +668,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
 
                 final int from = ++lastShadow;
                 lastShadow += s;
+                checkBounds(lastShadow, this.data.ll8.length, "lastShadow");
                 Arrays.fill(ll8, from, lastShadow + 1, ch);
 
                 if (lastShadow >= limitLast) {
@@ -835,28 +837,28 @@ public class BZip2CompressorInputStream extends CompressorInputStream
             this.currentState = RAND_PART_A_STATE;
             this.su_count = 1;
             return setupRandPartA();
-        } else if (++this.su_count >= 4) {
-            this.su_z = (char) (this.data.ll8[this.su_tPos] & 0xff);
-            checkBounds(this.su_tPos, this.data.tt.length, "su_tPos");
-            this.su_tPos = this.data.tt[this.su_tPos];
-            if (this.su_rNToGo == 0) {
-                this.su_rNToGo = Rand.rNums(this.su_rTPos) - 1;
-                if (++this.su_rTPos == 512) {
-                    this.su_rTPos = 0;
-                }
-            } else {
-                this.su_rNToGo--;
-            }
-            this.su_j2 = 0;
-            this.currentState = RAND_PART_C_STATE;
-            if (this.su_rNToGo == 1) {
-                this.su_z ^= 1;
-            }
-            return setupRandPartC();
-        } else {
+        }
+        if (++this.su_count < 4) {
             this.currentState = RAND_PART_A_STATE;
             return setupRandPartA();
         }
+        this.su_z = (char) (this.data.ll8[this.su_tPos] & 0xff);
+        checkBounds(this.su_tPos, this.data.tt.length, "su_tPos");
+        this.su_tPos = this.data.tt[this.su_tPos];
+        if (this.su_rNToGo == 0) {
+            this.su_rNToGo = Rand.rNums(this.su_rTPos) - 1;
+            if (++this.su_rTPos == 512) {
+                this.su_rTPos = 0;
+            }
+        } else {
+            this.su_rNToGo--;
+        }
+        this.su_j2 = 0;
+        this.currentState = RAND_PART_C_STATE;
+        if (this.su_rNToGo == 1) {
+            this.su_z ^= 1;
+        }
+        return setupRandPartC();
     }
 
     private int setupRandPartC() throws IOException {
@@ -875,15 +877,15 @@ public class BZip2CompressorInputStream extends CompressorInputStream
         if (this.su_ch2 != this.su_chPrev) {
             this.su_count = 1;
             return setupNoRandPartA();
-        } else if (++this.su_count >= 4) {
+        }
+        if (++this.su_count >= 4) {
             checkBounds(this.su_tPos, this.data.ll8.length, "su_tPos");
             this.su_z = (char) (this.data.ll8[this.su_tPos] & 0xff);
             this.su_tPos = this.data.tt[this.su_tPos];
             this.su_j2 = 0;
             return setupNoRandPartC();
-        } else {
-            return setupNoRandPartA();
         }
+        return setupNoRandPartA();
     }
 
     private int setupNoRandPartC() throws IOException {
@@ -928,7 +930,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
         // 60798 byte
 
         int[] tt; // 3600000 byte
-        byte[] ll8; // 900000 byte
+        final byte[] ll8; // 900000 byte
 
         // ---------------
         // 4560782 byte
@@ -942,7 +944,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
          * Initializes the {@link #tt} array.
          *
          * This method is called when the required length of the array is known.
-         * I don't initialize it at construction time to avoid unneccessary
+         * I don't initialize it at construction time to avoid unnecessary
          * memory allocation when compressing small files.
          */
         int[] initTT(final int length) {
