@@ -52,59 +52,60 @@ import org.apache.commons.compress.compressors.CompressorOutputStream;
  * &lt;code&gt;65k + (5 * blocksize)&lt;/code&gt;.
  * </pre>
  *
- * <table width="100%" border="1" summary="Memory usage by blocksize">
+ * <table style="width:100%" border="1">
+ * <caption>Memory usage by blocksize</caption>
  * <tr>
  * <th colspan="3">Memory usage by blocksize</th>
  * </tr>
  * <tr>
- * <th align="right">Blocksize</th> <th align="right">Compression<br>
- * memory usage</th> <th align="right">Decompression<br>
+ * <th style="text-align: right">Blocksize</th> <th style="text-align: right">Compression<br>
+ * memory usage</th> <th style="text-align: right">Decompression<br>
  * memory usage</th>
  * </tr>
  * <tr>
- * <td align="right">100k</td>
- * <td align="right">1300k</td>
- * <td align="right">565k</td>
+ * <td style="text-align: right">100k</td>
+ * <td style="text-align: right">1300k</td>
+ * <td style="text-align: right">565k</td>
  * </tr>
  * <tr>
- * <td align="right">200k</td>
- * <td align="right">2200k</td>
- * <td align="right">1065k</td>
+ * <td style="text-align: right">200k</td>
+ * <td style="text-align: right">2200k</td>
+ * <td style="text-align: right">1065k</td>
  * </tr>
  * <tr>
- * <td align="right">300k</td>
- * <td align="right">3100k</td>
- * <td align="right">1565k</td>
+ * <td style="text-align: right">300k</td>
+ * <td style="text-align: right">3100k</td>
+ * <td style="text-align: right">1565k</td>
  * </tr>
  * <tr>
- * <td align="right">400k</td>
- * <td align="right">4000k</td>
- * <td align="right">2065k</td>
+ * <td style="text-align: right">400k</td>
+ * <td style="text-align: right">4000k</td>
+ * <td style="text-align: right">2065k</td>
  * </tr>
  * <tr>
- * <td align="right">500k</td>
- * <td align="right">4900k</td>
- * <td align="right">2565k</td>
+ * <td style="text-align: right">500k</td>
+ * <td style="text-align: right">4900k</td>
+ * <td style="text-align: right">2565k</td>
  * </tr>
  * <tr>
- * <td align="right">600k</td>
- * <td align="right">5800k</td>
- * <td align="right">3065k</td>
+ * <td style="text-align: right">600k</td>
+ * <td style="text-align: right">5800k</td>
+ * <td style="text-align: right">3065k</td>
  * </tr>
  * <tr>
- * <td align="right">700k</td>
- * <td align="right">6700k</td>
- * <td align="right">3565k</td>
+ * <td style="text-align: right">700k</td>
+ * <td style="text-align: right">6700k</td>
+ * <td style="text-align: right">3565k</td>
  * </tr>
  * <tr>
- * <td align="right">800k</td>
- * <td align="right">7600k</td>
- * <td align="right">4065k</td>
+ * <td style="text-align: right">800k</td>
+ * <td style="text-align: right">7600k</td>
+ * <td style="text-align: right">4065k</td>
  * </tr>
  * <tr>
- * <td align="right">900k</td>
- * <td align="right">8500k</td>
- * <td align="right">4565k</td>
+ * <td style="text-align: right">900k</td>
+ * <td style="text-align: right">8500k</td>
+ * <td style="text-align: right">4565k</td>
  * </tr>
  * </table>
  *
@@ -309,7 +310,7 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
     private int nMTF;
 
     private int currentChar = -1;
-    private int runLength = 0;
+    private int runLength;
 
     private int blockCRC;
     private int combinedCRC;
@@ -393,11 +394,10 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
 
     @Override
     public void write(final int b) throws IOException {
-        if (!closed) {
-            write0(b);
-        } else {
-            throw new IOException("closed");
+        if (closed) {
+            throw new IOException("Closed");
         }
+        write0(b);
     }
 
     /**
@@ -468,7 +468,7 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
     }
 
     /**
-     * Overriden to warn about an unclosed stream.
+     * Overridden to warn about an unclosed stream.
      */
     @Override
     protected void finalize() throws Throwable {
@@ -500,11 +500,8 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
     @Override
     public void close() throws IOException {
         if (!closed) {
-            final OutputStream outShadow = this.out;
-            try {
+            try (OutputStream outShadow = this.out) {
                 finish();
-            } finally {
-                outShadow.close();
             }
         }
     }
@@ -519,7 +516,7 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
 
     /**
      * Writes magic bytes like BZ on the first position of the stream
-     * and bytes indiciating the file-format, which is
+     * and bytes indicating the file-format, which is
      * huffmanised, followed by a digit indicating blockSize100k.
      * @throws IOException if the magic bytes could not been written
      */
@@ -540,7 +537,7 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
 
     private void initBlock() {
         // blockNo++;
-        this.crc.initialiseCRC();
+        this.crc.initializeCRC();
         this.last = -1;
         // ch = 0;
 
@@ -633,7 +630,7 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
                                                 + buf.length + ").");
         }
         if (closed) {
-            throw new IOException("stream closed");
+            throw new IOException("Stream closed");
         }
 
         for (final int hi = offs + len; offs < hi;) {
@@ -974,6 +971,7 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
             for (int j = 16; --j >= 0;) {
                 if (inUse[i16 + j]) {
                     inUse16[i] = true;
+                    break;
                 }
             }
         }
@@ -1236,11 +1234,10 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
                             mtfFreq[RUNB]++;
                         }
 
-                        if (zPend >= 2) {
-                            zPend = (zPend - 2) >> 1;
-                        } else {
+                        if (zPend < 2) {
                             break;
                         }
+                        zPend = (zPend - 2) >> 1;
                     }
                     zPend = 0;
                 }
@@ -1263,11 +1260,10 @@ public class BZip2CompressorOutputStream extends CompressorOutputStream
                     mtfFreq[RUNB]++;
                 }
 
-                if (zPend >= 2) {
-                    zPend = (zPend - 2) >> 1;
-                } else {
+                if (zPend < 2) {
                     break;
                 }
+                zPend = (zPend - 2) >> 1;
             }
         }
 
