@@ -21,6 +21,7 @@ package org.apache.commons.compress.compressors.xz;
 import java.util.HashMap;
 import java.util.Map;
 import org.apache.commons.compress.compressors.FileNameUtil;
+import org.apache.commons.compress.utils.OsgiUtils;
 
 /**
  * Utility code for the xz compression format.
@@ -54,11 +55,7 @@ public class XZUtils {
         uncompressSuffix.put("-xz", "");
         fileNameUtil = new FileNameUtil(uncompressSuffix, ".xz");
         cachedXZAvailability = CachedAvailability.DONT_CACHE;
-        try {
-            Class.forName("org.osgi.framework.BundleEvent");
-        } catch (final Exception ex) {
-            setCacheXZAvailablity(true);
-        }
+        setCacheXZAvailablity(!OsgiUtils.isRunningInOsgiEnvironment());
     }
 
     /** Private constructor to prevent instantiation of this utility class. */
@@ -108,20 +105,20 @@ public class XZUtils {
         try {
             XZCompressorInputStream.matches(null, 0);
             return true;
-        } catch (final NoClassDefFoundError error) {
+        } catch (final NoClassDefFoundError error) { // NOSONAR
             return false;
         }
     }
 
     /**
-     * Detects common xz suffixes in the given filename.
+     * Detects common xz suffixes in the given file name.
      *
-     * @param filename name of a file
-     * @return {@code true} if the filename has a common xz suffix,
+     * @param fileName name of a file
+     * @return {@code true} if the file name has a common xz suffix,
      *         {@code false} otherwise
      */
-    public static boolean isCompressedFilename(final String filename) {
-        return fileNameUtil.isCompressedFilename(filename);
+    public static boolean isCompressedFilename(final String fileName) {
+        return fileNameUtil.isCompressedFilename(fileName);
     }
 
     /**
@@ -129,31 +126,31 @@ public class XZUtils {
      * file should have after uncompression. Commonly used file type specific
      * suffixes like ".txz" are automatically detected and
      * correctly mapped. For example the name "package.txz" is mapped to
-     * "package.tar". And any filenames with the generic ".xz" suffix
+     * "package.tar". And any file names with the generic ".xz" suffix
      * (or any other generic xz suffix) is mapped to a name without that
-     * suffix. If no xz suffix is detected, then the filename is returned
+     * suffix. If no xz suffix is detected, then the file name is returned
      * unmapped.
      *
-     * @param filename name of a file
+     * @param fileName name of a file
      * @return name of the corresponding uncompressed file
      */
-    public static String getUncompressedFilename(final String filename) {
-        return fileNameUtil.getUncompressedFilename(filename);
+    public static String getUncompressedFilename(final String fileName) {
+        return fileNameUtil.getUncompressedFilename(fileName);
     }
 
     /**
-     * Maps the given filename to the name that the file should have after
+     * Maps the given file name to the name that the file should have after
      * compression with xz. Common file types with custom suffixes for
      * compressed versions are automatically detected and correctly mapped.
      * For example the name "package.tar" is mapped to "package.txz". If no
      * custom mapping is applicable, then the default ".xz" suffix is appended
-     * to the filename.
+     * to the file name.
      *
-     * @param filename name of a file
+     * @param fileName name of a file
      * @return name of the corresponding compressed file
      */
-    public static String getCompressedFilename(final String filename) {
-        return fileNameUtil.getCompressedFilename(filename);
+    public static String getCompressedFilename(final String fileName) {
+        return fileNameUtil.getCompressedFilename(fileName);
     }
 
     /**
