@@ -22,7 +22,11 @@ import org.apache.commons.compress.archivers.sevenz.SevenZArchiveEntry;
 import org.apache.commons.compress.utils.ArchiveUtils;
 import org.junit.Test;
 
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
+import static org.junit.Assert.assertTrue;
 
 public class ArchiveUtilsTest extends AbstractTestCase {
 
@@ -99,7 +103,7 @@ public class ArchiveUtilsTest extends AbstractTestCase {
     @Test
     public void testIsEqualWithNullWithPositive() {
 
-        byte[] byteArray = new byte[8];
+        final byte[] byteArray = new byte[8];
         byteArray[1] = (byte) (-77);
 
         assertFalse(ArchiveUtils.isEqualWithNull(byteArray, 0, (byte)0, byteArray, (byte)0, (byte)80));
@@ -109,7 +113,7 @@ public class ArchiveUtilsTest extends AbstractTestCase {
     @Test
     public void testToAsciiBytes() {
 
-        byte[] byteArray = ArchiveUtils.toAsciiBytes("SOCKET");
+        final byte[] byteArray = ArchiveUtils.toAsciiBytes("SOCKET");
 
         assertArrayEquals(new byte[] {(byte)83, (byte)79, (byte)67, (byte)75, (byte)69, (byte)84}, byteArray);
 
@@ -120,8 +124,8 @@ public class ArchiveUtilsTest extends AbstractTestCase {
     @Test
     public void testToStringWithNonNull() {
 
-        SevenZArchiveEntry sevenZArchiveEntry = new SevenZArchiveEntry();
-        String string = ArchiveUtils.toString(sevenZArchiveEntry);
+        final SevenZArchiveEntry sevenZArchiveEntry = new SevenZArchiveEntry();
+        final String string = ArchiveUtils.toString(sevenZArchiveEntry);
 
         assertEquals("-       0 null", string);
 
@@ -130,14 +134,14 @@ public class ArchiveUtilsTest extends AbstractTestCase {
     @Test
     public void testIsEqual() {
 
-        assertTrue(ArchiveUtils.isEqual((byte[]) null, 0, 0, (byte[]) null, 0, 0));
+        assertTrue(ArchiveUtils.isEqual(null, 0, 0, null, 0, 0));
 
     }
 
     @Test(expected = StringIndexOutOfBoundsException.class)
     public void testToAsciiStringThrowsStringIndexOutOfBoundsException() {
 
-        byte[] byteArray = new byte[3];
+        final byte[] byteArray = new byte[3];
 
         ArchiveUtils.toAsciiString(byteArray, 940, 2730);
 
@@ -148,6 +152,6 @@ public class ArchiveUtilsTest extends AbstractTestCase {
     }
 
     private void asciiToByteAndBackFail(final String inputString) {
-        assertFalse(inputString.equals(ArchiveUtils.toAsciiString(ArchiveUtils.toAsciiBytes(inputString))));
+        assertNotEquals(inputString, ArchiveUtils.toAsciiString(ArchiveUtils.toAsciiBytes(inputString)));
     }
 }
