@@ -18,12 +18,14 @@
  */
 package org.apache.commons.compress.archivers;
 
+import static org.junit.Assert.fail;
+
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.nio.file.Files;
 
 import org.apache.commons.compress.AbstractTestCase;
 import org.apache.commons.compress.archivers.ar.ArArchiveEntry;
@@ -32,28 +34,20 @@ import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
 import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
 import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
 import org.apache.commons.compress.utils.IOUtils;
+import org.junit.Test;
 
 public class ArchiveOutputStreamTest extends AbstractTestCase {
 
-    @Override
-    protected void setUp() throws Exception {
-        super.setUp();
-    }
-
-    @Override
-    protected void tearDown() throws Exception {
-        super.tearDown();
-    }
-
+    @Test
     public void testFinish() throws Exception {
-        OutputStream out1 = new ByteArrayOutputStream();
+        final OutputStream out1 = new ByteArrayOutputStream();
 
         ArchiveOutputStream aos1 = factory.createArchiveOutputStream("zip", out1);
         aos1.putArchiveEntry(new ZipArchiveEntry("dummy"));
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (IOException io) {
+        } catch (final IOException io) {
             // Exception expected
         }
 
@@ -62,7 +56,7 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (IOException io) {
+        } catch (final IOException io) {
             // Exception expected
         }
 
@@ -71,7 +65,7 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (IOException io) {
+        } catch (final IOException io) {
             // Exception expected
         }
 
@@ -80,7 +74,7 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (IOException io) {
+        } catch (final IOException io) {
             // Exception expected
         }
 
@@ -89,61 +83,70 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (IOException io) {
+        } catch (final IOException io) {
             // Exception expected
         }
     }
 
+    @Test
     public void testOptionalFinish() throws Exception {
-        OutputStream out1 = new ByteArrayOutputStream();
+        final OutputStream out1 = new ByteArrayOutputStream();
 
-        ArchiveOutputStream aos1 = factory.createArchiveOutputStream("zip", out1);
-        aos1.putArchiveEntry(new ZipArchiveEntry("dummy"));
-        aos1.closeArchiveEntry();
-        aos1.close();
+        try (ArchiveOutputStream aos1 = factory.createArchiveOutputStream("zip", out1)) {
+            aos1.putArchiveEntry(new ZipArchiveEntry("dummy"));
+            aos1.closeArchiveEntry();
+        }
 
-        aos1 = factory.createArchiveOutputStream("jar", out1);
-        aos1.putArchiveEntry(new JarArchiveEntry("dummy"));
-        aos1.closeArchiveEntry();
-        aos1.close();
+        final ArchiveOutputStream finishTest;
+        try (ArchiveOutputStream aos1 = factory.createArchiveOutputStream("jar", out1)) {
+            finishTest = aos1;
+            aos1.putArchiveEntry(new JarArchiveEntry("dummy"));
+            aos1.closeArchiveEntry();
+        }
         try {
-            aos1.finish();
+            finishTest.finish();
             fail("finish() cannot follow close()");
-        } catch (IOException io) {
+        } catch (final IOException io) {
             // Exception expected
         }
+        finishTest.close();
     }
 
+    @Test
     public void testCallSequenceAr() throws Exception{
         doCallSequence("Ar");
     }
 
+    @Test
     public void testCallSequenceCpio() throws Exception{
         doCallSequence("Cpio");
     }
 
+    @Test
     public void testCallSequenceJar() throws Exception{
         doCallSequence("Jar");
     }
 
+    @Test
     public void testCallSequenceTar() throws Exception{
         doCallSequence("Tar");
     }
 
+    @Test
     public void testCallSequenceZip() throws Exception{
         doCallSequence("Zip");
     }
 
-    private void doCallSequence(String archiveType) throws Exception {
-        OutputStream out1 = new ByteArrayOutputStream();
-        File dummy = getFile("test1.xml"); // need a real file
+    private void doCallSequence(final String archiveType) throws Exception {
+        final OutputStream out1 = new ByteArrayOutputStream();
+        final File dummy = getFile("test1.xml"); // need a real file
 
         ArchiveOutputStream aos1;
         aos1 = factory.createArchiveOutputStream(archiveType, out1);
         aos1.putArchiveEntry(aos1.createArchiveEntry(dummy, "dummy"));
-        InputStream is = new FileInputStream(dummy);
-        IOUtils.copy(is, aos1);
-        is.close();
+        try (InputStream is = Files.newInputStream(dummy.toPath())) {
+            IOUtils.copy(is, aos1);
+        }
         aos1.closeArchiveEntry();
         aos1.close(); // omitted finish
 
@@ -153,38 +156,53 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.closeArchiveEntry();
             fail("Should have raised IOException - closeArchiveEntry() called before putArchiveEntry()");
-        } catch (IOException expected) {
+        } catch (final IOException expected) {
         }
 
         aos1.putArchiveEntry(aos1.createArchiveEntry(dummy, "dummy"));
-        is = new FileInputStream(dummy);
-        IOUtils.copy(is, aos1);
-        is.close();
+        try (InputStream is = Files.newInputStream(dummy.toPath())) {
+            IOUtils.copy(is, aos1);
+        }
 
         // TODO check if second putArchiveEntry() can follow without closeAE?
 
         try {
             aos1.finish();
             fail("Should have raised IOException - finish() called before closeArchiveEntry()");
-        } catch (IOException expected) {
+        } catch (final IOException expected) {
         }
         try {
             aos1.close();
             fail("Should have raised IOException - close() called before closeArchiveEntry()");
-        } catch (IOException expected) {
+        } catch (final IOException expected) {
         }
+
+        aos1 = createArchiveWithDummyEntry(archiveType, out1, dummy);
         aos1.closeArchiveEntry();
         try {
             aos1.closeArchiveEntry();
             fail("Should have raised IOException - closeArchiveEntry() called with no open entry");
-        } catch (IOException expected) {
+        } catch (final IOException expected) {
         }
+
+        aos1 = createArchiveWithDummyEntry(archiveType, out1, dummy);
+        aos1.closeArchiveEntry();
         aos1.finish();
         aos1.close();
         try {
             aos1.finish();
             fail("Should have raised IOException - finish() called after close()");
-        } catch (IOException expected) {
+        } catch (final IOException expected) {
         }
     }
+
+    private ArchiveOutputStream createArchiveWithDummyEntry(final String archiveType, final OutputStream out1, final File dummy)
+        throws Exception {
+        final ArchiveOutputStream aos1 = factory.createArchiveOutputStream(archiveType, out1);
+        aos1.putArchiveEntry(aos1.createArchiveEntry(dummy, "dummy"));
+        try (InputStream is = Files.newInputStream(dummy.toPath())) {
+            IOUtils.copy(is, aos1);
+        }
+        return aos1;
+    }
 }
