@@ -27,17 +27,20 @@ import static org.junit.Assert.fail;
 import java.io.BufferedInputStream;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
-import java.io.FileInputStream;
+import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.lang.reflect.Field;
+import java.nio.file.Files;
 
+import org.apache.commons.compress.MockEvilInputStream;
 import org.apache.commons.compress.archivers.arj.ArjArchiveInputStream;
 import org.apache.commons.compress.archivers.cpio.CpioArchiveInputStream;
 import org.apache.commons.compress.archivers.dump.DumpArchiveInputStream;
 import org.apache.commons.compress.archivers.jar.JarArchiveInputStream;
 import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
 import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
+import org.apache.commons.compress.utils.ByteUtils;
 import org.junit.Test;
 
 public class ArchiveStreamFactoryTest {
@@ -48,9 +51,9 @@ public class ArchiveStreamFactoryTest {
      * see https://issues.apache.org/jira/browse/COMPRESS-171
      */
     @Test
-    public void shortTextFilesAreNoTARs() throws Exception {
+    public void shortTextFilesAreNoTARs() {
         try {
-            new ArchiveStreamFactory()
+            ArchiveStreamFactory.DEFAULT
                 .createArchiveInputStream(new ByteArrayInputStream("This certainly is not a tar archive, really, no kidding".getBytes()));
             fail("created an input stream for a non-archive");
         } catch (final ArchiveException ae) {
@@ -63,9 +66,9 @@ public class ArchiveStreamFactoryTest {
      */
     @Test
     public void aiffFilesAreNoTARs() throws Exception {
-        try (FileInputStream fis = new FileInputStream("src/test/resources/testAIFF.aif")) {
+        try (InputStream fis = Files.newInputStream(new File("src/test/resources/testAIFF.aif").toPath())) {
             try (InputStream is = new BufferedInputStream(fis)) {
-                new ArchiveStreamFactory().createArchiveInputStream(is);
+                ArchiveStreamFactory.DEFAULT.createArchiveInputStream(is);
                 fail("created an input stream for a non-archive");
             } catch (final ArchiveException ae) {
                 assertTrue(ae.getMessage().startsWith("No Archiver found"));
@@ -75,9 +78,9 @@ public class ArchiveStreamFactoryTest {
 
     @Test
     public void testCOMPRESS209() throws Exception {
-        try (FileInputStream fis = new FileInputStream("src/test/resources/testCompress209.doc")) {
+        try (InputStream fis = Files.newInputStream(new File("src/test/resources/testCompress209.doc").toPath())) {
             try (InputStream bis = new BufferedInputStream(fis)) {
-                new ArchiveStreamFactory().createArchiveInputStream(bis);
+                ArchiveStreamFactory.DEFAULT.createArchiveInputStream(bis);
                 fail("created an input stream for a non-archive");
             } catch (final ArchiveException ae) {
                 assertTrue(ae.getMessage().startsWith("No Archiver found"));
@@ -87,28 +90,28 @@ public class ArchiveStreamFactoryTest {
 
     @Test(expected = StreamingNotSupportedException.class)
     public void cantRead7zFromStream() throws Exception {
-        new ArchiveStreamFactory()
+        ArchiveStreamFactory.DEFAULT
             .createArchiveInputStream(ArchiveStreamFactory.SEVEN_Z,
-                                      new ByteArrayInputStream(new byte[0]));
+                                      new ByteArrayInputStream(ByteUtils.EMPTY_BYTE_ARRAY));
     }
 
     @Test(expected = StreamingNotSupportedException.class)
     public void cantWrite7zToStream() throws Exception {
-        new ArchiveStreamFactory()
+        ArchiveStreamFactory.DEFAULT
             .createArchiveOutputStream(ArchiveStreamFactory.SEVEN_Z,
                                        new ByteArrayOutputStream());
     }
 
     /**
-     * Test case for 
+     * Test case for
      * <a href="https://issues.apache.org/jira/browse/COMPRESS-267"
      * >COMPRESS-267</a>.
      */
     @Test
     public void detectsAndThrowsFor7z() throws Exception {
-        try (FileInputStream fis = new FileInputStream("src/test/resources/bla.7z")) {
+        try (InputStream fis = Files.newInputStream(new File("src/test/resources/bla.7z").toPath())) {
             try (InputStream bis = new BufferedInputStream(fis)) {
-                new ArchiveStreamFactory().createArchiveInputStream(bis);
+                ArchiveStreamFactory.DEFAULT.createArchiveInputStream(bis);
                 fail("Expected a StreamingNotSupportedException");
             } catch (final StreamingNotSupportedException ex) {
                 assertEquals(ArchiveStreamFactory.SEVEN_Z, ex.getFormat());
@@ -117,21 +120,21 @@ public class ArchiveStreamFactoryTest {
     }
 
     /**
-     * Test case for 
+     * Test case for
      * <a href="https://issues.apache.org/jira/browse/COMPRESS-208"
      * >COMPRESS-208</a>.
      */
     @Test
     public void skipsPK00Prefix() throws Exception {
-        try (FileInputStream fis = new FileInputStream("src/test/resources/COMPRESS-208.zip")) {
+        try (InputStream fis = Files.newInputStream(new File("src/test/resources/COMPRESS-208.zip").toPath())) {
             try (InputStream bis = new BufferedInputStream(fis)) {
-                try (ArchiveInputStream ais = new ArchiveStreamFactory().createArchiveInputStream(bis)) {
+                try (ArchiveInputStream ais = ArchiveStreamFactory.DEFAULT.createArchiveInputStream(bis)) {
                     assertTrue(ais instanceof ZipArchiveInputStream);
                 }
             }
         }
     }
-    
+
     @Test
     public void testEncodingCtor() {
         ArchiveStreamFactory fac = new ArchiveStreamFactory();
@@ -168,7 +171,7 @@ public class ArchiveStreamFactoryTest {
         final String fieldName;
         final String type;
         final boolean hasOutputStream;
-        
+
         TestData(final String testFile, final String type, final boolean hasOut, final String expectedEncoding, final ArchiveStreamFactory fac, final String fieldName) {
             this.testFile = testFile;
             this.expectedEncoding = expectedEncoding;
@@ -177,7 +180,7 @@ public class ArchiveStreamFactoryTest {
             this.type = type;
             this.hasOutputStream = hasOut;
         }
-        
+
         @Override
         public String toString() {
             return "TestData [testFile=" + testFile + ", expectedEncoding=" + expectedEncoding + ", fac=" + fac
@@ -192,7 +195,7 @@ public class ArchiveStreamFactoryTest {
         return fac;
     }
     // The different factory types
-    private static final ArchiveStreamFactory FACTORY = new ArchiveStreamFactory();
+    private static final ArchiveStreamFactory FACTORY = ArchiveStreamFactory.DEFAULT;
     private static final ArchiveStreamFactory FACTORY_UTF8 = new ArchiveStreamFactory("UTF-8");
     private static final ArchiveStreamFactory FACTORY_ASCII = new ArchiveStreamFactory("ASCII");
     private static final ArchiveStreamFactory FACTORY_SET_UTF8 = getFactory("UTF-8");
@@ -212,20 +215,64 @@ public class ArchiveStreamFactoryTest {
         String dflt;
         dflt = UNKNOWN;
         try {
-            dflt = getField(new ArjArchiveInputStream(new FileInputStream(getFile("bla.arj"))), "charsetName");
+            dflt = getField(new ArjArchiveInputStream(Files.newInputStream(getFile("bla.arj").toPath())), "charsetName");
         } catch (final Exception e) {
             e.printStackTrace();
         }
         ARJ_DEFAULT = dflt;
         dflt = UNKNOWN;
         try {
-            dflt = getField(new DumpArchiveInputStream(new FileInputStream(getFile("bla.dump"))), "encoding");
+            dflt = getField(new DumpArchiveInputStream(Files.newInputStream(getFile("bla.dump").toPath())), "encoding");
         } catch (final Exception e) {
             e.printStackTrace();
         }
         DUMP_DEFAULT = dflt;
     }
 
+    @Test
+    public void testDetect() throws Exception {
+        for (final String extension : new String[]{
+                ArchiveStreamFactory.AR,
+                ArchiveStreamFactory.ARJ,
+                ArchiveStreamFactory.CPIO,
+                ArchiveStreamFactory.DUMP,
+                // Compress doesn't know how to detect JARs, see COMPRESS-91
+ //               ArchiveStreamFactory.JAR,
+                ArchiveStreamFactory.SEVEN_Z,
+                ArchiveStreamFactory.TAR,
+                ArchiveStreamFactory.ZIP
+        }) {
+            assertEquals(extension, detect("bla."+extension));
+        }
+
+        try {
+            ArchiveStreamFactory.detect(new BufferedInputStream(new ByteArrayInputStream(ByteUtils.EMPTY_BYTE_ARRAY)));
+            fail("shouldn't be able to detect empty stream");
+        } catch (final ArchiveException e) {
+            assertEquals("No Archiver found for the stream signature", e.getMessage());
+        }
+
+        try {
+            ArchiveStreamFactory.detect(null);
+            fail("shouldn't be able to detect null stream");
+        } catch (final IllegalArgumentException e) {
+            assertEquals("Stream must not be null.", e.getMessage());
+        }
+
+        try {
+            ArchiveStreamFactory.detect(new BufferedInputStream(new MockEvilInputStream()));
+            fail("Expected ArchiveException");
+        } catch (final ArchiveException e) {
+            assertEquals("IOException while reading signature.", e.getMessage());
+        }
+    }
+
+    private String detect(final String resource) throws IOException, ArchiveException {
+        try(InputStream in = new BufferedInputStream(Files.newInputStream(getFile(resource).toPath()))) {
+            return ArchiveStreamFactory.detect(in);
+        }
+    }
+
     static final TestData[] TESTS = {
         new TestData("bla.arj", ArchiveStreamFactory.ARJ, false, ARJ_DEFAULT, FACTORY, "charsetName"),
         new TestData("bla.arj", ArchiveStreamFactory.ARJ, false, "UTF-8", FACTORY_UTF8, "charsetName"),
@@ -342,7 +389,7 @@ public class ArchiveStreamFactoryTest {
                 } catch (final NoSuchFieldException e) {
                     System.out.println("Cannot find " + name + " in class " + instance.getClass().getSimpleName());
                     return UNKNOWN;
-                }                
+                }
         }
         final boolean isAccessible = fld.isAccessible();
         try {
@@ -368,20 +415,18 @@ public class ArchiveStreamFactoryTest {
     private ArchiveInputStream getInputStreamFor(final String resource, final ArchiveStreamFactory factory)
             throws IOException, ArchiveException {
         return factory.createArchiveInputStream(
-                   new BufferedInputStream(new FileInputStream(
-                       getFile(resource))));
+                   new BufferedInputStream(Files.newInputStream(getFile(resource).toPath())));
     }
 
     private ArchiveInputStream getInputStreamFor(final String type, final String resource, final ArchiveStreamFactory factory)
             throws IOException, ArchiveException {
         return factory.createArchiveInputStream(
                    type,
-                   new BufferedInputStream(new FileInputStream(
-                       getFile(resource))));
+                   new BufferedInputStream(Files.newInputStream(getFile(resource).toPath())));
     }
 
     private ArchiveOutputStream getOutputStreamFor(final String type, final ArchiveStreamFactory factory)
-            throws IOException, ArchiveException {
+            throws ArchiveException {
         return factory.createArchiveOutputStream(type, new ByteArrayOutputStream());
     }
 }
