@@ -13,7 +13,7 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
+ *
  */
 
 package org.apache.commons.compress.archivers;
@@ -24,16 +24,13 @@ import static org.junit.Assert.fail;
 import java.io.BufferedInputStream;
 import java.io.BufferedReader;
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileReader;
-import java.io.FilenameFilter;
+import java.net.URISyntaxException;
+import java.nio.file.Files;
 import java.util.ArrayList;
 import java.util.Collection;
 
 import junit.framework.AssertionFailedError;
-
 import org.apache.commons.compress.AbstractTestCase;
-import org.apache.commons.compress.archivers.ArchiveEntry;
 import org.apache.commons.compress.archivers.ar.ArArchiveInputStream;
 import org.apache.commons.compress.archivers.cpio.CpioArchiveInputStream;
 import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
@@ -47,7 +44,7 @@ import org.junit.runners.Parameterized.Parameters;
 
 /**
  * Test that can read various tar file examples.
- * 
+ *
   * Files must be in resources/longsymlink, and there must be a file.txt containing
  * the list of files in the archives.
 */
@@ -55,9 +52,17 @@ import org.junit.runners.Parameterized.Parameters;
 public class LongSymLinkTest extends AbstractTestCase {
 
     private static final ClassLoader CLASSLOADER = LongSymLinkTest.class.getClassLoader();
-    private static final File ARCDIR = new File(CLASSLOADER.getResource("longsymlink").getFile());
+    private static final File ARCDIR;
     private static final ArrayList<String> FILELIST = new ArrayList<>();
 
+    static {
+        try {
+            ARCDIR = new File(CLASSLOADER.getResource("longsymlink").toURI());
+        } catch (final URISyntaxException e) {
+            throw new RuntimeException(e);
+        }
+    }
+
     private final File file;
 
     public LongSymLinkTest(final String file){
@@ -69,7 +74,7 @@ public class LongSymLinkTest extends AbstractTestCase {
         assertTrue(ARCDIR.exists());
         final File listing= new File(ARCDIR,"files.txt");
         assertTrue("files.txt is readable",listing.canRead());
-        final BufferedReader br = new BufferedReader(new FileReader(listing));
+        final BufferedReader br = new BufferedReader(Files.newBufferedReader(listing.toPath()));
         String line;
         while ((line=br.readLine())!=null){
             if (!line.startsWith("#")){
@@ -82,12 +87,7 @@ public class LongSymLinkTest extends AbstractTestCase {
     @Parameters(name = "file={0}")
     public static Collection<Object[]> data() {
         final Collection<Object[]> params = new ArrayList<>();
-        for (final String f : ARCDIR.list(new FilenameFilter() {
-            @Override
-            public boolean accept(final File dir, final String name) {
-                return !name.endsWith(".txt");
-            }
-        })) 
+        for (final String f : ARCDIR.list((dir, name) -> !name.endsWith(".txt")))
         {
             params.add(new Object[] { f });
         }
@@ -116,7 +116,7 @@ public class LongSymLinkTest extends AbstractTestCase {
             expected.add("META-INF/");
             expected.add("META-INF/MANIFEST.MF");
         }
-        final ArchiveInputStream ais = factory.createArchiveInputStream(new BufferedInputStream(new FileInputStream(file)));
+        final ArchiveInputStream ais = factory.createArchiveInputStream(new BufferedInputStream(Files.newInputStream(file.toPath())));
         // check if expected type recognized
         if (name.endsWith(".tar")){
             assertTrue(ais instanceof TarArchiveInputStream);
@@ -139,7 +139,7 @@ public class LongSymLinkTest extends AbstractTestCase {
                 if (!ent.endsWith("/")) {// not a directory
                     final int lastSlash = ent.lastIndexOf('/');
                     if (lastSlash >= 0) { // extract path name
-                        expected.add(ent.substring(lastSlash + 1, ent.length()));
+                        expected.add(ent.substring(lastSlash + 1));
                     } else {
                         expected.add(ent);
                     }
