@@ -18,18 +18,24 @@
  */
 package org.apache.commons.compress.archivers;
 
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.io.ByteArrayInputStream;
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.nio.channels.Channels;
 import java.nio.channels.SeekableByteChannel;
 import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
+import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Enumeration;
@@ -55,7 +61,7 @@ import org.junit.Assert;
 import org.junit.Test;
 
 public final class ZipTestCase extends AbstractTestCase {
-    
+
     /**
      * Archives 2 files and unarchives it again. If the file length of result
      * and source is the same, it looks like the operations have worked
@@ -68,16 +74,16 @@ public final class ZipTestCase extends AbstractTestCase {
         final File file1 = getFile("test1.xml");
         final File file2 = getFile("test2.xml");
 
-        try (final OutputStream out = new FileOutputStream(output)) {
-            try (ArchiveOutputStream os = new ArchiveStreamFactory().createArchiveOutputStream("zip", out)) {
+        try (final OutputStream out = Files.newOutputStream(output.toPath())) {
+            try (ArchiveOutputStream os = ArchiveStreamFactory.DEFAULT.createArchiveOutputStream("zip", out)) {
                 os.putArchiveEntry(new ZipArchiveEntry("testdata/test1.xml"));
-                try (final FileInputStream input = new FileInputStream(file1)) {
+                try (final InputStream input = Files.newInputStream(file1.toPath())) {
                     IOUtils.copy(input, os);
                 }
                 os.closeArchiveEntry();
 
                 os.putArchiveEntry(new ZipArchiveEntry("testdata/test2.xml"));
-                try (final FileInputStream input = new FileInputStream(file2)) {
+                try (final InputStream input = Files.newInputStream(file2.toPath())) {
                     IOUtils.copy(input, os);
                 }
                 os.closeArchiveEntry();
@@ -87,14 +93,14 @@ public final class ZipTestCase extends AbstractTestCase {
         // Unarchive the same
         final List<File> results = new ArrayList<>();
 
-        try (final InputStream fileInputStream = new FileInputStream(output)) {
-            try (ArchiveInputStream archiveInputStream = new ArchiveStreamFactory().createArchiveInputStream("zip",
+        try (final InputStream fileInputStream = Files.newInputStream(output.toPath())) {
+            try (ArchiveInputStream archiveInputStream = ArchiveStreamFactory.DEFAULT.createArchiveInputStream("zip",
                 fileInputStream)) {
                 ZipArchiveEntry entry = null;
                 while ((entry = (ZipArchiveEntry) archiveInputStream.getNextEntry()) != null) {
                     final File outfile = new File(resultDir.getCanonicalPath() + "/result/" + entry.getName());
                     outfile.getParentFile().mkdirs();
-                    try (OutputStream o = new FileOutputStream(outfile)) {
+                    try (OutputStream o = Files.newOutputStream(outfile.toPath())) {
                         IOUtils.copy(archiveInputStream, o);
                     }
                     results.add(outfile);
@@ -136,12 +142,12 @@ public final class ZipTestCase extends AbstractTestCase {
             }
 
             // Unarchive the same
-            try (ArchiveInputStream inputStream = new ArchiveStreamFactory().createArchiveInputStream("zip",
+            try (ArchiveInputStream inputStream = ArchiveStreamFactory.DEFAULT.createArchiveInputStream("zip",
                 new ByteArrayInputStream(channel.array()))) {
 
                 ZipArchiveEntry entry;
                 while ((entry = (ZipArchiveEntry) inputStream.getNextEntry()) != null) {
-                    byte[] result = new byte[(int) entry.getSize()];
+                    final byte[] result = new byte[(int) entry.getSize()];
                     IOUtils.readFully(inputStream, result);
                     results.add(result);
                 }
@@ -158,10 +164,10 @@ public final class ZipTestCase extends AbstractTestCase {
     @Test
     public void testZipUnarchive() throws Exception {
         final File input = getFile("bla.zip");
-        try (final InputStream is = new FileInputStream(input);
-                final ArchiveInputStream in = new ArchiveStreamFactory().createArchiveInputStream("zip", is)) {
+        try (final InputStream is = Files.newInputStream(input.toPath());
+                final ArchiveInputStream in = ArchiveStreamFactory.DEFAULT.createArchiveInputStream("zip", is)) {
             final ZipArchiveEntry entry = (ZipArchiveEntry) in.getNextEntry();
-            try (final OutputStream out = new FileOutputStream(new File(dir, entry.getName()))) {
+            try (final OutputStream out = Files.newOutputStream(new File(dir, entry.getName()).toPath())) {
                 IOUtils.copy(in, out);
             }
         }
@@ -178,7 +184,7 @@ public final class ZipTestCase extends AbstractTestCase {
         final ArrayList<String> al = new ArrayList<>();
         al.add("test1.xml");
         al.add("test2.xml");
-        try (InputStream fis = new FileInputStream(input)) {
+        try (InputStream fis = Files.newInputStream(input.toPath())) {
             checkArchiveContent(new ZipArchiveInputStream(fis), al);
         }
     }
@@ -208,7 +214,7 @@ public final class ZipTestCase extends AbstractTestCase {
     @Test
     public void testSkipEntryWithUnsupportedCompressionMethod()
             throws IOException {
-        try (ZipArchiveInputStream zip = new ZipArchiveInputStream(new FileInputStream(getFile("moby.zip")))) {
+        try (ZipArchiveInputStream zip = new ZipArchiveInputStream(Files.newInputStream(getFile("moby.zip").toPath()))) {
             final ZipArchiveEntry entry = zip.getNextZipEntry();
             assertEquals("method", ZipMethod.TOKENIZATION.getCode(), entry.getMethod());
             assertEquals("README", entry.getName());
@@ -238,20 +244,20 @@ public final class ZipTestCase extends AbstractTestCase {
         final List<String> results = new ArrayList<>();
         final List<ZipException> expectedExceptions = new ArrayList<>();
 
-        try (final InputStream fis = new FileInputStream(input);
-            ArchiveInputStream in = new ArchiveStreamFactory().createArchiveInputStream("zip", fis)) {
+        try (final InputStream fis = Files.newInputStream(input.toPath());
+            ArchiveInputStream in = ArchiveStreamFactory.DEFAULT.createArchiveInputStream("zip", fis)) {
 
             ZipArchiveEntry entry = null;
             while ((entry = (ZipArchiveEntry) in.getNextEntry()) != null) {
                 results.add(entry.getName());
 
-                final ArchiveInputStream nestedIn = new ArchiveStreamFactory().createArchiveInputStream("zip", in);
+                final ArchiveInputStream nestedIn = ArchiveStreamFactory.DEFAULT.createArchiveInputStream("zip", in);
                 try {
                     ZipArchiveEntry nestedEntry = null;
                     while ((nestedEntry = (ZipArchiveEntry) nestedIn.getNextEntry()) != null) {
                         results.add(nestedEntry.getName());
                     }
-                } catch (ZipException ex) {
+                } catch (final ZipException ex) {
                     // expected since you cannot create a final ArchiveInputStream from test3.xml
                     expectedExceptions.add(ex);
                 }
@@ -337,14 +343,9 @@ public final class ZipTestCase extends AbstractTestCase {
             rmdir(tmp[0]);
         }
     }
-    String first_payload = "ABBA";
-    String second_payload = "AAAAAAAAAAAA";
-    ZipArchiveEntryPredicate allFilesPredicate = new ZipArchiveEntryPredicate() {
-        @Override
-        public boolean test(final ZipArchiveEntry zipArchiveEntry) {
-            return true;
-        }
-    };
+    final String first_payload = "ABBA";
+    final String second_payload = "AAAAAAAAAAAA";
+    final ZipArchiveEntryPredicate allFilesPredicate = zipArchiveEntry -> true;
 
     @Test
     public void testCopyRawEntriesFromFile()
@@ -415,7 +416,9 @@ public final class ZipTestCase extends AbstractTestCase {
 
             final ZipArchiveEntry archiveEntry = new ZipArchiveEntry("fred");
             archiveEntry.setUnixMode(0664);
-            archiveEntry.setMethod(ZipEntry.DEFLATED);
+            archiveEntry.setMethod(ZipEntry.STORED);
+            archiveEntry.setSize(3);
+            archiveEntry.setCompressedSize(3);
             zos.addRawArchiveEntry(archiveEntry, new ByteArrayInputStream("fud".getBytes()));
         }
 
@@ -497,15 +500,17 @@ public final class ZipTestCase extends AbstractTestCase {
         File archive = null;
         ZipArchiveOutputStream zos = null;
         ZipFile zf = null;
-        FileInputStream fis = null;
+        InputStream fis = null;
+        final File tmpDir = tmp[0];
+        final File tmpFile = tmp[1];
         try {
-            archive = File.createTempFile("test.", ".zip", tmp[0]);
+            archive = File.createTempFile("test.", ".zip", tmpDir);
             archive.deleteOnExit();
             zos = new ZipArchiveOutputStream(archive);
-            final ZipArchiveEntry in = new ZipArchiveEntry(tmp[1], "foo");
+            final ZipArchiveEntry in = new ZipArchiveEntry(tmpFile, "foo");
             zos.putArchiveEntry(in);
-            final byte[] b = new byte[(int) tmp[1].length()];
-            fis = new FileInputStream(tmp[1]);
+            final byte[] b = new byte[(int) tmpFile.length()];
+            fis = Files.newInputStream(tmpFile.toPath());
             while (fis.read(b) > 0) {
                 zos.write(b);
             }
@@ -518,8 +523,8 @@ public final class ZipTestCase extends AbstractTestCase {
             final ZipArchiveEntry out = zf.getEntry("foo");
             assertNotNull(out);
             assertEquals("foo", out.getName());
-            assertEquals(tmp[1].length(), out.getSize());
-            assertEquals(tmp[1].lastModified() / 2000,
+            assertEquals(tmpFile.length(), out.getSize());
+            assertEquals(tmpFile.lastModified() / 2000,
                          out.getLastModifiedDate().getTime() / 2000);
             assertFalse(out.isDirectory());
         } finally {
@@ -531,8 +536,58 @@ public final class ZipTestCase extends AbstractTestCase {
             if (fis != null) {
                 fis.close();
             }
-            tryHardToDelete(tmp[1]);
-            rmdir(tmp[0]);
+            tryHardToDelete(tmpFile);
+            rmdir(tmpDir);
+        }
+    }
+
+    @Test
+    public void testZipArchiveEntryNewFromPath() throws Exception {
+        final File[] tmp = createTempDirAndFile();
+        File archiveFile = null;
+        Path archivePath = null;
+        ZipArchiveOutputStream zos = null;
+        ZipFile zf = null;
+        InputStream fis = null;
+        final File tmpDir = tmp[0];
+        final File tmpFile = tmp[1];
+        final Path tmpFilePath = tmpFile.toPath();
+        try {
+            archiveFile = File.createTempFile("test.", ".zip", tmpDir);
+            archivePath = archiveFile.toPath();
+            archiveFile.deleteOnExit();
+            zos = new ZipArchiveOutputStream(archivePath);
+            final ZipArchiveEntry in = (ZipArchiveEntry) zos.createArchiveEntry(tmpFilePath, "foo");
+            zos.putArchiveEntry(in);
+            final byte[] b = new byte[(int) tmpFile.length()];
+            fis = Files.newInputStream(tmpFile.toPath());
+            while (fis.read(b) > 0) {
+                zos.write(b);
+            }
+            fis.close();
+            fis = null;
+            zos.closeArchiveEntry();
+            zos.close();
+            zos = null;
+            zf = new ZipFile(archiveFile);
+            final ZipArchiveEntry out = zf.getEntry("foo");
+            assertNotNull(out);
+            assertEquals("foo", out.getName());
+            assertEquals(tmpFile.length(), out.getSize());
+            assertEquals(tmpFile.lastModified() / 2000,
+                         out.getLastModifiedDate().getTime() / 2000);
+            assertFalse(out.isDirectory());
+        } finally {
+            ZipFile.closeQuietly(zf);
+            if (zos != null) {
+                zos.close();
+            }
+            tryHardToDelete(archiveFile);
+            if (fis != null) {
+                fis.close();
+            }
+            tryHardToDelete(tmpFile);
+            rmdir(tmpDir);
         }
     }
 
@@ -542,7 +597,7 @@ public final class ZipTestCase extends AbstractTestCase {
         File archive = null;
         ZipArchiveOutputStream zos = null;
         ZipFile zf = null;
-        FileInputStream fis = null;
+        InputStream fis = null;
         try {
             archive = File.createTempFile("test.", ".zip", tmp[0]);
             archive.deleteOnExit();
@@ -552,7 +607,7 @@ public final class ZipTestCase extends AbstractTestCase {
             in.setSize(tmp[1].length());
             zos.putArchiveEntry(in);
             final byte[] b = new byte[(int) tmp[1].length()];
-            fis = new FileInputStream(tmp[1]);
+            fis = Files.newInputStream(tmp[1].toPath());
             while (fis.read(b) > 0) {
                 zos.write(b);
             }
@@ -585,7 +640,7 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test
     public void inputStreamStatisticsOfZipBombExcel() throws IOException, ArchiveException {
-        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+        final Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
             put("[Content_Types].xml", Arrays.asList(8390036L, 8600L));
             put("xl/worksheets/sheet1.xml", Arrays.asList(1348L, 508L));
         }};
@@ -594,7 +649,7 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test
     public void inputStreamStatisticsForImplodedEntry() throws IOException, ArchiveException {
-        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+        final Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
             put("LICENSE.TXT", Arrays.asList(11560L, 4131L));
         }};
         testInputStreamStatistics("imploding-8Kdict-3trees.zip", expected);
@@ -602,7 +657,7 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test
     public void inputStreamStatisticsForShrunkEntry() throws IOException, ArchiveException {
-        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+        final Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
             put("TEST1.XML", Arrays.asList(76L, 66L));
             put("TEST2.XML", Arrays.asList(81L, 76L));
         }};
@@ -611,7 +666,7 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test
     public void inputStreamStatisticsForStoredEntry() throws IOException, ArchiveException {
-        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+        final Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
             put("test.txt", Arrays.asList(5L, 5L));
         }};
         testInputStreamStatistics("COMPRESS-264.zip", expected);
@@ -619,7 +674,7 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test
     public void inputStreamStatisticsForBzip2Entry() throws IOException, ArchiveException {
-        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+        final Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
             put("lots-of-as", Arrays.asList(42L, 39L));
         }};
         testInputStreamStatistics("bzip2-zip.zip", expected);
@@ -627,7 +682,7 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test
     public void inputStreamStatisticsForDeflate64Entry() throws IOException, ArchiveException {
-        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+        final Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
             put("input2", Arrays.asList(3072L, 2111L));
         }};
         testInputStreamStatistics("COMPRESS-380/COMPRESS-380.zip", expected);
@@ -645,14 +700,14 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test(expected = IOException.class)
     public void buildSplitZipWithSegmentAlreadyExistThrowsException() throws IOException {
-        File directoryToZip = getFilesToZip();
-        File outputZipFile = new File(dir, "splitZip.zip");
-        long splitSize = 100 * 1024L; /* 100 KB */
+        final File directoryToZip = getFilesToZip();
+        final File outputZipFile = new File(dir, "splitZip.zip");
+        final long splitSize = 100 * 1024L; /* 100 KB */
         try (final ZipArchiveOutputStream zipArchiveOutputStream = new ZipArchiveOutputStream(outputZipFile,
             splitSize)) {
 
             // create a file that has the same name of one of the created split segments
-            File sameNameFile = new File(dir, "splitZip.z01");
+            final File sameNameFile = new File(dir, "splitZip.z01");
             sameNameFile.createNewFile();
 
             addFilesToZip(zipArchiveOutputStream, directoryToZip);
@@ -661,25 +716,25 @@ public final class ZipTestCase extends AbstractTestCase {
 
     @Test
     public void buildSplitZipTest() throws IOException {
-        File directoryToZip = getFilesToZip();
+        final File directoryToZip = getFilesToZip();
         createTestSplitZipSegments();
 
-        File lastFile = new File(dir, "splitZip.zip");
+        final File lastFile = new File(dir, "splitZip.zip");
         try (SeekableByteChannel channel = ZipSplitReadOnlySeekableByteChannel.buildFromLastSplitSegment(lastFile);
             InputStream inputStream = Channels.newInputStream(channel);
             ZipArchiveInputStream splitInputStream = new ZipArchiveInputStream(inputStream,
                 StandardCharsets.UTF_8.toString(), true, false, true)) {
 
             ArchiveEntry entry;
-            int filesNum = countNonDirectories(directoryToZip);
+            final int filesNum = countNonDirectories(directoryToZip);
             int filesCount = 0;
             while ((entry = splitInputStream.getNextEntry()) != null) {
                 if (entry.isDirectory()) {
                     continue;
                 }
                 // compare all files one by one
-                File fileToCompare = new File(entry.getName());
-                try (InputStream inputStreamToCompare = new FileInputStream(fileToCompare)) {
+                final File fileToCompare = new File(entry.getName());
+                try (InputStream inputStreamToCompare = Files.newInputStream(fileToCompare.toPath())) {
                     assertArrayEquals(IOUtils.toByteArray(splitInputStream),
                         IOUtils.toByteArray(inputStreamToCompare));
                 }
@@ -690,15 +745,15 @@ public final class ZipTestCase extends AbstractTestCase {
         }
     }
 
-    private void testInputStreamStatistics(String fileName, Map<String, List<Long>> expectedStatistics)
+    private void testInputStreamStatistics(final String fileName, final Map<String, List<Long>> expectedStatistics)
         throws IOException, ArchiveException {
         final File input = getFile(fileName);
 
         final Map<String,List<List<Long>>> actualStatistics = new HashMap<>();
 
         // stream access
-        try (final FileInputStream fis = new FileInputStream(input);
-            final ArchiveInputStream in = new ArchiveStreamFactory().createArchiveInputStream("zip", fis)) {
+        try (final InputStream fis = Files.newInputStream(input.toPath());
+            final ArchiveInputStream in = ArchiveStreamFactory.DEFAULT.createArchiveInputStream("zip", fis)) {
             for (ArchiveEntry entry; (entry = in.getNextEntry()) != null; ) {
                 readStream(in, entry, actualStatistics);
             }
@@ -716,12 +771,12 @@ public final class ZipTestCase extends AbstractTestCase {
         }
 
         // compare statistics of stream / file access
-        for (Map.Entry<String,List<List<Long>>> me : actualStatistics.entrySet()) {
+        for (final Map.Entry<String,List<List<Long>>> me : actualStatistics.entrySet()) {
             assertEquals("Mismatch of stats for: " + me.getKey(),
                          me.getValue().get(0), me.getValue().get(1));
         }
 
-        for (Map.Entry<String, List<Long>> me : expectedStatistics.entrySet()) {
+        for (final Map.Entry<String, List<Long>> me : expectedStatistics.entrySet()) {
             assertEquals("Mismatch of stats with expected value for: " + me.getKey(),
                 me.getValue(), actualStatistics.get(me.getKey()).get(0));
         }
@@ -730,7 +785,9 @@ public final class ZipTestCase extends AbstractTestCase {
     private void readStream(final InputStream in, final ArchiveEntry entry, final Map<String,List<List<Long>>> map) throws IOException {
         final byte[] buf = new byte[4096];
         final InputStreamStatistics stats = (InputStreamStatistics) in;
-        while (in.read(buf) != -1);
+        while (in.read(buf) != -1) {
+            // consume all.
+        }
 
         final String name = entry.getName();
         final List<List<Long>> l;
@@ -746,9 +803,9 @@ public final class ZipTestCase extends AbstractTestCase {
     }
 
     private File getFilesToZip() throws IOException {
-        File originalZipFile = getFile("COMPRESS-477/split_zip_created_by_zip/zip_to_compare_created_by_zip.zip");
+        final File originalZipFile = getFile("COMPRESS-477/split_zip_created_by_zip/zip_to_compare_created_by_zip.zip");
         try (ZipFile zipFile = new ZipFile(originalZipFile)) {
-            Enumeration<ZipArchiveEntry> zipEntries = zipFile.getEntries();
+            final Enumeration<ZipArchiveEntry> zipEntries = zipFile.getEntries();
             ZipArchiveEntry zipEntry;
             File outputFile;
             byte[] buffer;
@@ -767,7 +824,7 @@ public final class ZipTestCase extends AbstractTestCase {
                 outputFile = new File(dir, zipEntry.getName());
 
                 try (InputStream inputStream = zipFile.getInputStream(zipEntry);
-                    OutputStream outputStream = new FileOutputStream(outputFile)) {
+                    OutputStream outputStream = Files.newOutputStream(outputFile.toPath())) {
                     buffer = new byte[(int) zipEntry.getSize()];
                     while ((readLen = inputStream.read(buffer)) > 0) {
                         outputStream.write(buffer, 0, readLen);
@@ -779,39 +836,39 @@ public final class ZipTestCase extends AbstractTestCase {
     }
 
     private void createTestSplitZipSegments() throws IOException {
-        File directoryToZip = getFilesToZip();
-        File outputZipFile = new File(dir, "splitZip.zip");
-        long splitSize = 100 * 1024L; /* 100 KB */
+        final File directoryToZip = getFilesToZip();
+        final File outputZipFile = new File(dir, "splitZip.zip");
+        final long splitSize = 100 * 1024L; /* 100 KB */
         try (final ZipArchiveOutputStream zipArchiveOutputStream = new ZipArchiveOutputStream(outputZipFile,
             splitSize)) {
             addFilesToZip(zipArchiveOutputStream, directoryToZip);
         }
     }
 
-    private void addFilesToZip(ZipArchiveOutputStream zipArchiveOutputStream, File fileToAdd) throws IOException {
+    private void addFilesToZip(final ZipArchiveOutputStream zipArchiveOutputStream, final File fileToAdd) throws IOException {
         if (fileToAdd.isDirectory()) {
-            for (File file : fileToAdd.listFiles()) {
+            for (final File file : fileToAdd.listFiles()) {
                 addFilesToZip(zipArchiveOutputStream, file);
             }
         } else {
-            ZipArchiveEntry zipArchiveEntry = new ZipArchiveEntry(fileToAdd.getPath());
+            final ZipArchiveEntry zipArchiveEntry = new ZipArchiveEntry(fileToAdd.getPath());
             zipArchiveEntry.setMethod(ZipEntry.DEFLATED);
 
             zipArchiveOutputStream.putArchiveEntry(zipArchiveEntry);
-            try (final FileInputStream input = new FileInputStream(fileToAdd)) {
+            try (final InputStream input = Files.newInputStream(fileToAdd.toPath())) {
                 IOUtils.copy(input, zipArchiveOutputStream);
             }
             zipArchiveOutputStream.closeArchiveEntry();
         }
     }
 
-    private int countNonDirectories(File file) {
+    private int countNonDirectories(final File file) {
         if(!file.isDirectory()) {
             return 1;
         }
 
         int result = 0;
-        for (File fileInDirectory : file.listFiles()) {
+        for (final File fileInDirectory : file.listFiles()) {
             result += countNonDirectories(fileInDirectory);
         }
 
