@@ -17,18 +17,29 @@
  */
 package org.apache.commons.compress.archivers.sevenz;
 
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
+import java.nio.file.Path;
 import java.security.NoSuchAlgorithmException;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 import java.util.Random;
 
@@ -49,7 +60,7 @@ public class SevenZFileTest extends AbstractTestCase {
         + " connections>\r\n<meinxml>\r\n\t<leer />\r\n</meinxml>\n";
 
     @Rule
-    public ExpectedException thrown = ExpectedException.none();
+    public final ExpectedException thrown = ExpectedException.none();
 
     // https://issues.apache.org/jira/browse/COMPRESS-320
     @Test
@@ -96,8 +107,7 @@ public class SevenZFileTest extends AbstractTestCase {
                     if (entry.hasStream()) {
                         assertTrue(entriesByName.containsKey(entry.getName()));
                         final byte[] content = readFully(archive);
-                        assertTrue("Content mismatch on: " + fileName + "!" + entry.getName(),
-                            Arrays.equals(content, entriesByName.get(entry.getName())));
+                        assertArrayEquals("Content mismatch on: " + fileName + "!" + entry.getName(), content, entriesByName.get(entry.getName()));
                     }
                 }
 
@@ -150,7 +160,7 @@ public class SevenZFileTest extends AbstractTestCase {
     public void test7zDecryptUnarchive() throws Exception {
         if (isStrongCryptoAvailable()) {
             test7zUnarchive(getFile("bla.encrypted.7z"), SevenZMethod.LZMA, // stack LZMA + AES
-                            "foo".getBytes("UTF-16LE"));
+                            "foo".getBytes(StandardCharsets.UTF_16LE));
         }
     }
 
@@ -163,7 +173,12 @@ public class SevenZFileTest extends AbstractTestCase {
     }
 
     private void test7zUnarchive(final File f, final SevenZMethod m) throws Exception {
-        test7zUnarchive(f, m, (char[]) null);
+        test7zUnarchive(f, m, false);
+    }
+
+    private void test7zUnarchive(final File f, final SevenZMethod m, boolean tryToRecoverBrokenArchives)
+        throws Exception {
+        test7zUnarchive(f, m, (char[]) null, tryToRecoverBrokenArchives);
     }
 
     @Test
@@ -268,7 +283,7 @@ public class SevenZFileTest extends AbstractTestCase {
     @Test
     public void getEntriesOfUnarchiveInMemoryTest() throws IOException {
         byte[] data = null;
-        try (FileInputStream fis = new FileInputStream(getFile("bla.7z"))) {
+        try (InputStream fis = Files.newInputStream(getFile("bla.7z").toPath())) {
             data = IOUtils.toByteArray(fis);
         }
         try (SevenZFile sevenZFile = new SevenZFile(new SeekableInMemoryByteChannel(data))) {
@@ -378,11 +393,11 @@ public class SevenZFileTest extends AbstractTestCase {
     }
 
     /**
-     * @see https://issues.apache.org/jira/browse/COMPRESS-492
+     * @see <a href="https://issues.apache.org/jira/browse/COMPRESS-492">COMPRESS-492</a>
      */
     @Test
     public void handlesEmptyArchiveWithFilesInfo() throws Exception {
-        File f = new File(dir, "empty.7z");
+        final File f = new File(dir, "empty.7z");
         try (SevenZOutputFile s = new SevenZOutputFile(f)) {
         }
         try (SevenZFile z = new SevenZFile(f)) {
@@ -392,7 +407,7 @@ public class SevenZFileTest extends AbstractTestCase {
     }
 
     /**
-     * @see https://issues.apache.org/jira/browse/COMPRESS-492
+     * @see <a href="https://issues.apache.org/jira/browse/COMPRESS-492">COMPRESS-492</a>
      */
     @Test
     public void handlesEmptyArchiveWithoutFilesInfo() throws Exception {
@@ -403,10 +418,16 @@ public class SevenZFileTest extends AbstractTestCase {
     }
 
     @Test
-    public void test7zUnarchiveWithDefectHeader() throws Exception {
+    public void test7zUnarchiveWithDefectHeaderFailsByDefault() throws Exception {
+        thrown.expect(IOException.class);
         test7zUnarchive(getFile("bla.noendheaderoffset.7z"), SevenZMethod.LZMA);
     }
 
+    @Test
+    public void test7zUnarchiveWithDefectHeader() throws Exception {
+        test7zUnarchive(getFile("bla.noendheaderoffset.7z"), SevenZMethod.LZMA, true);
+    }
+
     @Test
     public void extractSpecifiedFile() throws Exception {
         try (SevenZFile sevenZFile = new SevenZFile(getFile("COMPRESS-256.7z"))) {
@@ -421,17 +442,17 @@ public class SevenZFileTest extends AbstractTestCase {
                     "111111111111111111111111111000101011\n" +
                     "111111111111111111111111111000101011";
 
-            for(SevenZArchiveEntry entry : sevenZFile.getEntries()) {
+            for(final SevenZArchiveEntry entry : sevenZFile.getEntries()) {
                 if(entry.getName().equals("commons-compress-1.7-src/src/test/resources/test.txt")) {
                     final byte[] contents = new byte[(int) entry.getSize()];
                     int off = 0;
-                    InputStream inputStream = sevenZFile.getInputStream(entry);
+                    final InputStream inputStream = sevenZFile.getInputStream(entry);
                     while (off < contents.length) {
                         final int bytesRead = inputStream.read(contents, off, contents.length - off);
                         assert (bytesRead >= 0);
                         off += bytesRead;
                     }
-                    assertEquals(testTxtContents, new String(contents, "UTF-8"));
+                    assertEquals(testTxtContents, new String(contents, StandardCharsets.UTF_8));
                     break;
                 }
             }
@@ -473,19 +494,19 @@ public class SevenZFileTest extends AbstractTestCase {
             sevenZFile.getNextEntry();
             sevenZFile.getNextEntry();
 
-            for(SevenZArchiveEntry entry : sevenZFile.getEntries()) {
+            for(final SevenZArchiveEntry entry : sevenZFile.getEntries()) {
                 // commons-compress-1.7-src/src/test/resources/test.txt
                 if(entry.getName().equals("commons-compress-1.7-src/src/test/resources/longsymlink/files.txt")) {
                     contents = new byte[(int) entry.getSize()];
                     off = 0;
-                    InputStream inputStream = sevenZFile.getInputStream(entry);
+                    final InputStream inputStream = sevenZFile.getInputStream(entry);
                     while (off < contents.length) {
                         final int bytesRead = inputStream.read(contents, off, contents.length - off);
                         assert (bytesRead >= 0);
                         off += bytesRead;
                     }
                     assertEquals(SevenZMethod.LZMA2, entry.getContentMethods().iterator().next().getMethod());
-                    assertEquals(filesTxtContents, new String(contents, "UTF-8"));
+                    assertEquals(filesTxtContents, new String(contents, StandardCharsets.UTF_8));
                     break;
                 }
             }
@@ -504,7 +525,7 @@ public class SevenZFileTest extends AbstractTestCase {
                 off += bytesRead;
             }
             assertEquals(SevenZMethod.LZMA2, nextEntry.getContentMethods().iterator().next().getMethod());
-            assertEquals(testTxtContents, new String(contents, "UTF-8"));
+            assertEquals(testTxtContents, new String(contents, StandardCharsets.UTF_8));
         }
     }
 
@@ -531,19 +552,133 @@ public class SevenZFileTest extends AbstractTestCase {
                 }
             }
 
-            sevenZFile.getNextEntry();
-            sevenZFile.getNextEntry();
-            // skip all the entries and jump backwards
-            byte[] contents = new byte[(int) testTxtEntry.getSize()];
+            // read the next entry and jump back using random access
+            final SevenZArchiveEntry entryAfterTestTxtEntry = sevenZFile.getNextEntry();
+            final byte[] entryAfterTestTxtEntryContents = new byte[(int) entryAfterTestTxtEntry.getSize()];
+            int off = 0;
+            while (off < entryAfterTestTxtEntryContents.length) {
+                final int bytesRead = sevenZFile.read(entryAfterTestTxtEntryContents, off, entryAfterTestTxtEntryContents.length - off);
+                assert (bytesRead >= 0);
+                off += bytesRead;
+            }
+
+            // jump backwards
+            final byte[] contents = new byte[(int) testTxtEntry.getSize()];
             try (InputStream inputStream = sevenZFile.getInputStream(testTxtEntry)) {
-                int off = 0;
+                off = 0;
                 while (off < contents.length) {
                     final int bytesRead = inputStream.read(contents, off, contents.length - off);
                     assert (bytesRead >= 0);
                     off += bytesRead;
                 }
                 assertEquals(SevenZMethod.LZMA2, testTxtEntry.getContentMethods().iterator().next().getMethod());
-                assertEquals(testTxtContents, new String(contents, "UTF-8"));
+                assertEquals(testTxtContents, new String(contents, StandardCharsets.UTF_8));
+            }
+
+            // then read the next entry using getNextEntry
+            final SevenZArchiveEntry nextTestTxtEntry = sevenZFile.getNextEntry();
+            final byte[] nextTestContents = new byte[(int) nextTestTxtEntry.getSize()];
+            off = 0;
+            while (off < nextTestContents.length) {
+                final int bytesRead = sevenZFile.read(nextTestContents, off, nextTestContents.length - off);
+                assert (bytesRead >= 0);
+                off += bytesRead;
+            }
+
+            assertEquals(nextTestTxtEntry.getName(), entryAfterTestTxtEntry.getName());
+            assertEquals(nextTestTxtEntry.getSize(), entryAfterTestTxtEntry.getSize());
+            assertArrayEquals(nextTestContents, entryAfterTestTxtEntryContents);
+        }
+    }
+
+    @Test
+    public void testRandomAccessWhenJumpingForwards() throws Exception {
+        try (SevenZFile sevenZFile = new SevenZFile(getFile("COMPRESS-256.7z"))) {
+            final String testTxtContents = "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011";
+
+            SevenZArchiveEntry testTxtEntry = null;
+            final Iterable<SevenZArchiveEntry> entries = sevenZFile.getEntries();
+            final Iterator<SevenZArchiveEntry> iter = entries.iterator();
+            while(iter.hasNext()) {
+                testTxtEntry = iter.next();
+                if (testTxtEntry.getName().equals("commons-compress-1.7-src/src/test/resources/test.txt")) {
+                    break;
+                }
+            }
+            final SevenZArchiveEntry firstEntry = sevenZFile.getNextEntry();
+            // only read some of the data of the first entry
+            byte[] contents = new byte[(int) firstEntry.getSize()/2];
+            sevenZFile.read(contents);
+
+            // and the third entry
+            sevenZFile.getNextEntry();
+            final SevenZArchiveEntry thirdEntry = sevenZFile.getNextEntry();
+            contents = new byte[(int) thirdEntry.getSize()/2];
+            sevenZFile.read(contents);
+
+            // and then read a file after the first entry using random access
+            contents = new byte[(int) testTxtEntry.getSize()];
+            int numberOfReads = 10;
+            while (numberOfReads-- > 0) {
+                try (InputStream inputStream = sevenZFile.getInputStream(testTxtEntry)) {
+                    int off = 0;
+                    while (off < contents.length) {
+                        final int bytesRead = inputStream.read(contents, off, contents.length - off);
+                        assert (bytesRead >= 0);
+                        off += bytesRead;
+                    }
+                    assertEquals(SevenZMethod.LZMA2, testTxtEntry.getContentMethods().iterator().next().getMethod());
+                    assertEquals(testTxtContents, new String(contents, StandardCharsets.UTF_8));
+                }
+            }
+        }
+    }
+
+    @Test
+    public void testRandomAccessMultipleReadSameFile() throws Exception {
+        try (SevenZFile sevenZFile = new SevenZFile(getFile("COMPRESS-256.7z"))) {
+            final String testTxtContents = "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011\n" +
+                    "111111111111111111111111111000101011";
+
+            SevenZArchiveEntry entry;
+            SevenZArchiveEntry testTxtEntry = null;
+            while ((entry = sevenZFile.getNextEntry()) != null) {
+                if (entry.getName().equals("commons-compress-1.7-src/src/test/resources/test.txt")) {
+                    testTxtEntry = entry;
+                    break;
+                }
+            }
+
+            final byte[] contents = new byte[(int) testTxtEntry.getSize()];
+            int numberOfReads = 10;
+            while (numberOfReads-- > 0) {
+                try (InputStream inputStream = sevenZFile.getInputStream(testTxtEntry)) {
+                    int off = 0;
+                    while (off < contents.length) {
+                        final int bytesRead = inputStream.read(contents, off, contents.length - off);
+                        assert (bytesRead >= 0);
+                        off += bytesRead;
+                    }
+                    assertEquals(SevenZMethod.LZMA2, testTxtEntry.getContentMethods().iterator().next().getMethod());
+                    assertEquals(testTxtContents, new String(contents, StandardCharsets.UTF_8));
+                }
             }
         }
     }
@@ -552,13 +687,75 @@ public class SevenZFileTest extends AbstractTestCase {
     public void extractNonExistSpecifiedFile() throws Exception {
         try (SevenZFile sevenZFile = new SevenZFile(getFile("COMPRESS-256.7z"));
             SevenZFile anotherSevenZFile = new SevenZFile(getFile("bla.7z"))) {
-            for (SevenZArchiveEntry nonExistEntry : anotherSevenZFile.getEntries()) {
+            for (final SevenZArchiveEntry nonExistEntry : anotherSevenZFile.getEntries()) {
                 thrown.expect(IllegalArgumentException.class);
                 sevenZFile.getInputStream(nonExistEntry);
             }
         }
     }
 
+    @Test
+    public void retrieveInputStreamForAllEntriesMultipleTimes() throws IOException {
+        try (SevenZFile sevenZFile = new SevenZFile(getFile("bla.7z"))) {
+            for (final SevenZArchiveEntry entry : sevenZFile.getEntries()) {
+                final byte[] firstRead = IOUtils.toByteArray(sevenZFile.getInputStream(entry));
+                final byte[] secondRead = IOUtils.toByteArray(sevenZFile.getInputStream(entry));
+                assertArrayEquals(firstRead, secondRead);
+            }
+        }
+    }
+
+    @Test
+    public void retrieveInputStreamForAllEntriesWithoutCRCMultipleTimes() throws IOException {
+        try (final SevenZOutputFile out = new SevenZOutputFile(new File(dir, "test.7z"))) {
+            final Path inputFile = Files.createTempFile("SevenZTestTemp", "");
+
+            final SevenZArchiveEntry entry = out.createArchiveEntry(inputFile.toFile(), "test.txt");
+            out.putArchiveEntry(entry);
+            out.write("Test".getBytes(StandardCharsets.UTF_8));
+            out.closeArchiveEntry();
+
+            Files.deleteIfExists(inputFile);
+        }
+
+        try (SevenZFile sevenZFile = new SevenZFile(new File(dir, "test.7z"))) {
+            for (final SevenZArchiveEntry entry : sevenZFile.getEntries()) {
+                final byte[] firstRead = IOUtils.toByteArray(sevenZFile.getInputStream(entry));
+                final byte[] secondRead = IOUtils.toByteArray(sevenZFile.getInputStream(entry));
+                assertArrayEquals(firstRead, secondRead);
+            }
+        }
+    }
+
+    @Test
+    public void testNoOOMOnCorruptedHeader() throws IOException {
+        final List<Path> testFiles = new ArrayList<>();
+        testFiles.add(getPath("COMPRESS-542-1.7z"));
+        testFiles.add(getPath("COMPRESS-542-2.7z"));
+        testFiles.add(getPath("COMPRESS-542-endheadercorrupted.7z"));
+        testFiles.add(getPath("COMPRESS-542-endheadercorrupted2.7z"));
+
+        for (final Path file : testFiles) {
+            try (SevenZFile sevenZFile = new SevenZFile(Files.newByteChannel(file),
+                     SevenZFileOptions.builder().withTryToRecoverBrokenArchives(true).build())) {
+                fail("Expected IOException: start header corrupt and unable to guess end header");
+            } catch (final IOException e) {
+                assertEquals("Start header corrupt and unable to guess end header", e.getMessage());
+            }
+        }
+    }
+
+    @Test
+    public void retrieveInputStreamForShuffledEntries() throws IOException {
+        try (final SevenZFile sevenZFile = new SevenZFile(getFile("COMPRESS-348.7z"))) {
+            List<SevenZArchiveEntry> entries = (List<SevenZArchiveEntry>) sevenZFile.getEntries();
+            Collections.shuffle(entries);
+            for (final SevenZArchiveEntry entry : entries) {
+                IOUtils.toByteArray(sevenZFile.getInputStream(entry));
+            }
+        }
+    }
+
     private void test7zUnarchive(final File f, final SevenZMethod m, final byte[] password) throws Exception {
         try (SevenZFile sevenZFile = new SevenZFile(f, password)) {
             test7zUnarchive(sevenZFile, m);
@@ -566,12 +763,18 @@ public class SevenZFileTest extends AbstractTestCase {
     }
 
     private void test7zUnarchive(final File f, final SevenZMethod m, final char[] password) throws Exception {
-        try (SevenZFile sevenZFile = new SevenZFile(f, password)) {
+        test7zUnarchive(f, m, password, false);
+    }
+
+    private void test7zUnarchive(final File f, final SevenZMethod m, final char[] password,
+        final boolean tryToRecoverBrokenArchives) throws Exception {
+        try (SevenZFile sevenZFile = new SevenZFile(f, password,
+                 SevenZFileOptions.builder().withTryToRecoverBrokenArchives(tryToRecoverBrokenArchives).build())) {
             test7zUnarchive(sevenZFile, m);
         }
     }
 
-    private void test7zUnarchive(SevenZFile sevenZFile, final SevenZMethod m) throws Exception {
+    private void test7zUnarchive(final SevenZFile sevenZFile, final SevenZMethod m) throws Exception {
         SevenZArchiveEntry entry = sevenZFile.getNextEntry();
         assertEquals("test1.xml", entry.getName());
         assertEquals(m, entry.getContentMethods().iterator().next().getMethod());
@@ -585,7 +788,7 @@ public class SevenZFileTest extends AbstractTestCase {
             assert (bytesRead >= 0);
             off += bytesRead;
         }
-        assertEquals(TEST2_CONTENT, new String(contents, "UTF-8"));
+        assertEquals(TEST2_CONTENT, new String(contents, StandardCharsets.UTF_8));
         assertNull(sevenZFile.getNextEntry());
     }
 
@@ -600,7 +803,7 @@ public class SevenZFileTest extends AbstractTestCase {
                 assert (bytesRead >= 0);
                 off += bytesRead;
             }
-            assertEquals("Hello, world!\n", new String(contents, "UTF-8"));
+            assertEquals("Hello, world!\n", new String(contents, StandardCharsets.UTF_8));
             assertNull(sevenZFile.getNextEntry());
         }
     }
