@@ -17,11 +17,19 @@
  */
 package org.apache.commons.compress.archivers.sevenz;
 
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+
 import org.junit.Test;
 
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Files;
+import java.nio.file.Paths;
 import java.util.Arrays;
 import java.util.ArrayList;
 import java.util.Calendar;
@@ -29,6 +37,7 @@ import java.util.Collections;
 import java.util.Date;
 import java.util.Iterator;
 import org.apache.commons.compress.AbstractTestCase;
+import org.apache.commons.compress.utils.ByteUtils;
 import org.apache.commons.compress.utils.SeekableInMemoryByteChannel;
 import org.tukaani.xz.LZMA2Options;
 
@@ -73,7 +82,39 @@ public class SevenZOutputFileTest extends AbstractTestCase {
             entry.setCreationDate(creationDate);
             entry.setAccessDate(accessDate);
             outArchive.putArchiveEntry(entry);
-            outArchive.write(new byte[0]);
+            outArchive.write(ByteUtils.EMPTY_BYTE_ARRAY);
+            outArchive.closeArchiveEntry();
+
+            entry = new SevenZArchiveEntry();
+            entry.setName("foo/bar/boo0");
+            entry.setCreationDate(creationDate);
+            entry.setAccessDate(accessDate);
+            outArchive.putArchiveEntry(entry);
+            outArchive.write(new ByteArrayInputStream(ByteUtils.EMPTY_BYTE_ARRAY));
+            outArchive.closeArchiveEntry();
+
+            entry = new SevenZArchiveEntry();
+            entry.setName("foo/bar/boo1");
+            entry.setCreationDate(creationDate);
+            entry.setAccessDate(accessDate);
+            outArchive.putArchiveEntry(entry);
+            outArchive.write(new ByteArrayInputStream(new byte[] {'a'}));
+            outArchive.closeArchiveEntry();
+
+            entry = new SevenZArchiveEntry();
+            entry.setName("foo/bar/boo10000");
+            entry.setCreationDate(creationDate);
+            entry.setAccessDate(accessDate);
+            outArchive.putArchiveEntry(entry);
+            outArchive.write(new ByteArrayInputStream(new byte[10000]));
+            outArchive.closeArchiveEntry();
+
+            entry = new SevenZArchiveEntry();
+            entry.setName("foo/bar/test.txt");
+            entry.setCreationDate(creationDate);
+            entry.setAccessDate(accessDate);
+            outArchive.putArchiveEntry(entry);
+            outArchive.write(Paths.get("src/test/resources/test.txt"));
             outArchive.closeArchiveEntry();
 
             entry = new SevenZArchiveEntry();
@@ -87,6 +128,11 @@ public class SevenZOutputFileTest extends AbstractTestCase {
             outArchive.putArchiveEntry(entry);
             outArchive.closeArchiveEntry();
 
+            entry = outArchive.createArchiveEntry(dir.toPath(), "baz2/");
+            entry.setAntiItem(true);
+            outArchive.putArchiveEntry(entry);
+            outArchive.closeArchiveEntry();
+
             entry = new SevenZArchiveEntry();
             entry.setName("dada");
             entry.setHasWindowsAttributes(true);
@@ -116,6 +162,46 @@ public class SevenZOutputFileTest extends AbstractTestCase {
             assertEquals(accessDate, entry.getAccessDate());
             assertEquals(creationDate, entry.getCreationDate());
 
+            entry = archive.getNextEntry();
+            assert (entry != null);
+            assertEquals("foo/bar/boo0", entry.getName());
+            assertFalse(entry.isDirectory());
+            assertFalse(entry.isAntiItem());
+            assertEquals(0, entry.getSize());
+            assertFalse(entry.getHasLastModifiedDate());
+            assertEquals(accessDate, entry.getAccessDate());
+            assertEquals(creationDate, entry.getCreationDate());
+
+            entry = archive.getNextEntry();
+            assert (entry != null);
+            assertEquals("foo/bar/boo1", entry.getName());
+            assertFalse(entry.isDirectory());
+            assertFalse(entry.isAntiItem());
+            assertEquals(1, entry.getSize());
+            assertFalse(entry.getHasLastModifiedDate());
+            assertEquals(accessDate, entry.getAccessDate());
+            assertEquals(creationDate, entry.getCreationDate());
+
+            entry = archive.getNextEntry();
+            assert (entry != null);
+            assertEquals("foo/bar/boo10000", entry.getName());
+            assertFalse(entry.isDirectory());
+            assertFalse(entry.isAntiItem());
+            assertEquals(10000, entry.getSize());
+            assertFalse(entry.getHasLastModifiedDate());
+            assertEquals(accessDate, entry.getAccessDate());
+            assertEquals(creationDate, entry.getCreationDate());
+
+            entry = archive.getNextEntry();
+            assert (entry != null);
+            assertEquals("foo/bar/test.txt", entry.getName());
+            assertFalse(entry.isDirectory());
+            assertFalse(entry.isAntiItem());
+            assertEquals(Files.size(Paths.get("src/test/resources/test.txt")), entry.getSize());
+            assertFalse(entry.getHasLastModifiedDate());
+            assertEquals(accessDate, entry.getAccessDate());
+            assertEquals(creationDate, entry.getCreationDate());
+
             entry = archive.getNextEntry();
             assert (entry != null);
             assertEquals("xyzzy", entry.getName());
@@ -130,6 +216,12 @@ public class SevenZOutputFileTest extends AbstractTestCase {
             assertTrue(entry.isDirectory());
             assertTrue(entry.isAntiItem());
 
+            entry = archive.getNextEntry();
+            assert (entry != null);
+            assertEquals("baz2/", entry.getName());
+            assertTrue(entry.isDirectory());
+            assertTrue(entry.isAntiItem());
+
             entry = archive.getNextEntry();
             assert (entry != null);
             assertEquals("dada", entry.getName());
@@ -450,7 +542,7 @@ public class SevenZOutputFileTest extends AbstractTestCase {
             return null;
         }
         assertEquals("foo/" + index + ".txt", entry.getName());
-        assertEquals(false, entry.isDirectory());
+        assertFalse(entry.isDirectory());
         if (entry.getSize() == 0) {
             return Boolean.FALSE;
         }
