@@ -28,12 +28,13 @@ import static org.junit.Assert.fail;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.OutputStream;
 import java.io.Reader;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
 import java.security.MessageDigest;
 import java.util.Calendar;
 import java.util.Date;
@@ -44,7 +45,6 @@ import org.apache.commons.compress.AbstractTestCase;
 import org.apache.commons.compress.archivers.ArchiveEntry;
 import org.apache.commons.compress.archivers.ArchiveOutputStream;
 import org.apache.commons.compress.archivers.ArchiveStreamFactory;
-import org.apache.commons.compress.utils.CharsetNames;
 import org.apache.commons.compress.utils.IOUtils;
 import org.junit.Assert;
 import org.junit.Test;
@@ -55,16 +55,16 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
     public void testCount() throws Exception {
         final File f = File.createTempFile("commons-compress-tarcount", ".tar");
         f.deleteOnExit();
-        final FileOutputStream fos = new FileOutputStream(f);
+        final OutputStream fos = Files.newOutputStream(f.toPath());
 
-        final ArchiveOutputStream tarOut = new ArchiveStreamFactory()
+        final ArchiveOutputStream tarOut = ArchiveStreamFactory.DEFAULT
             .createArchiveOutputStream(ArchiveStreamFactory.TAR, fos);
 
         final File file1 = getFile("test1.xml");
         final TarArchiveEntry sEntry = new TarArchiveEntry(file1, file1.getName());
         tarOut.putArchiveEntry(sEntry);
 
-        final FileInputStream in = new FileInputStream(file1);
+        final InputStream in = Files.newInputStream(file1.toPath());
         final byte[] buf = new byte[8192];
 
         int read = 0;
@@ -95,6 +95,15 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         }
     }
 
+    @Test(expected = IllegalArgumentException.class)
+    public void testBigNumberErrorMode() throws Exception {
+        final TarArchiveEntry t = new TarArchiveEntry("foo");
+        t.setSize(0100000000000L);
+        final ByteArrayOutputStream bos = new ByteArrayOutputStream();
+        final TarArchiveOutputStream tos = new TarArchiveOutputStream(bos);
+        tos.putArchiveEntry(t);
+    }
+
     @Test
     public void testBigNumberStarMode() throws Exception {
         final TarArchiveEntry t = new TarArchiveEntry("foo");
@@ -138,7 +147,7 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
                     + TarConstants.MODELEN
                     + TarConstants.UIDLEN
                     + TarConstants.GIDLEN, 12,
-                CharsetNames.UTF_8));
+                    StandardCharsets.UTF_8));
         final TarArchiveInputStream tin =
             new TarArchiveInputStream(new ByteArrayInputStream(data));
         final TarArchiveEntry e = tin.getNextTarEntry();
@@ -159,8 +168,8 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
                 + TarConstants.MODELEN
                 + TarConstants.UIDLEN
                 + TarConstants.GIDLEN, 12,
-                CharsetNames.UTF_8));
-        assertEquals("6 a=b\n", new String(data, 512, 6, CharsetNames.UTF_8));
+                    StandardCharsets.UTF_8));
+        assertEquals("6 a=b\n", new String(data, 512, 6, StandardCharsets.UTF_8));
     }
 
     @Test
@@ -176,10 +185,10 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
                 + TarConstants.MODELEN
                 + TarConstants.UIDLEN
                 + TarConstants.GIDLEN, 12,
-                CharsetNames.UTF_8));
+                    StandardCharsets.UTF_8));
         assertEquals("99 a=0123456789012345678901234567890123456789"
             + "01234567890123456789012345678901234567890123456789"
-            + "012\n", new String(data, 512, 99, CharsetNames.UTF_8));
+            + "012\n", new String(data, 512, 99, StandardCharsets.UTF_8));
     }
 
     @Test
@@ -195,10 +204,10 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
                 + TarConstants.MODELEN
                 + TarConstants.UIDLEN
                 + TarConstants.GIDLEN, 12,
-                CharsetNames.UTF_8));
+                    StandardCharsets.UTF_8));
         assertEquals("101 a=0123456789012345678901234567890123456789"
             + "01234567890123456789012345678901234567890123456789"
-            + "0123\n", new String(data, 512, 101, CharsetNames.UTF_8));
+            + "0123\n", new String(data, 512, 101, StandardCharsets.UTF_8));
     }
 
     private byte[] writePaxHeader(final Map<String, String> m) throws Exception {
@@ -233,7 +242,7 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         tos.closeArchiveEntry();
         final byte[] data = bos.toByteArray();
         assertEquals("160 path=" + n + "\n",
-            new String(data, 512, 160, CharsetNames.UTF_8));
+            new String(data, 512, 160, StandardCharsets.UTF_8));
         final TarArchiveInputStream tin =
             new TarArchiveInputStream(new ByteArrayInputStream(data));
         final TarArchiveEntry e = tin.getNextTarEntry();
@@ -292,7 +301,7 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
                     + TarConstants.UIDLEN
                     + TarConstants.GIDLEN
                     + TarConstants.SIZELEN, 12,
-                CharsetNames.UTF_8));
+                    StandardCharsets.UTF_8));
         final TarArchiveInputStream tin =
             new TarArchiveInputStream(new ByteArrayInputStream(data));
         final TarArchiveEntry e = tin.getNextTarEntry();
@@ -335,7 +344,7 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         tos.close();
         final byte[] data = bos.toByteArray();
         assertEquals("11 path=" + n + "\n",
-            new String(data, 512, 11, CharsetNames.UTF_8));
+            new String(data, 512, 11, StandardCharsets.UTF_8));
         final TarArchiveInputStream tin =
             new TarArchiveInputStream(new ByteArrayInputStream(data));
         final TarArchiveEntry e = tin.getNextTarEntry();
@@ -358,7 +367,7 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         tos.close();
         final byte[] data = bos.toByteArray();
         assertEquals("15 linkpath=" + n + "\n",
-            new String(data, 512, 15, CharsetNames.UTF_8));
+            new String(data, 512, 15, StandardCharsets.UTF_8));
         final TarArchiveInputStream tin =
             new TarArchiveInputStream(new ByteArrayInputStream(data));
         final TarArchiveEntry e = tin.getNextTarEntry();
@@ -617,17 +626,17 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
     @SuppressWarnings("deprecation")
     @Test public void testRecordSize() throws IOException {
         try {
-            TarArchiveOutputStream tos =
+            final TarArchiveOutputStream tos =
                 new TarArchiveOutputStream(new ByteArrayOutputStream(),512,511);
             fail("should have rejected recordSize of 511");
-        } catch(IllegalArgumentException e) {
+        } catch(final IllegalArgumentException e) {
             // expected;
         }
         try {
-            TarArchiveOutputStream tos =
+            final TarArchiveOutputStream tos =
                 new TarArchiveOutputStream(new ByteArrayOutputStream(),512,511,null);
             fail("should have rejected recordSize of 511");
-        } catch(IllegalArgumentException e) {
+        } catch(final IllegalArgumentException e) {
             // expected;
         }
         try (TarArchiveOutputStream tos = new TarArchiveOutputStream(new ByteArrayOutputStream(),
@@ -641,7 +650,7 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
     }
     @Test
     public void testBlockSizes() throws Exception {
-        String fileName = "/test1.xml";
+        final String fileName = "/test1.xml";
         byte[] contents = getResourceContents(fileName);
         testPadding(TarConstants.DEFAULT_BLKSIZE, fileName, contents); // USTAR / pre-pax
         testPadding(5120, fileName, contents); // PAX default
@@ -650,13 +659,13 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         try {
             testPadding(511, fileName, contents);    // don't specify a block size -> use minimum length
             fail("should have thrown an illegal argument exception");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             //expected
         }
         try {
             testPadding(0, fileName, contents);    // don't specify a block size -> use minimum length
             fail("should have thrown an illegal argument exception");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             //expected
         }
         // test with "content" that is an exact multiple of record length
@@ -665,10 +674,10 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         testPadding(TarConstants.DEFAULT_BLKSIZE, fileName, contents);
     }
 
-    private void testPadding(int blockSize, String fileName, byte[] contents) throws IOException {
+    private void testPadding(int blockSize, final String fileName, final byte[] contents) throws IOException {
         final File f = File.createTempFile("commons-compress-padding", ".tar");
         f.deleteOnExit();
-        final FileOutputStream fos = new FileOutputStream(f);
+        final OutputStream fos = Files.newOutputStream(f.toPath());
         final TarArchiveOutputStream tos;
         if (blockSize != -2) {
             tos = new TarArchiveOutputStream(fos, blockSize);
@@ -683,15 +692,15 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         tos.write(contents);
         tos.closeArchiveEntry();
         tos.close();
-        int fileRecordsSize = (int) Math.ceil((double) contents.length / 512) * 512;
+        final int fileRecordsSize = (int) Math.ceil((double) contents.length / 512) * 512;
         final int headerSize = 512;
         final int endOfArchiveSize = 1024;
-        int unpaddedSize = headerSize + fileRecordsSize + endOfArchiveSize;
-        int paddedSize = (int) Math.ceil((double)unpaddedSize/blockSize)*blockSize;
+        final int unpaddedSize = headerSize + fileRecordsSize + endOfArchiveSize;
+        final int paddedSize = (int) Math.ceil((double)unpaddedSize/blockSize)*blockSize;
         assertEquals(paddedSize, f.length());
     }
 
-    private byte[] getResourceContents(String name) throws IOException {
+    private byte[] getResourceContents(final String name) throws IOException {
         ByteArrayOutputStream bos;
         try (InputStream resourceAsStream = getClass().getResourceAsStream(name)) {
             bos = new ByteArrayOutputStream();
@@ -700,35 +709,35 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         return bos.toByteArray();
     }
     @Test public void testPutGlobalPaxHeaderEntry() throws IOException {
-        ByteArrayOutputStream bos = new ByteArrayOutputStream();
-        TarArchiveOutputStream tos = new TarArchiveOutputStream(bos);
-        int pid = 73;
-        int globCount = 1;
-        byte lfPaxGlobalExtendedHeader = TarConstants.LF_PAX_GLOBAL_EXTENDED_HEADER;
-        TarArchiveEntry globalHeader = new TarArchiveEntry("/tmp/GlobalHead." + pid + "." + globCount,
+        final ByteArrayOutputStream bos = new ByteArrayOutputStream();
+        final TarArchiveOutputStream tos = new TarArchiveOutputStream(bos);
+        final int pid = 73;
+        final int globCount = 1;
+        final byte lfPaxGlobalExtendedHeader = TarConstants.LF_PAX_GLOBAL_EXTENDED_HEADER;
+        final TarArchiveEntry globalHeader = new TarArchiveEntry("/tmp/GlobalHead." + pid + "." + globCount,
             lfPaxGlobalExtendedHeader);
         globalHeader.addPaxHeader("SCHILLY.xattr.user.org.apache.weasels","global-weasels");
         tos.putArchiveEntry(globalHeader);
         TarArchiveEntry entry = new TarArchiveEntry("message");
-        String x = "If at first you don't succeed, give up";
+        final String x = "If at first you don't succeed, give up";
         entry.setSize(x.length());
         tos.putArchiveEntry(entry);
         tos.write(x.getBytes());
         tos.closeArchiveEntry();
         entry = new TarArchiveEntry("counter-message");
-        String y = "Nothing succeeds like excess";
+        final String y = "Nothing succeeds like excess";
         entry.setSize(y.length());
         entry.addPaxHeader("SCHILLY.xattr.user.org.apache.weasels.species","unknown");
         tos.putArchiveEntry(entry);
         tos.write(y.getBytes());
         tos.closeArchiveEntry();
         tos.close();
-        TarArchiveInputStream in = new TarArchiveInputStream(new ByteArrayInputStream(bos.toByteArray()));
+        final TarArchiveInputStream in = new TarArchiveInputStream(new ByteArrayInputStream(bos.toByteArray()));
         TarArchiveEntry entryIn = in.getNextTarEntry();
         assertNotNull(entryIn);
         assertEquals("message",entryIn.getName());
         assertEquals("global-weasels",entryIn.getExtraPaxHeader("SCHILLY.xattr.user.org.apache.weasels"));
-        Reader reader = new InputStreamReader(in);
+        final Reader reader = new InputStreamReader(in);
         for(int i=0;i<x.length();i++) {
             assertEquals(x.charAt(i),reader.read());
         }
@@ -748,17 +757,16 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
     @Test
     public void testLongNameMd5Hash() throws Exception {
         final String longFileName = "a/considerably/longer/file/name/which/forces/use/of/the/long/link/header/which/appears/to/always/use/the/current/time/as/modification/date";
-        final String fname = longFileName;
         final Date modificationDate = new Date();
 
-        final byte[] archive1 = createTarArchiveContainingOneDirectory(fname, modificationDate);
+        final byte[] archive1 = createTarArchiveContainingOneDirectory(longFileName, modificationDate);
         final byte[] digest1 = MessageDigest.getInstance("MD5").digest(archive1);
 
         // let a second elapse otherwise the modification dates will be equal
         Thread.sleep(1000L);
 
         // now recreate exactly the same tar file
-        final byte[] archive2 = createTarArchiveContainingOneDirectory(fname, modificationDate);
+        final byte[] archive2 = createTarArchiveContainingOneDirectory(longFileName, modificationDate);
         // and I would expect the MD5 hash to be the same, but for long names it isn't
         final byte[] digest2 = MessageDigest.getInstance("MD5").digest(archive2);
 
@@ -777,6 +785,16 @@ public class TarArchiveOutputStreamTest extends AbstractTestCase {
         tarIn.close();
     }
 
+    @Test(expected = IllegalArgumentException.class)
+    public void testWriteLongFileNameThrowsException() throws Exception {
+        final String n = "01234567890123456789012345678901234567890123456789"
+                + "01234567890123456789012345678901234567890123456789"
+                + "01234567890123456789012345678901234567890123456789";
+        final TarArchiveEntry t = new TarArchiveEntry(n);
+        final TarArchiveOutputStream tos = new TarArchiveOutputStream(new ByteArrayOutputStream(), "ASCII");
+        tos.putArchiveEntry(t);
+    }
+
     private static byte[] createTarArchiveContainingOneDirectory(final String fname,
         final Date modificationDate) throws IOException {
         final ByteArrayOutputStream baos = new ByteArrayOutputStream();
