@@ -19,26 +19,21 @@
 
 package org.apache.commons.compress.archivers.zip;
 
-import static org.junit.Assert.*;
-
 import java.io.ByteArrayInputStream;
 
-import org.junit.Test;
+import junit.framework.TestCase;
 
-public class BitStreamTest {
+public class BitStreamTest extends TestCase {
 
-    @Test
     public void testEmptyStream() throws Exception {
-        final BitStream stream = new BitStream(new ByteArrayInputStream(new byte[0]));
+        BitStream stream = new BitStream(new ByteArrayInputStream(new byte[0]));
         assertEquals("next bit", -1, stream.nextBit());
         assertEquals("next bit", -1, stream.nextBit());
         assertEquals("next bit", -1, stream.nextBit());
-        stream.close();
     }
 
-    @Test
     public void testStream() throws Exception {
-        final BitStream stream = new BitStream(new ByteArrayInputStream(new byte[] { (byte) 0xEA, 0x03 }));
+        BitStream stream = new BitStream(new ByteArrayInputStream(new byte[] { (byte) 0xEA, 0x03 }));
 
         assertEquals("bit 0", 0, stream.nextBit());
         assertEquals("bit 1", 1, stream.nextBit());
@@ -57,38 +52,31 @@ public class BitStreamTest {
         assertEquals("bit 13", 0, stream.nextBit());
         assertEquals("bit 14", 0, stream.nextBit());
         assertEquals("bit 15", 0, stream.nextBit());
-
+        
         assertEquals("next bit", -1, stream.nextBit());
-        stream.close();
     }
 
-    @Test
     public void testNextByteFromEmptyStream() throws Exception {
-        final BitStream stream = new BitStream(new ByteArrayInputStream(new byte[0]));
+        BitStream stream = new BitStream(new ByteArrayInputStream(new byte[0]));
         assertEquals("next byte", -1, stream.nextByte());
         assertEquals("next byte", -1, stream.nextByte());
-        stream.close();
     }
 
-    @Test
     public void testReadAlignedBytes() throws Exception {
-        final BitStream stream = new BitStream(new ByteArrayInputStream(new byte[] { (byte) 0xEA, 0x35 }));
+        BitStream stream = new BitStream(new ByteArrayInputStream(new byte[] { (byte) 0xEA, 0x35 }));
         assertEquals("next byte", 0xEA, stream.nextByte());
         assertEquals("next byte", 0x35, stream.nextByte());
         assertEquals("next byte", -1, stream.nextByte());
-        stream.close();
     }
 
-    @Test
     public void testNextByte() throws Exception {
-        final BitStream stream = new BitStream(new ByteArrayInputStream(new byte[] { (byte) 0xEA, 0x35 }));
+        BitStream stream = new BitStream(new ByteArrayInputStream(new byte[] { (byte) 0xEA, 0x35 }));
         assertEquals("bit 0", 0, stream.nextBit());
         assertEquals("bit 1", 1, stream.nextBit());
         assertEquals("bit 2", 0, stream.nextBit());
         assertEquals("bit 3", 1, stream.nextBit());
-
+        
         assertEquals("next byte", 0x5E, stream.nextByte());
         assertEquals("next byte", -1, stream.nextByte()); // not enough bits left to read a byte
-        stream.close();
     }
 }
