@@ -29,17 +29,22 @@ import static org.junit.Assert.fail;
 
 import java.io.BufferedInputStream;
 import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.EOFException;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.nio.channels.Channels;
 import java.nio.channels.SeekableByteChannel;
+import java.nio.file.Files;
 import java.util.Arrays;
+import java.util.zip.ZipEntry;
 import java.util.zip.ZipException;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
+import org.apache.commons.compress.archivers.ArchiveInputStream;
+import org.apache.commons.compress.archivers.ArchiveStreamFactory;
+import org.apache.commons.compress.utils.ByteUtils;
 import org.apache.commons.compress.utils.IOUtils;
 import org.junit.Assert;
 import org.junit.Rule;
@@ -49,24 +54,18 @@ import org.junit.rules.ExpectedException;
 public class ZipArchiveInputStreamTest {
 
     @Rule
-    public ExpectedException thrown = ExpectedException.none();
+    public final ExpectedException thrown = ExpectedException.none();
 
     /**
      * @see "https://issues.apache.org/jira/browse/COMPRESS-176"
      */
     @Test
     public void winzipBackSlashWorkaround() throws Exception {
-        ZipArchiveInputStream in = null;
-        try {
-            in = new ZipArchiveInputStream(new FileInputStream(getFile("test-winzip.zip")));
+        try (ZipArchiveInputStream in = new ZipArchiveInputStream(Files.newInputStream(getFile("test-winzip.zip").toPath()))) {
             ZipArchiveEntry zae = in.getNextZipEntry();
             zae = in.getNextZipEntry();
             zae = in.getNextZipEntry();
             assertEquals("\u00e4/", zae.getName());
-        } finally {
-            if (in != null) {
-                in.close();
-            }
         }
     }
 
@@ -119,14 +118,8 @@ public class ZipArchiveInputStreamTest {
      */
     @Test
     public void shouldReadNestedZip() throws IOException {
-        ZipArchiveInputStream in = null;
-        try {
-            in = new ZipArchiveInputStream(new FileInputStream(getFile("COMPRESS-219.zip")));
+        try (ZipArchiveInputStream in = new ZipArchiveInputStream(Files.newInputStream(getFile("COMPRESS-219.zip").toPath()))) {
             extractZipInputStream(in);
-        } finally {
-            if (in != null) {
-                in.close();
-            }
         }
     }
 
@@ -143,13 +136,13 @@ public class ZipArchiveInputStreamTest {
 
     @Test
     public void testUnshrinkEntry() throws Exception {
-        final ZipArchiveInputStream in = new ZipArchiveInputStream(new FileInputStream(getFile("SHRUNK.ZIP")));
+        final ZipArchiveInputStream in = new ZipArchiveInputStream(Files.newInputStream(getFile("SHRUNK.ZIP").toPath()));
 
         ZipArchiveEntry entry = in.getNextZipEntry();
         assertEquals("method", ZipMethod.UNSHRINKING.getCode(), entry.getMethod());
         assertTrue(in.canReadEntryData(entry));
 
-        FileInputStream original = new FileInputStream(getFile("test1.xml"));
+        InputStream original = Files.newInputStream(getFile("test1.xml").toPath());
         try {
             assertArrayEquals(IOUtils.toByteArray(original), IOUtils.toByteArray(in));
         } finally {
@@ -160,7 +153,7 @@ public class ZipArchiveInputStreamTest {
         assertEquals("method", ZipMethod.UNSHRINKING.getCode(), entry.getMethod());
         assertTrue(in.canReadEntryData(entry));
 
-        original = new FileInputStream(getFile("test2.xml"));
+        original = Files.newInputStream(getFile("test2.xml").toPath());
         try {
             assertArrayEquals(IOUtils.toByteArray(original), IOUtils.toByteArray(in));
         } finally {
@@ -177,7 +170,7 @@ public class ZipArchiveInputStreamTest {
     @Test
     public void testReadingOfFirstStoredEntry() throws Exception {
 
-        try (ZipArchiveInputStream in = new ZipArchiveInputStream(new FileInputStream(getFile("COMPRESS-264.zip")))) {
+        try (ZipArchiveInputStream in = new ZipArchiveInputStream(Files.newInputStream(getFile("COMPRESS-264.zip").toPath()))) {
             final ZipArchiveEntry ze = in.getNextZipEntry();
             assertEquals(5, ze.getSize());
             assertArrayEquals(new byte[] { 'd', 'a', 't', 'a', '\n' },
@@ -192,7 +185,7 @@ public class ZipArchiveInputStreamTest {
      */
     @Test
     public void testMessageWithCorruptFileName() throws Exception {
-        try (ZipArchiveInputStream in = new ZipArchiveInputStream(new FileInputStream(getFile("COMPRESS-351.zip")))) {
+        try (ZipArchiveInputStream in = new ZipArchiveInputStream(Files.newInputStream(getFile("COMPRESS-351.zip").toPath()))) {
             ZipArchiveEntry ze = in.getNextZipEntry();
             while (ze != null) {
                 ze = in.getNextZipEntry();
@@ -207,7 +200,7 @@ public class ZipArchiveInputStreamTest {
     @Test
     public void testUnzipBZip2CompressedEntry() throws Exception {
 
-        try (ZipArchiveInputStream in = new ZipArchiveInputStream(new FileInputStream(getFile("bzip2-zip.zip")))) {
+        try (ZipArchiveInputStream in = new ZipArchiveInputStream(Files.newInputStream(getFile("bzip2-zip.zip").toPath()))) {
             final ZipArchiveEntry ze = in.getNextZipEntry();
             assertEquals(42, ze.getSize());
             final byte[] expected = new byte[42];
@@ -223,11 +216,11 @@ public class ZipArchiveInputStreamTest {
     public void readDeflate64CompressedStream() throws Exception {
         final File input = getFile("COMPRESS-380/COMPRESS-380-input");
         final File archive = getFile("COMPRESS-380/COMPRESS-380.zip");
-        try (FileInputStream in = new FileInputStream(input);
-             ZipArchiveInputStream zin = new ZipArchiveInputStream(new FileInputStream(archive))) {
-            byte[] orig = IOUtils.toByteArray(in);
-            ZipArchiveEntry e = zin.getNextZipEntry();
-            byte[] fromZip = IOUtils.toByteArray(zin);
+        try (InputStream in = Files.newInputStream(input.toPath());
+             ZipArchiveInputStream zin = new ZipArchiveInputStream(Files.newInputStream(archive.toPath()))) {
+            final byte[] orig = IOUtils.toByteArray(in);
+            final ZipArchiveEntry e = zin.getNextZipEntry();
+            final byte[] fromZip = IOUtils.toByteArray(zin);
             assertArrayEquals(orig, fromZip);
         }
     }
@@ -236,12 +229,12 @@ public class ZipArchiveInputStreamTest {
     public void readDeflate64CompressedStreamWithDataDescriptor() throws Exception {
         // this is a copy of bla.jar with META-INF/MANIFEST.MF's method manually changed to ENHANCED_DEFLATED
         final File archive = getFile("COMPRESS-380/COMPRESS-380-dd.zip");
-        try (ZipArchiveInputStream zin = new ZipArchiveInputStream(new FileInputStream(archive))) {
-            ZipArchiveEntry e = zin.getNextZipEntry();
+        try (ZipArchiveInputStream zin = new ZipArchiveInputStream(Files.newInputStream(archive.toPath()))) {
+            final ZipArchiveEntry e = zin.getNextZipEntry();
             assertEquals(-1, e.getSize());
             assertEquals(ZipMethod.ENHANCED_DEFLATED.getCode(), e.getMethod());
-            byte[] fromZip = IOUtils.toByteArray(zin);
-            byte[] expected = new byte[] {
+            final byte[] fromZip = IOUtils.toByteArray(zin);
+            final byte[] expected = new byte[] {
                 'M', 'a', 'n', 'i', 'f', 'e', 's', 't', '-', 'V', 'e', 'r', 's', 'i', 'o', 'n', ':', ' ', '1', '.', '0',
                 '\r', '\n', '\r', '\n'
             };
@@ -261,9 +254,8 @@ public class ZipArchiveInputStreamTest {
         final int expectedNumEntries = 2;
         final InputStream is = ZipArchiveInputStreamTest.class
                 .getResourceAsStream("/archive_with_bytes_after_data.zip");
-        final ZipArchiveInputStream zip = new ZipArchiveInputStream(is);
 
-        try {
+        try (final ZipArchiveInputStream zip = new ZipArchiveInputStream(is)) {
             int actualNumEntries = 0;
             ZipArchiveEntry zae = zip.getNextZipEntry();
             while (zae != null) {
@@ -272,8 +264,6 @@ public class ZipArchiveInputStreamTest {
                 zae = zip.getNextZipEntry();
             }
             assertEquals(expectedNumEntries, actualNumEntries);
-        } finally {
-            zip.close();
         }
     }
 
@@ -285,15 +275,12 @@ public class ZipArchiveInputStreamTest {
     public void testThrowOnInvalidEntry() throws Exception {
         final InputStream is = ZipArchiveInputStreamTest.class
                 .getResourceAsStream("/invalid-zip.zip");
-        final ZipArchiveInputStream zip = new ZipArchiveInputStream(is);
 
-        try {
+        try (final ZipArchiveInputStream zip = new ZipArchiveInputStream(is)) {
             zip.getNextZipEntry();
             fail("IOException expected");
-        } catch (ZipException expected) {
+        } catch (final ZipException expected) {
             assertTrue(expected.getMessage().contains("Unexpected record signature"));
-        } finally {
-            zip.close();
         }
     }
 
@@ -306,11 +293,11 @@ public class ZipArchiveInputStreamTest {
         try (InputStream archiveStream = ZipArchiveInputStream.class.getResourceAsStream("/mixed.zip");
              ZipArchiveInputStream zipStream =  new ZipArchiveInputStream((archiveStream))
         ) {
-            ZipArchiveEntry inflatedEntry = zipStream.getNextZipEntry();
+            final ZipArchiveEntry inflatedEntry = zipStream.getNextZipEntry();
             Assert.assertEquals("inflated.txt", inflatedEntry.getName());
             Assert.assertEquals(0x0000, inflatedEntry.getLocalHeaderOffset());
             Assert.assertEquals(0x0046, inflatedEntry.getDataOffset());
-            ZipArchiveEntry storedEntry = zipStream.getNextZipEntry();
+            final ZipArchiveEntry storedEntry = zipStream.getNextZipEntry();
             Assert.assertEquals("stored.txt", storedEntry.getName());
             Assert.assertEquals(0x5892, storedEntry.getLocalHeaderOffset());
             Assert.assertEquals(0x58d6, storedEntry.getDataOffset());
@@ -338,8 +325,8 @@ public class ZipArchiveInputStreamTest {
     @Test
     public void properlyMarksEntriesAsUnreadableIfUncompressedSizeIsUnknown() throws Exception {
         // we never read any data
-        try (ZipArchiveInputStream zis = new ZipArchiveInputStream(new ByteArrayInputStream(new byte[0]))) {
-            ZipArchiveEntry e = new ZipArchiveEntry("test");
+        try (ZipArchiveInputStream zis = new ZipArchiveInputStream(new ByteArrayInputStream(ByteUtils.EMPTY_BYTE_ARRAY))) {
+            final ZipArchiveEntry e = new ZipArchiveEntry("test");
             e.setMethod(ZipMethod.DEFLATED.getCode());
             assertTrue(zis.canReadEntryData(e));
             e.setMethod(ZipMethod.ENHANCED_DEFLATED.getCode());
@@ -379,10 +366,10 @@ public class ZipArchiveInputStreamTest {
         singleByteReadConsistentlyReturnsMinusOneAtEof(getFile("bzip2-zip.zip"));
     }
 
-    private void singleByteReadConsistentlyReturnsMinusOneAtEof(File file) throws Exception {
-        try (FileInputStream in = new FileInputStream(file);
+    private void singleByteReadConsistentlyReturnsMinusOneAtEof(final File file) throws Exception {
+        try (InputStream in = Files.newInputStream(file.toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(in)) {
-            ArchiveEntry e = archive.getNextEntry();
+            final ArchiveEntry e = archive.getNextEntry();
             IOUtils.toByteArray(archive);
             assertEquals(-1, archive.read());
             assertEquals(-1, archive.read());
@@ -419,11 +406,11 @@ public class ZipArchiveInputStreamTest {
         multiByteReadConsistentlyReturnsMinusOneAtEof(getFile("bzip2-zip.zip"));
     }
 
-    private void multiByteReadConsistentlyReturnsMinusOneAtEof(File file) throws Exception {
-        byte[] buf = new byte[2];
-        try (FileInputStream in = new FileInputStream(getFile("bla.zip"));
+    private void multiByteReadConsistentlyReturnsMinusOneAtEof(final File file) throws Exception {
+        final byte[] buf = new byte[2];
+        try (InputStream in = Files.newInputStream(getFile("bla.zip").toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(in)) {
-            ArchiveEntry e = archive.getNextEntry();
+            final ArchiveEntry e = archive.getNextEntry();
             IOUtils.toByteArray(archive);
             assertEquals(-1, archive.read(buf));
             assertEquals(-1, archive.read(buf));
@@ -433,7 +420,7 @@ public class ZipArchiveInputStreamTest {
     @Test
     public void singleByteReadThrowsAtEofForCorruptedStoredEntry() throws Exception {
         byte[] content;
-        try (FileInputStream fs = new FileInputStream(getFile("COMPRESS-264.zip"))) {
+        try (InputStream fs = Files.newInputStream(getFile("COMPRESS-264.zip").toPath())) {
             content = IOUtils.toByteArray(fs);
         }
         // make size much bigger than entry's real size
@@ -442,23 +429,23 @@ public class ZipArchiveInputStreamTest {
         }
         try (ByteArrayInputStream in = new ByteArrayInputStream(content);
              ZipArchiveInputStream archive = new ZipArchiveInputStream(in)) {
-            ArchiveEntry e = archive.getNextEntry();
+            final ArchiveEntry e = archive.getNextEntry();
             try {
                 IOUtils.toByteArray(archive);
                 fail("expected exception");
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 assertEquals("Truncated ZIP file", ex.getMessage());
             }
             try {
                 archive.read();
                 fail("expected exception");
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 assertEquals("Truncated ZIP file", ex.getMessage());
             }
             try {
                 archive.read();
                 fail("expected exception");
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 assertEquals("Truncated ZIP file", ex.getMessage());
             }
         }
@@ -467,33 +454,33 @@ public class ZipArchiveInputStreamTest {
     @Test
     public void multiByteReadThrowsAtEofForCorruptedStoredEntry() throws Exception {
         byte[] content;
-        try (FileInputStream fs = new FileInputStream(getFile("COMPRESS-264.zip"))) {
+        try (InputStream fs = Files.newInputStream(getFile("COMPRESS-264.zip").toPath())) {
             content = IOUtils.toByteArray(fs);
         }
         // make size much bigger than entry's real size
         for (int i = 17; i < 26; i++) {
             content[i] = (byte) 0xff;
         }
-        byte[] buf = new byte[2];
+        final byte[] buf = new byte[2];
         try (ByteArrayInputStream in = new ByteArrayInputStream(content);
              ZipArchiveInputStream archive = new ZipArchiveInputStream(in)) {
-            ArchiveEntry e = archive.getNextEntry();
+            final ArchiveEntry e = archive.getNextEntry();
             try {
                 IOUtils.toByteArray(archive);
                 fail("expected exception");
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 assertEquals("Truncated ZIP file", ex.getMessage());
             }
             try {
                 archive.read(buf);
                 fail("expected exception");
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 assertEquals("Truncated ZIP file", ex.getMessage());
             }
             try {
                 archive.read(buf);
                 fail("expected exception");
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 assertEquals("Truncated ZIP file", ex.getMessage());
             }
         }
@@ -501,7 +488,7 @@ public class ZipArchiveInputStreamTest {
 
     @Test
     public void properlyReadsStoredEntries() throws IOException {
-        try (FileInputStream fs = new FileInputStream(getFile("bla-stored.zip"));
+        try (InputStream fs = Files.newInputStream(getFile("bla-stored.zip").toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(fs)) {
             ZipArchiveEntry e = archive.getNextZipEntry();
             assertNotNull(e);
@@ -523,29 +510,29 @@ public class ZipArchiveInputStreamTest {
 
     @Test
     public void rejectsStoredEntriesWithDataDescriptorByDefault() throws IOException {
-        try (FileInputStream fs = new FileInputStream(getFile("bla-stored-dd.zip"));
+        try (InputStream fs = Files.newInputStream(getFile("bla-stored-dd.zip").toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(fs)) {
-            ZipArchiveEntry e = archive.getNextZipEntry();
+            final ZipArchiveEntry e = archive.getNextZipEntry();
             assertNotNull(e);
             assertEquals("test1.xml", e.getName());
             assertEquals(-1, e.getCompressedSize());
             assertEquals(-1, e.getSize());
             thrown.expect(UnsupportedZipFeatureException.class);
             thrown.expectMessage("Unsupported feature data descriptor used in entry test1.xml");
-            byte[] data = IOUtils.toByteArray(archive);
+            final byte[] data = IOUtils.toByteArray(archive);
         }
     }
 
     @Test
     public void properlyReadsStoredEntryWithDataDescriptorWithSignature() throws IOException {
-        try (FileInputStream fs = new FileInputStream(getFile("bla-stored-dd.zip"));
+        try (InputStream fs = Files.newInputStream(getFile("bla-stored-dd.zip").toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(fs, "UTF-8", true, true)) {
-            ZipArchiveEntry e = archive.getNextZipEntry();
+            final ZipArchiveEntry e = archive.getNextZipEntry();
             assertNotNull(e);
             assertEquals("test1.xml", e.getName());
             assertEquals(-1, e.getCompressedSize());
             assertEquals(-1, e.getSize());
-            byte[] data = IOUtils.toByteArray(archive);
+            final byte[] data = IOUtils.toByteArray(archive);
             assertEquals(610, data.length);
             assertEquals(610, e.getCompressedSize());
             assertEquals(610, e.getSize());
@@ -554,14 +541,14 @@ public class ZipArchiveInputStreamTest {
 
     @Test
     public void properlyReadsStoredEntryWithDataDescriptorWithoutSignature() throws IOException {
-        try (FileInputStream fs = new FileInputStream(getFile("bla-stored-dd-nosig.zip"));
+        try (InputStream fs = Files.newInputStream(getFile("bla-stored-dd-nosig.zip").toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(fs, "UTF-8", true, true)) {
-            ZipArchiveEntry e = archive.getNextZipEntry();
+            final ZipArchiveEntry e = archive.getNextZipEntry();
             assertNotNull(e);
             assertEquals("test1.xml", e.getName());
             assertEquals(-1, e.getCompressedSize());
             assertEquals(-1, e.getSize());
-            byte[] data = IOUtils.toByteArray(archive);
+            final byte[] data = IOUtils.toByteArray(archive);
             assertEquals(610, data.length);
             assertEquals(610, e.getCompressedSize());
             assertEquals(610, e.getSize());
@@ -570,43 +557,43 @@ public class ZipArchiveInputStreamTest {
 
     @Test
     public void throwsIfStoredDDIsInconsistent() throws IOException {
-        try (FileInputStream fs = new FileInputStream(getFile("bla-stored-dd-sizes-differ.zip"));
+        try (InputStream fs = Files.newInputStream(getFile("bla-stored-dd-sizes-differ.zip").toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(fs, "UTF-8", true, true)) {
-            ZipArchiveEntry e = archive.getNextZipEntry();
+            final ZipArchiveEntry e = archive.getNextZipEntry();
             assertNotNull(e);
             assertEquals("test1.xml", e.getName());
             assertEquals(-1, e.getCompressedSize());
             assertEquals(-1, e.getSize());
             thrown.expect(ZipException.class);
             thrown.expectMessage("compressed and uncompressed size don't match");
-            byte[] data = IOUtils.toByteArray(archive);
+            final byte[] data = IOUtils.toByteArray(archive);
         }
     }
 
     @Test
     public void throwsIfStoredDDIsDifferentFromLengthRead() throws IOException {
-        try (FileInputStream fs = new FileInputStream(getFile("bla-stored-dd-contradicts-actualsize.zip"));
+        try (InputStream fs = Files.newInputStream(getFile("bla-stored-dd-contradicts-actualsize.zip").toPath());
              ZipArchiveInputStream archive = new ZipArchiveInputStream(fs, "UTF-8", true, true)) {
-            ZipArchiveEntry e = archive.getNextZipEntry();
+            final ZipArchiveEntry e = archive.getNextZipEntry();
             assertNotNull(e);
             assertEquals("test1.xml", e.getName());
             assertEquals(-1, e.getCompressedSize());
             assertEquals(-1, e.getSize());
             thrown.expect(ZipException.class);
             thrown.expectMessage("actual and claimed size don't match");
-            byte[] data = IOUtils.toByteArray(archive);
+            final byte[] data = IOUtils.toByteArray(archive);
         }
     }
 
     @Test
     public void testSplitZipCreatedByZip() throws IOException {
-        File lastFile = getFile("COMPRESS-477/split_zip_created_by_zip/split_zip_created_by_zip.zip");
+        final File lastFile = getFile("COMPRESS-477/split_zip_created_by_zip/split_zip_created_by_zip.zip");
         try (SeekableByteChannel channel = ZipSplitReadOnlySeekableByteChannel.buildFromLastSplitSegment(lastFile);
              InputStream inputStream = Channels.newInputStream(channel);
              ZipArchiveInputStream splitInputStream = new ZipArchiveInputStream(inputStream, ZipEncodingHelper.UTF8, true, false, true)) {
 
-            File fileToCompare = getFile("COMPRESS-477/split_zip_created_by_zip/zip_to_compare_created_by_zip.zip");
-            try (ZipArchiveInputStream inputStreamToCompare = new ZipArchiveInputStream(new FileInputStream(fileToCompare), ZipEncodingHelper.UTF8, true, false, true)) {
+            final File fileToCompare = getFile("COMPRESS-477/split_zip_created_by_zip/zip_to_compare_created_by_zip.zip");
+            try (ZipArchiveInputStream inputStreamToCompare = new ZipArchiveInputStream(Files.newInputStream(fileToCompare.toPath()), ZipEncodingHelper.UTF8, true, false, true)) {
 
                 ArchiveEntry entry;
                 while((entry = splitInputStream.getNextEntry()) != null && inputStreamToCompare.getNextEntry() != null) {
@@ -622,13 +609,13 @@ public class ZipArchiveInputStreamTest {
 
     @Test
     public void testSplitZipCreatedByZipOfZip64() throws IOException {
-        File lastFile = getFile("COMPRESS-477/split_zip_created_by_zip/split_zip_created_by_zip_zip64.zip");
+        final File lastFile = getFile("COMPRESS-477/split_zip_created_by_zip/split_zip_created_by_zip_zip64.zip");
         try (SeekableByteChannel channel = ZipSplitReadOnlySeekableByteChannel.buildFromLastSplitSegment(lastFile);
              InputStream inputStream = Channels.newInputStream(channel);
              ZipArchiveInputStream splitInputStream = new ZipArchiveInputStream(inputStream, ZipEncodingHelper.UTF8, true, false, true)) {
 
-            File fileToCompare = getFile("COMPRESS-477/split_zip_created_by_zip/zip_to_compare_created_by_zip_zip64.zip");
-            try (ZipArchiveInputStream inputStreamToCompare = new ZipArchiveInputStream(new FileInputStream(fileToCompare), ZipEncodingHelper.UTF8, true, false, true)) {
+            final File fileToCompare = getFile("COMPRESS-477/split_zip_created_by_zip/zip_to_compare_created_by_zip_zip64.zip");
+            try (ZipArchiveInputStream inputStreamToCompare = new ZipArchiveInputStream(Files.newInputStream(fileToCompare.toPath()), ZipEncodingHelper.UTF8, true, false, true)) {
 
                 ArchiveEntry entry;
                 while((entry = splitInputStream.getNextEntry()) != null && inputStreamToCompare.getNextEntry() != null) {
@@ -644,13 +631,13 @@ public class ZipArchiveInputStreamTest {
 
     @Test
     public void testSplitZipCreatedByWinrar() throws IOException {
-        File lastFile = getFile("COMPRESS-477/split_zip_created_by_winrar/split_zip_created_by_winrar.zip");
+        final File lastFile = getFile("COMPRESS-477/split_zip_created_by_winrar/split_zip_created_by_winrar.zip");
         try (SeekableByteChannel channel = ZipSplitReadOnlySeekableByteChannel.buildFromLastSplitSegment(lastFile);
              InputStream inputStream = Channels.newInputStream(channel);
              ZipArchiveInputStream splitInputStream = new ZipArchiveInputStream(inputStream, ZipEncodingHelper.UTF8, true, false, true)) {
 
-            File fileToCompare = getFile("COMPRESS-477/split_zip_created_by_winrar/zip_to_compare_created_by_winrar.zip");
-            try (ZipArchiveInputStream inputStreamToCompare = new ZipArchiveInputStream(new FileInputStream(fileToCompare), ZipEncodingHelper.UTF8, true, false, true)) {
+            final File fileToCompare = getFile("COMPRESS-477/split_zip_created_by_winrar/zip_to_compare_created_by_winrar.zip");
+            try (ZipArchiveInputStream inputStreamToCompare = new ZipArchiveInputStream(Files.newInputStream(fileToCompare.toPath()), ZipEncodingHelper.UTF8, true, false, true)) {
 
                 ArchiveEntry entry;
                 while((entry = splitInputStream.getNextEntry()) != null && inputStreamToCompare.getNextEntry() != null) {
@@ -667,9 +654,9 @@ public class ZipArchiveInputStreamTest {
     @Test
     public void testSplitZipCreatedByZipThrowsException() throws IOException {
         thrown.expect(EOFException.class);
-        File zipSplitFile = getFile("COMPRESS-477/split_zip_created_by_zip/split_zip_created_by_zip.z01");
-        InputStream fileInputStream = new FileInputStream(zipSplitFile);
-        ZipArchiveInputStream inputStream = new ZipArchiveInputStream(fileInputStream, ZipEncodingHelper.UTF8, true, false, true);
+        final File zipSplitFile = getFile("COMPRESS-477/split_zip_created_by_zip/split_zip_created_by_zip.z01");
+        final InputStream fileInputStream = Files.newInputStream(zipSplitFile.toPath());
+        final ZipArchiveInputStream inputStream = new ZipArchiveInputStream(fileInputStream, ZipEncodingHelper.UTF8, true, false, true);
 
         ArchiveEntry entry = inputStream.getNextEntry();
         while(entry != null){
@@ -677,7 +664,65 @@ public class ZipArchiveInputStreamTest {
         }
     }
 
-    private static byte[] readEntry(ZipArchiveInputStream zip, ZipArchiveEntry zae) throws IOException {
+    /**
+     * @see <a href="https://issues.apache.org/jira/browse/COMPRESS-518">COMPRESS-518</a>
+     */
+    @Test
+    public void throwsIfZip64ExtraCouldNotBeUnderstood() throws Exception {
+        thrown.expect(ZipException.class);
+        fuzzingTest(new int[] {
+            0x50, 0x4b, 0x03, 0x04, 0x2e, 0x00, 0x00, 0x00, 0x0c, 0x00,
+            0x84, 0xb6, 0xba, 0x46, 0x72, 0xb6, 0xfe, 0x77, 0x63, 0x00,
+            0x00, 0x00, 0x6b, 0x00, 0x00, 0x00, 0x03, 0x00, 0x1c, 0x00,
+            0x62, 0x62, 0x62, 0x01, 0x00, 0x09, 0x00, 0x03, 0xe7, 0xce,
+            0x64, 0x55, 0xf3, 0xce, 0x64, 0x55, 0x75, 0x78, 0x0b, 0x00,
+            0x01, 0x04, 0x5c, 0xf9, 0x01, 0x00, 0x04, 0x88, 0x13, 0x00,
+            0x00
+        });
+    }
+
+    /**
+     * @see <a href="https://issues.apache.org/jira/browse/COMPRESS-523">COMPRESS-523</a>
+     */
+    @Test
+    public void throwsIfThereIsNoEocd() throws Exception {
+        thrown.expect(IOException.class);
+        thrown.expectMessage("Truncated ZIP file");
+        fuzzingTest(new int[] {
+            0x50, 0x4b, 0x01, 0x02, 0x14, 0x00, 0x14, 0x00, 0x08, 0x00,
+            0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x43, 0xbe, 0x00, 0x00,
+            0x00, 0xb7, 0xe8, 0x07, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00
+        });
+    }
+
+    @Test(expected = IOException.class)
+    public void throwsIOExceptionIfThereIsCorruptedZip64Extra() throws IOException {
+        try (InputStream fis = Files.newInputStream(getFile("COMPRESS-546.zip").toPath());
+             ZipArchiveInputStream zipInputStream = new ZipArchiveInputStream(fis)) {
+            while (zipInputStream.getNextZipEntry() != null) {
+            }
+        }
+    }
+
+    @Test
+    public void testZipWithBadExtraFields() throws IOException {
+        try (InputStream fis = Files.newInputStream(getFile("COMPRESS-548.zip").toPath());
+             ZipArchiveInputStream zipInputStream = new ZipArchiveInputStream(fis)) {
+            while (zipInputStream.getNextZipEntry() != null) {
+            }
+        }
+    }
+
+    @Test
+    public void testZipUsingStoredWithDDAndNoDDSignature() throws IOException {
+        try (InputStream inputStream = forgeZipInputStream();
+             ZipArchiveInputStream zipInputStream = new ZipArchiveInputStream(inputStream, "UTF-8", true, true)) {
+            while (zipInputStream.getNextZipEntry() != null) {
+            }
+        }
+    }
+
+    private static byte[] readEntry(final ZipArchiveInputStream zip, final ZipArchiveEntry zae) throws IOException {
         final int len = (int)zae.getSize();
         final byte[] buff = new byte[len];
         zip.read(buff, 0, len);
@@ -685,13 +730,13 @@ public class ZipArchiveInputStreamTest {
         return buff;
     }
 
-    private static void nameSource(String archive, String entry, ZipArchiveEntry.NameSource expected) throws Exception {
+    private static void nameSource(final String archive, final String entry, final ZipArchiveEntry.NameSource expected) throws Exception {
         nameSource(archive, entry, 1, expected);
     }
 
-    private static void nameSource(String archive, String entry, int entryNo, ZipArchiveEntry.NameSource expected)
+    private static void nameSource(final String archive, final String entry, int entryNo, final ZipArchiveEntry.NameSource expected)
         throws Exception {
-        try (ZipArchiveInputStream zis = new ZipArchiveInputStream(new FileInputStream(getFile(archive)))) {
+        try (ZipArchiveInputStream zis = new ZipArchiveInputStream(Files.newInputStream(getFile(archive).toPath()))) {
             ZipArchiveEntry ze;
             do {
                 ze = zis.getNextZipEntry();
@@ -700,4 +745,56 @@ public class ZipArchiveInputStreamTest {
             assertEquals(expected, ze.getNameSource());
         }
     }
+
+    private void fuzzingTest(final int[] bytes) throws Exception {
+        final int len = bytes.length;
+        final byte[] input = new byte[len];
+        for (int i = 0; i < len; i++) {
+            input[i] = (byte) bytes[i];
+        }
+        try (ArchiveInputStream ais = ArchiveStreamFactory.DEFAULT
+             .createArchiveInputStream("zip", new ByteArrayInputStream(input))) {
+            ais.getNextEntry();
+            IOUtils.toByteArray(ais);
+        }
+    }
+
+    /**
+     * Forge a zip archive in memory, using STORED and
+     * Data Descriptor, and without signature of Data
+     * Descriptor.
+     *
+     * @return the input stream of the generated zip
+     * @throws IOException there are problems
+     */
+    private InputStream forgeZipInputStream() throws IOException {
+        try (final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
+             final ZipArchiveOutputStream zo = new ZipArchiveOutputStream(byteArrayOutputStream)){
+
+            final ZipArchiveEntry entryA = new ZipArchiveEntry("foo");
+            entryA.setMethod(ZipEntry.STORED);
+            entryA.setSize(4);
+            entryA.setCrc(0xb63cfbcdL);
+            zo.putArchiveEntry(entryA);
+            zo.write(new byte[] { 1, 2, 3, 4 });
+            zo.closeArchiveEntry();
+            zo.close();
+
+            final byte[] zipContent = byteArrayOutputStream.toByteArray();
+            final byte[] old = Arrays.copyOf(zipContent, zipContent.length);
+
+            final byte[] zipContentWithDataDescriptor = new byte[zipContent.length + 12];
+            System.arraycopy(zipContent, 0, zipContentWithDataDescriptor, 0, 37);
+            // modify the general purpose bit flag
+            zipContentWithDataDescriptor[6] = 8;
+
+            // copy the crc-32, compressed size and uncompressed size to the data descriptor
+            System.arraycopy(zipContent, 14, zipContentWithDataDescriptor, 37, 12);
+
+            // and copy the rest of the zip content
+            System.arraycopy(zipContent, 37, zipContentWithDataDescriptor, 49, zipContent.length - 37);
+
+            return new ByteArrayInputStream(zipContentWithDataDescriptor);
+        }
+    }
 }
