@@ -28,15 +28,15 @@ import static org.junit.Assert.fail;
 public class HuffmanDecoderTest {
     @Test
     public void decodeUncompressedBlock() throws Exception {
-        byte[] data = {
+        final byte[] data = {
                 0b1, // end of block + no compression mode
                 11, 0, -12, -1, // len & ~len
                 'H', 'e', 'l', 'l', 'o', ' ', 'W', 'o', 'r', 'l', 'd'
         };
 
-        HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
-        byte[] result = new byte[100];
-        int len = decoder.decode(result);
+        final HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
+        final byte[] result = new byte[100];
+        final int len = decoder.decode(result);
 
         assertEquals(11, len);
         assertEquals("Hello World", new String(result, 0, len));
@@ -44,25 +44,25 @@ public class HuffmanDecoderTest {
 
     @Test
     public void decodeUncompressedBlockWithInvalidLenNLenValue() throws Exception {
-        byte[] data = {
+        final byte[] data = {
                 0b1, // end of block + no compression mode
                 11, 0, -12, -2, // len & ~len
                 'H', 'e', 'l', 'l', 'o', ' ', 'W', 'o', 'r', 'l', 'd'
         };
 
-        HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
-        byte[] result = new byte[100];
+        final HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
+        final byte[] result = new byte[100];
         try {
-            int len = decoder.decode(result);
+            final int len = decoder.decode(result);
             fail("Should have failed but returned " + len + " entries: " + Arrays.toString(Arrays.copyOf(result, len)));
-        } catch (IllegalStateException e) {
+        } catch (final IllegalStateException e) {
             assertEquals("Illegal LEN / NLEN values", e.getMessage());
         }
     }
 
     @Test
     public void decodeSimpleFixedHuffmanBlock() throws Exception {
-        byte[] data = {
+        final byte[] data = {
                 //|--- binary filling ---|76543210
                 0b11111111111111111111111111110011, // final block + fixed huffman + H
                 0b00000000000000000000000001001000, // H + e
@@ -79,9 +79,9 @@ public class HuffmanDecoderTest {
                 0b11111111111111111111111111111100 // end of block (00) + garbage
         };
 
-        HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
-        byte[] result = new byte[100];
-        int len = decoder.decode(result);
+        final HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
+        final byte[] result = new byte[100];
+        final int len = decoder.decode(result);
 
         assertEquals(11, len);
         assertEquals("Hello World", new String(result, 0, len));
@@ -89,7 +89,7 @@ public class HuffmanDecoderTest {
 
     @Test
     public void decodeSimpleFixedHuffmanBlockToSmallBuffer() throws Exception {
-        byte[] data = {
+        final byte[] data = {
                 //|--- binary filling ---|76543210
                 0b11111111111111111111111111110011, // final block + fixed huffman + H
                 0b00000000000000000000000001001000, // H + e
@@ -106,8 +106,8 @@ public class HuffmanDecoderTest {
                 0b11111111111111111111111111111100 // end of block (00) + garbage
         };
 
-        HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
-        byte[] result = new byte[10];
+        final HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
+        final byte[] result = new byte[10];
         int len;
         len = decoder.decode(result);
         assertEquals(10, len);
@@ -120,7 +120,7 @@ public class HuffmanDecoderTest {
 
     @Test
     public void decodeFixedHuffmanBlockWithMemoryLookup() throws Exception {
-        byte[] data = {
+        final byte[] data = {
                 //|--- binary filling ---|76543210
                 0b11111111111111111111111111110011, // final block + fixed huffman + H
                 0b00000000000000000000000001001000, // H + e
@@ -140,9 +140,9 @@ public class HuffmanDecoderTest {
                 0b11111111111111111111111111111000 // end of block (0000) + garbage
         };
 
-        HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
-        byte[] result = new byte[100];
-        int len = decoder.decode(result);
+        final HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
+        final byte[] result = new byte[100];
+        final int len = decoder.decode(result);
 
         assertEquals(48, len);
         assertEquals("Hello World\nHello World\nHello World\nHello World\n", new String(result, 0, len));
@@ -150,7 +150,7 @@ public class HuffmanDecoderTest {
 
     @Test
     public void decodeFixedHuffmanBlockWithMemoryLookupInSmallBuffer() throws Exception {
-        byte[] data = {
+        final byte[] data = {
                 //|--- binary filling ---|76543210
                 0b11111111111111111111111111110011, // final block + fixed huffman + H
                 0b00000000000000000000000001001000, // H + e
@@ -170,8 +170,8 @@ public class HuffmanDecoderTest {
                 0b11111111111111111111111111111000 // end of block (0000) + garbage
         };
 
-        HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
-        byte[] result = new byte[30];
+        final HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
+        final byte[] result = new byte[30];
         int len;
 
         len = decoder.decode(result);
@@ -185,7 +185,7 @@ public class HuffmanDecoderTest {
 
     @Test
     public void decodeFixedHuffmanBlockWithMemoryLookupInExactBuffer() throws Exception {
-        byte[] data = {
+        final byte[] data = {
                 //|--- binary filling ---|76543210
                 0b11111111111111111111111111110011, // final block + fixed huffman + H
                 0b00000000000000000000000001001000, // H + e
@@ -205,8 +205,8 @@ public class HuffmanDecoderTest {
                 0b11111111111111111111111111111000 // end of block (0000) + garbage
         };
 
-        HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
-        byte[] result = new byte[48];
+        final HuffmanDecoder decoder = new HuffmanDecoder(new ByteArrayInputStream(data));
+        final byte[] result = new byte[48];
         int len;
 
         len = decoder.decode(result);
